@@ -6,18 +6,25 @@
 #ifndef mozilla_image_Image_h
 #define mozilla_image_Image_h
 
+#include "mozilla/Attributes.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/MemoryReporting.h"
-#include "mozilla/Tuple.h"
+#include "mozilla/ProfilerMarkers.h"
+#include "mozilla/SizeOfState.h"
+#include "mozilla/ThreadSafeWeakPtr.h"
 #include "mozilla/TimeStamp.h"
+#include "mozilla/Tuple.h"
 #include "gfx2DGlue.h"
 #include "imgIContainer.h"
-#include "ImageURL.h"
 #include "ImageContainer.h"
+#include "ImageRegion.h"
 #include "LookupResult.h"
 #include "nsStringFwd.h"
 #include "ProgressTracker.h"
 #include "SurfaceCache.h"
+#include "WebRenderImageProvider.h"
 
+class imgRequest;
 class nsIRequest;
 class nsIInputStream;
 
@@ -30,14 +37,16 @@ class Image;
 // Memory Reporting
 ///////////////////////////////////////////////////////////////////////////////
 
-struct MemoryCounter
-{
+struct MemoryCounter {
   MemoryCounter()
-    : mSource(0)
-    , mDecodedHeap(0)
-    , mDecodedNonHeap(0)
-    , mExternalHandles(0)
-  { }
+      : mSource(0),
+        mDecodedHeap(0),
+        mDecodedNonHeap(0),
+        mDecodedUnknown(0),
+        mExternalHandles(0),
+        mFrameIndex(0),
+        mExternalId(0),
+        mSurfaceTypes(0) {}
 
   void SetSource(size_t aCount) { mSource = aCount; }
   size_t Source() const { return mSource; }
@@ -45,46 +54,51 @@ struct MemoryCounter
   size_t DecodedHeap() const { return mDecodedHeap; }
   void SetDecodedNonHeap(size_t aCount) { mDecodedNonHeap = aCount; }
   size_t DecodedNonHeap() const { return mDecodedNonHeap; }
+  void SetDecodedUnknown(size_t aCount) { mDecodedUnknown = aCount; }
+  size_t DecodedUnknown() const { return mDecodedUnknown; }
   void SetExternalHandles(size_t aCount) { mExternalHandles = aCount; }
   size_t ExternalHandles() const { return mExternalHandles; }
+  void SetFrameIndex(size_t aIndex) { mFrameIndex = aIndex; }
+  size_t FrameIndex() const { return mFrameIndex; }
+  void SetExternalId(uint64_t aId) { mExternalId = aId; }
+  uint64_t ExternalId() const { return mExternalId; }
+  void SetSurfaceTypes(uint32_t aTypes) { mSurfaceTypes = aTypes; }
+  uint32_t SurfaceTypes() const { return mSurfaceTypes; }
 
-  MemoryCounter& operator+=(const MemoryCounter& aOther)
-  {
+  MemoryCounter& operator+=(const MemoryCounter& aOther) {
     mSource += aOther.mSource;
     mDecodedHeap += aOther.mDecodedHeap;
     mDecodedNonHeap += aOther.mDecodedNonHeap;
+    mDecodedUnknown += aOther.mDecodedUnknown;
     mExternalHandles += aOther.mExternalHandles;
+    mSurfaceTypes |= aOther.mSurfaceTypes;
     return *this;
   }
 
-private:
+ private:
   size_t mSource;
   size_t mDecodedHeap;
   size_t mDecodedNonHeap;
+  size_t mDecodedUnknown;
   size_t mExternalHandles;
+  size_t mFrameIndex;
+  uint64_t mExternalId;
+  uint32_t mSurfaceTypes;
 };
 
-enum class SurfaceMemoryCounterType
-{
-  NORMAL,
-  COMPOSITING,
-  COMPOSITING_PREV
-};
+enum class SurfaceMemoryCounterType { NORMAL, CONTAINER };
 
-struct SurfaceMemoryCounter
-{
-  SurfaceMemoryCounter(const SurfaceKey& aKey,
-                       bool aIsLocked,
-                       bool aCannotSubstitute,
-                       bool aIsFactor2,
-                       SurfaceMemoryCounterType aType =
-                         SurfaceMemoryCounterType::NORMAL)
-    : mKey(aKey)
-    , mType(aType)
-    , mIsLocked(aIsLocked)
-    , mCannotSubstitute(aCannotSubstitute)
-    , mIsFactor2(aIsFactor2)
-  { }
+struct SurfaceMemoryCounter {
+  SurfaceMemoryCounter(
+      const SurfaceKey& aKey, bool aIsLocked, bool aCannotSubstitute,
+      bool aIsFactor2, bool aFinished,
+      SurfaceMemoryCounterType aType = SurfaceMemoryCounterType::NORMAL)
+      : mKey(aKey),
+        mType(aType),
+        mIsLocked(aIsLocked),
+        mCannotSubstitute(aCannotSubstitute),
+        mIsFactor2(aIsFactor2),
+        mFinished(aFinished) {}
 
   const SurfaceKey& Key() const { return mKey; }
   MemoryCounter& Values() { return mValues; }
@@ -93,53 +107,78 @@ struct SurfaceMemoryCounter
   bool IsLocked() const { return mIsLocked; }
   bool CannotSubstitute() const { return mCannotSubstitute; }
   bool IsFactor2() const { return mIsFactor2; }
+  bool IsFinished() const { return mFinished; }
 
-private:
+ private:
   const SurfaceKey mKey;
   MemoryCounter mValues;
   const SurfaceMemoryCounterType mType;
   const bool mIsLocked;
   const bool mCannotSubstitute;
   const bool mIsFactor2;
+  const bool mFinished;
 };
 
-struct ImageMemoryCounter
-{
-  ImageMemoryCounter(Image* aImage, SizeOfState& aState, bool aIsUsed);
+struct ImageMemoryCounter {
+  ImageMemoryCounter(imgRequest* aRequest, SizeOfState& aState, bool aIsUsed);
+  ImageMemoryCounter(imgRequest* aRequest, Image* aImage, SizeOfState& aState,
+                     bool aIsUsed);
 
   nsCString& URI() { return mURI; }
   const nsCString& URI() const { return mURI; }
   const nsTArray<SurfaceMemoryCounter>& Surfaces() const { return mSurfaces; }
   const gfx::IntSize IntrinsicSize() const { return mIntrinsicSize; }
   const MemoryCounter& Values() const { return mValues; }
+  uint32_t Progress() const { return mProgress; }
   uint16_t Type() const { return mType; }
   bool IsUsed() const { return mIsUsed; }
+  bool HasError() const { return mHasError; }
+  bool IsValidating() const { return mValidating; }
 
-  bool IsNotable() const
-  {
+  bool IsNotable() const {
+    // Errors or requests without images are always notable.
+    if (mHasError || mValidating || mProgress == UINT32_MAX ||
+        mProgress & FLAG_HAS_ERROR || mType == imgIContainer::TYPE_REQUEST) {
+      return true;
+    }
+
+    // Sufficiently large images are notable.
     const size_t NotableThreshold = 16 * 1024;
-    size_t total = mValues.Source() + mValues.DecodedHeap()
-                                    + mValues.DecodedNonHeap();
-    return total >= NotableThreshold;
+    size_t total = mValues.Source() + mValues.DecodedHeap() +
+                   mValues.DecodedNonHeap() + mValues.DecodedUnknown();
+    if (total >= NotableThreshold) {
+      return true;
+    }
+
+    // Incomplete images are always notable as well; the odds of capturing
+    // mid-decode should be fairly low.
+    for (const auto& surface : mSurfaces) {
+      if (!surface.IsFinished()) {
+        return true;
+      }
+    }
+
+    return false;
   }
 
-private:
+ private:
   nsCString mURI;
   nsTArray<SurfaceMemoryCounter> mSurfaces;
   gfx::IntSize mIntrinsicSize;
   MemoryCounter mValues;
+  uint32_t mProgress;
   uint16_t mType;
   const bool mIsUsed;
+  bool mHasError;
+  bool mValidating;
 };
 
-
 ///////////////////////////////////////////////////////////////////////////////
 // Image Base Types
 ///////////////////////////////////////////////////////////////////////////////
 
-class Image : public imgIContainer
-{
-public:
+class Image : public imgIContainer {
+ public:
   /**
    * Flags for Image initialization.
    *
@@ -161,11 +200,11 @@ public:
    * INIT_FLAG_SYNC_LOAD: The container is being loaded synchronously, so
    * it should avoid relying on async workers to get the container ready.
    */
-  static const uint32_t INIT_FLAG_NONE                     = 0x0;
-  static const uint32_t INIT_FLAG_DISCARDABLE              = 0x1;
-  static const uint32_t INIT_FLAG_DECODE_IMMEDIATELY       = 0x2;
-  static const uint32_t INIT_FLAG_TRANSIENT                = 0x4;
-  static const uint32_t INIT_FLAG_SYNC_LOAD                = 0x8;
+  static const uint32_t INIT_FLAG_NONE = 0x0;
+  static const uint32_t INIT_FLAG_DISCARDABLE = 0x1;
+  static const uint32_t INIT_FLAG_DECODE_IMMEDIATELY = 0x2;
+  static const uint32_t INIT_FLAG_TRANSIENT = 0x4;
+  static const uint32_t INIT_FLAG_SYNC_LOAD = 0x8;
 
   virtual already_AddRefed<ProgressTracker> GetProgressTracker() = 0;
   virtual void SetProgressTracker(ProgressTracker* aProgressTracker) {}
@@ -175,8 +214,8 @@ public:
    * If MallocSizeOf does not work on this platform, uses a fallback approach to
    * ensure that something reasonable is always returned.
    */
-  virtual size_t
-    SizeOfSourceWithComputedFallback(SizeOfState& aState) const = 0;
+  virtual size_t SizeOfSourceWithComputedFallback(
+      SizeOfState& aState) const = 0;
 
   /**
    * Collect an accounting of the memory occupied by the image's surfaces (which
@@ -204,7 +243,6 @@ public:
    * until OnImageDataAvailable is called.
    */
   virtual nsresult OnImageDataAvailable(nsIRequest* aRequest,
-                                        nsISupports* aContext,
                                         nsIInputStream* aInStr,
                                         uint64_t aSourceOffset,
                                         uint32_t aCount) = 0;
@@ -213,13 +251,10 @@ public:
    * Called from OnStopRequest when the image's underlying request completes.
    *
    * @param aRequest  The completed request.
-   * @param aContext  Context from Necko's OnStopRequest.
    * @param aStatus   A success or failure code.
    * @param aLastPart Whether this is the final part of the underlying request.
    */
-  virtual nsresult OnImageDataComplete(nsIRequest* aRequest,
-                                       nsISupports* aContext,
-                                       nsresult aStatus,
+  virtual nsresult OnImageDataComplete(nsIRequest* aRequest, nsresult aStatus,
                                        bool aLastPart) = 0;
 
   /**
@@ -233,23 +268,27 @@ public:
   virtual bool HasError() = 0;
   virtual void SetHasError() = 0;
 
-  virtual ImageURL* GetURI() = 0;
+  virtual nsIURI* GetURI() const = 0;
 
-  virtual void ReportUseCounters() { }
+  NS_IMETHOD GetHotspotX(int32_t* aX) override {
+    *aX = 0;
+    return NS_OK;
+  }
+  NS_IMETHOD GetHotspotY(int32_t* aY) override {
+    *aY = 0;
+    return NS_OK;
+  }
 };
 
-class ImageResource : public Image
-{
-public:
-  already_AddRefed<ProgressTracker> GetProgressTracker() override
-  {
+class ImageResource : public Image {
+ public:
+  already_AddRefed<ProgressTracker> GetProgressTracker() override {
     RefPtr<ProgressTracker> progressTracker = mProgressTracker;
     MOZ_ASSERT(progressTracker);
     return progressTracker.forget();
   }
 
-  void SetProgressTracker(ProgressTracker* aProgressTracker) final
-  {
+  void SetProgressTracker(ProgressTracker* aProgressTracker) final {
     MOZ_ASSERT(aProgressTracker);
     MOZ_ASSERT(!mProgressTracker);
     mProgressTracker = aProgressTracker;
@@ -258,33 +297,43 @@ public:
   virtual void IncrementAnimationConsumers() override;
   virtual void DecrementAnimationConsumers() override;
 #ifdef DEBUG
-  virtual uint32_t GetAnimationConsumers() override
-  {
+  virtual uint32_t GetAnimationConsumers() override {
     return mAnimationConsumers;
   }
 #endif
 
-  virtual void OnSurfaceDiscarded(const SurfaceKey& aSurfaceKey) override { }
+  virtual void OnSurfaceDiscarded(const SurfaceKey& aSurfaceKey) override {}
 
-  virtual void SetInnerWindowID(uint64_t aInnerWindowId) override
-  {
+  virtual void SetInnerWindowID(uint64_t aInnerWindowId) override {
     mInnerWindowId = aInnerWindowId;
   }
   virtual uint64_t InnerWindowID() const override { return mInnerWindowId; }
 
-  virtual bool HasError() override    { return mError; }
+  virtual bool HasError() override { return mError; }
   virtual void SetHasError() override { mError = true; }
 
   /*
    * Returns a non-AddRefed pointer to the URI associated with this image.
    * Illegal to use off-main-thread.
    */
-  virtual ImageURL* GetURI() override { return mURI.get(); }
+  nsIURI* GetURI() const override { return mURI; }
 
-protected:
-  explicit ImageResource(ImageURL* aURI);
+  /*
+   * Should be called by its subclasses after they populate @aCounters so that
+   * we can cross reference against any of our ImageContainers that contain
+   * surfaces not in the cache.
+   */
+  void CollectSizeOfSurfaces(nsTArray<SurfaceMemoryCounter>& aCounters,
+                             MallocSizeOf aMallocSizeOf) const override;
+
+  ImageProviderId GetImageProviderId() const { return mProviderId; }
+
+ protected:
+  explicit ImageResource(nsIURI* aURI);
   ~ImageResource();
 
+  bool GetSpecTruncatedTo1k(nsCString& aSpec) const;
+
   // Shared functionality for implementors of imgIContainer. Every
   // implementation of attribute animationMode should forward here.
   nsresult GetAnimationModeInternal(uint16_t* aAnimationMode);
@@ -326,85 +375,48 @@ protected:
 #endif
 
   // Member data shared by all implementations of this abstract class
-  RefPtr<ProgressTracker>     mProgressTracker;
-  RefPtr<ImageURL>            mURI;
-  TimeStamp                     mLastRefreshTime;
-  uint64_t                      mInnerWindowId;
-  uint32_t                      mAnimationConsumers;
-  uint16_t                      mAnimationMode; // Enum values in imgIContainer
-  bool                          mInitialized:1; // Have we been initalized?
-  bool                          mAnimating:1;   // Are we currently animating?
-  bool                          mError:1;       // Error handling
+  RefPtr<ProgressTracker> mProgressTracker;
+  nsCOMPtr<nsIURI> mURI;
+  TimeStamp mLastRefreshTime;
+  uint64_t mInnerWindowId;
+  uint32_t mAnimationConsumers;
+  uint16_t mAnimationMode;  // Enum values in imgIContainer
+  bool mInitialized : 1;    // Have we been initialized?
+  bool mAnimating : 1;      // Are we currently animating?
+  bool mError : 1;          // Error handling
 
-  virtual Tuple<ImgDrawResult, gfx::IntSize, RefPtr<gfx::SourceSurface>>
-    GetFrameInternal(const gfx::IntSize& aSize,
-                     const Maybe<SVGImageContext>& aSVGContext,
-                     uint32_t aWhichFrame,
-                     uint32_t aFlags)
-  {
-    return MakeTuple(ImgDrawResult::BAD_IMAGE, aSize,
-                     RefPtr<gfx::SourceSurface>());
-  }
+  class MOZ_RAII AutoProfilerImagePaintMarker {
+   public:
+    explicit AutoProfilerImagePaintMarker(ImageResource* self)
+        : mStartTime(TimeStamp::Now()) {
+      nsAutoCString spec;
+      if (self->mURI && profiler_thread_is_being_profiled_for_markers()) {
+        static const size_t sMaxTruncatedLength = 1024;
+        self->mURI->GetSpec(mSpec);
+        if (mSpec.Length() >= sMaxTruncatedLength) {
+          mSpec.Truncate(sMaxTruncatedLength);
+        }
+      }
+    }
 
-  /**
-   * Calculate the estimated size to use for an image container with the given
-   * parameters. It may not be the same as the given size, and it may not be
-   * the same as the size of the surface in the image container, but it is the
-   * best effort estimate.
-   */
-  virtual gfx::IntSize GetImageContainerSize(layers::LayerManager* aManager,
-                                             const gfx::IntSize& aSize,
-                                             uint32_t aFlags)
-  {
-    return gfx::IntSize(0, 0);
-  }
+    ~AutoProfilerImagePaintMarker() {
+      if (!mSpec.IsEmpty()) {
+        PROFILER_MARKER_TEXT("Image Paint", GRAPHICS,
+                             MarkerTiming::IntervalUntilNowFrom(mStartTime),
+                             mSpec);
+      }
+    }
 
-  already_AddRefed<layers::ImageContainer>
-    GetImageContainerImpl(layers::LayerManager* aManager,
-                          const gfx::IntSize& aSize,
-                          const Maybe<SVGImageContext>& aSVGContext,
-                          uint32_t aFlags);
-
-  void UpdateImageContainer();
-
-  void ReleaseImageContainer();
-
-private:
-  void SetCurrentImage(layers::ImageContainer* aContainer,
-                       gfx::SourceSurface* aSurface,
-                       bool aInTransaction);
-
-  struct ImageContainerEntry {
-    ImageContainerEntry(const gfx::IntSize& aSize,
-                        const Maybe<SVGImageContext>& aSVGContext,
-                        layers::ImageContainer* aContainer,
-                        uint32_t aFlags)
-      : mSize(aSize)
-      , mSVGContext(aSVGContext)
-      , mContainer(aContainer)
-      , mLastDrawResult(ImgDrawResult::NOT_READY)
-      , mFlags(aFlags)
-    { }
-
-    gfx::IntSize                        mSize;
-    Maybe<SVGImageContext>              mSVGContext;
-    // A weak pointer to our ImageContainer, which stays alive only as long as
-    // the layer system needs it.
-    WeakPtr<layers::ImageContainer>     mContainer;
-    // If mContainer is non-null, this contains the ImgDrawResult we obtained
-    // the last time we updated it.
-    ImgDrawResult                          mLastDrawResult;
-    // Cached flags to use for decoding. FLAG_ASYNC_NOTIFY should always be set
-    // but FLAG_HIGH_QUALITY_SCALING may vary.
-    uint32_t                            mFlags;
+   protected:
+    TimeStamp mStartTime;
+    nsAutoCString mSpec;
   };
 
-  AutoTArray<ImageContainerEntry, 1> mImageContainers;
-  layers::ImageContainer::ProducerID mImageProducerID;
-  layers::ImageContainer::FrameID mLastFrameID;
+ private:
+  ImageProviderId mProviderId;
 };
 
-} // namespace image
-} // namespace mozilla
+}  // namespace image
+}  // namespace mozilla
 
-#endif // mozilla_image_Image_h
+#endif  // mozilla_image_Image_h
