@@ -19,26 +19,25 @@
 #include "nsError.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "mozilla/Mutex.h"
-#include "mozilla/net/ReferrerPolicy.h"
 #include "ImageCacheKey.h"
 
 class imgCacheValidator;
 class imgLoader;
 class imgRequestProxy;
 class imgCacheEntry;
-class nsIApplicationCache;
 class nsIProperties;
 class nsIRequest;
 class nsITimedChannel;
 class nsIURI;
+class nsIReferrerInfo;
 
 namespace mozilla {
+enum CORSMode : uint8_t;
 namespace image {
 class Image;
-class ImageURL;
 class ProgressTracker;
-} // namespace image
-} // namespace mozilla
+}  // namespace image
+}  // namespace mozilla
 
 struct NewPartResult;
 
@@ -46,15 +45,13 @@ class imgRequest final : public nsIStreamListener,
                          public nsIThreadRetargetableStreamListener,
                          public nsIChannelEventSink,
                          public nsIInterfaceRequestor,
-                         public nsIAsyncVerifyRedirectCallback
-{
+                         public nsIAsyncVerifyRedirectCallback {
   typedef mozilla::image::Image Image;
   typedef mozilla::image::ImageCacheKey ImageCacheKey;
-  typedef mozilla::image::ImageURL ImageURL;
   typedef mozilla::image::ProgressTracker ProgressTracker;
-  typedef mozilla::net::ReferrerPolicy ReferrerPolicy;
+  typedef mozilla::dom::ReferrerPolicy ReferrerPolicy;
 
-public:
+ public:
   imgRequest(imgLoader* aLoader, const ImageCacheKey& aCacheKey);
 
   NS_DECL_THREADSAFE_ISUPPORTS
@@ -65,22 +62,23 @@ public:
   NS_DECL_NSIINTERFACEREQUESTOR
   NS_DECL_NSIASYNCVERIFYREDIRECTCALLBACK
 
-  MOZ_MUST_USE nsresult Init(nsIURI* aURI,
-                             nsIURI* aFinalURI,
-                             bool aHadInsecureRedirect,
-                             nsIRequest* aRequest,
-                             nsIChannel* aChannel,
-                             imgCacheEntry* aCacheEntry,
-                             nsISupports* aCX,
-                             nsIPrincipal* aTriggeringPrincipal,
-                             int32_t aCORSMode,
-                             ReferrerPolicy aReferrerPolicy);
+  [[nodiscard]] nsresult Init(nsIURI* aURI, nsIURI* aFinalURI,
+                              bool aHadInsecureRedirect, nsIRequest* aRequest,
+                              nsIChannel* aChannel, imgCacheEntry* aCacheEntry,
+                              mozilla::dom::Document* aLoadingDocument,
+                              nsIPrincipal* aTriggeringPrincipal,
+                              mozilla::CORSMode aCORSMode,
+                              nsIReferrerInfo* aReferrerInfo);
 
   void ClearLoader();
 
   // Callers must call imgRequestProxy::Notify later.
   void AddProxy(imgRequestProxy* proxy);
 
+  // Whether a given document is allowed to reuse this request without any
+  // revalidation.
+  bool CanReuseWithoutValidation(mozilla::dom::Document*) const;
+
   nsresult RemoveProxy(imgRequestProxy* proxy, nsresult aStatus);
 
   // Cancel, but also ensure that all work done in Init() is undone. Call this
@@ -97,9 +95,8 @@ public:
   // Request that we start decoding the image as soon as data becomes available.
   void StartDecoding();
 
-  inline uint64_t InnerWindowID() const {
-    return mInnerWindowId;
-  }
+  uint64_t InnerWindowID() const;
+  void SetInnerWindowID(uint64_t aInnerWindowId);
 
   // Set the cache validation information (expiry time, whether we must
   // validate, etc) on the cache entry based on the request information.
@@ -107,12 +104,6 @@ public:
   // wins.
   static void SetCacheValidation(imgCacheEntry* aEntry, nsIRequest* aRequest);
 
-  // Check if application cache of the original load is different from
-  // application cache of the new load.  Also lack of application cache
-  // on one of the loads is considered a change of a loading cache since
-  // HTTP cache may contain a different data then app cache.
-  bool CacheChanged(nsIRequest* aNewRequest);
-
   bool GetMultipart() const;
 
   // Returns whether we went through an insecure (non-HTTPS) redirect at some
@@ -120,15 +111,14 @@ public:
   bool HadInsecureRedirect() const;
 
   // The CORS mode for which we loaded this image.
-  int32_t GetCORSMode() const { return mCORSMode; }
+  mozilla::CORSMode GetCORSMode() const { return mCORSMode; }
 
-  // The Referrer Policy in effect when loading this image.
-  ReferrerPolicy GetReferrerPolicy() const { return mReferrerPolicy; }
+  // The ReferrerInfo in effect when loading this image.
+  nsIReferrerInfo* GetReferrerInfo() const { return mReferrerInfo; }
 
   // The principal for the document that loaded this image. Used when trying to
   // validate a CORS image load.
-  already_AddRefed<nsIPrincipal> GetTriggeringPrincipal() const
-  {
+  already_AddRefed<nsIPrincipal> GetTriggeringPrincipal() const {
     nsCOMPtr<nsIPrincipal> principal = mTriggeringPrincipal;
     return principal.forget();
   }
@@ -151,20 +141,18 @@ public:
   void ResetCacheEntry();
 
   // OK to use on any thread.
-  nsresult GetURI(ImageURL** aURI);
+  nsresult GetURI(nsIURI** aURI);
   nsresult GetFinalURI(nsIURI** aURI);
-  bool IsScheme(const char* aScheme) const;
   bool IsChrome() const;
   bool IsData() const;
 
   nsresult GetImageErrorCode(void);
 
-  /// Returns true if we've received any data.
-  bool HasTransferredData() const;
-
   /// Returns a non-owning pointer to this imgRequest's MIME type.
   const char* GetMimeType() const { return mContentType.get(); }
 
+  void GetFileName(nsACString& aFileName);
+
   /// @return the priority of the underlying network request, or
   /// PRIORITY_NORMAL if it doesn't support nsISupportsPriority.
   int32_t Priority() const;
@@ -180,8 +168,6 @@ public:
 
   nsITimedChannel* GetTimedChannel() const { return mTimedChannel; }
 
-  nsresult GetSecurityInfo(nsISupports** aSecurityInfoOut);
-
   imgCacheValidator* GetValidator() const { return mValidator; }
   void SetValidator(imgCacheValidator* aValidator) { mValidator = aValidator; }
 
@@ -212,7 +198,15 @@ public:
 
   bool HasConsumers() const;
 
-private:
+  bool ImageAvailable() const;
+
+  bool IsDeniedCrossSiteCORSRequest() const {
+    return mIsDeniedCrossSiteCORSRequest;
+  }
+
+  bool IsCrossSiteNoCORSRequest() const { return mIsCrossSiteNoCORSRequest; }
+
+ private:
   friend class FinishPreparingForNewPartRunnable;
 
   virtual ~imgRequest();
@@ -235,20 +229,19 @@ private:
   // The original URI we were loaded with. This is the same as the URI we are
   // keyed on in the cache. We store a string here to avoid off main thread
   // refcounting issues with nsStandardURL.
-  RefPtr<ImageURL> mURI;
+  nsCOMPtr<nsIURI> mURI;
   // The URI of the resource we ended up loading after all redirects, etc.
   nsCOMPtr<nsIURI> mFinalURI;
   // The principal which triggered the load of this image. Generally either
   // the principal of the document the image is being loaded into, or of the
-  // stylesheet which specified the image to load. Used when validating for CORS.
+  // stylesheet which specified the image to load. Used when validating for
+  // CORS.
   nsCOMPtr<nsIPrincipal> mTriggeringPrincipal;
   // The principal of this image.
   nsCOMPtr<nsIPrincipal> mPrincipal;
   nsCOMPtr<nsIProperties> mProperties;
-  nsCOMPtr<nsISupports> mSecurityInfo;
   nsCOMPtr<nsIChannel> mChannel;
   nsCOMPtr<nsIInterfaceRequestor> mPrevChannelSink;
-  nsCOMPtr<nsIApplicationCache> mApplicationCache;
 
   nsCOMPtr<nsITimedChannel> mTimedChannel;
 
@@ -270,34 +263,37 @@ private:
   nsCOMPtr<nsIAsyncVerifyRedirectCallback> mRedirectCallback;
   nsCOMPtr<nsIChannel> mNewRedirectChannel;
 
-  // The ID of the inner window origin, used for error reporting.
-  uint64_t mInnerWindowId;
-
   // The CORS mode (defined in imgIRequest) this image was loaded with. By
-  // default, imgIRequest::CORS_NONE.
-  int32_t mCORSMode;
+  // default, CORS_NONE.
+  mozilla::CORSMode mCORSMode;
 
-  // The Referrer Policy (defined in ReferrerPolicy.h) used for this image.
-  ReferrerPolicy mReferrerPolicy;
+  // The ReferrerInfo used for this image.
+  nsCOMPtr<nsIReferrerInfo> mReferrerInfo;
 
   nsresult mImageErrorCode;
 
   // The categories of prioritization strategy that have been requested.
   uint32_t mBoostCategoriesRequested = 0;
 
+  // If we've called OnImageAvailable.
+  bool mImageAvailable;
+  bool mIsDeniedCrossSiteCORSRequest;
+  bool mIsCrossSiteNoCORSRequest;
+
   mutable mozilla::Mutex mMutex;
 
   // Member variables protected by mMutex. Note that *all* flags in our bitfield
   // are protected by mMutex; if you're adding a new flag that isn'protected, it
   // must not be a part of this bitfield.
-  RefPtr<ProgressTracker> mProgressTracker;
-  RefPtr<Image> mImage;
-  bool mIsMultiPartChannel : 1;
-  bool mGotData : 1;
-  bool mIsInCache : 1;
-  bool mDecodeRequested : 1;
-  bool mNewPartPending : 1;
-  bool mHadInsecureRedirect : 1;
+  RefPtr<ProgressTracker> mProgressTracker GUARDED_BY(mMutex);
+  RefPtr<Image> mImage GUARDED_BY(mMutex);
+  bool mIsMultiPartChannel : 1 GUARDED_BY(mMutex);
+  bool mIsInCache : 1 GUARDED_BY(mMutex);
+  bool mDecodeRequested : 1 GUARDED_BY(mMutex);
+  bool mNewPartPending : 1 GUARDED_BY(mMutex);
+  bool mHadInsecureRedirect : 1 GUARDED_BY(mMutex);
+  // The ID of the inner window origin, used for error reporting.
+  uint64_t mInnerWindowId GUARDED_BY(mMutex);
 };
 
-#endif // mozilla_image_imgRequest_h
+#endif  // mozilla_image_imgRequest_h
