@@ -5,24 +5,18 @@
 
 /*
  * This module implements a simple archive extractor for the PKZIP format.
- *
- * The underlying nsZipArchive is NOT thread-safe. Do not pass references
- * or pointers to it across thread boundaries.
  */
 
 #define READTYPE int32_t
 #include "zlib.h"
-#ifdef MOZ_JAR_BROTLI
-#  include "brotli/decode.h"  // brotli
-#endif
 #include "nsISupportsUtils.h"
-#include "MmapFaultHandler.h"
+#include "mozilla/MmapFaultHandler.h"
 #include "prio.h"
-#include "plstr.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/Logging.h"
 #include "mozilla/MemUtils.h"
 #include "mozilla/UniquePtrExtensions.h"
+#include "mozilla/StaticMutex.h"
 #include "stdlib.h"
 #include "nsDirectoryService.h"
 #include "nsWildCard.h"
@@ -70,6 +64,18 @@
 
 using namespace mozilla;
 
+static LazyLogModule gZipLog("nsZipArchive");
+
+#ifdef LOG
+#  undef LOG
+#endif
+#ifdef LOG_ENABLED
+#  undef LOG_ENABLED
+#endif
+
+#define LOG(args) MOZ_LOG(gZipLog, mozilla::LogLevel::Debug, args)
+#define LOG_ENABLED() MOZ_LOG_TEST(gZipLog, mozilla::LogLevel::Debug)
+
 static const uint32_t kMaxNameLength = PATH_MAX; /* Maximum name length */
 // For synthetic zip entries. Date/time corresponds to 1980-01-01 00:00.
 static const uint16_t kSyntheticTime = 0;
@@ -82,7 +88,13 @@ static uint32_t HashName(const char* aName, uint16_t nameLen);
 class ZipArchiveLogger {
  public:
   void Init(const char* env) {
-    if (!fd) {
+    StaticMutexAutoLock lock(sLock);
+
+    // AddRef
+    MOZ_ASSERT(mRefCnt >= 0);
+    ++mRefCnt;
+
+    if (!mFd) {
       nsCOMPtr<nsIFile> logFile;
       nsresult rv = NS_NewLocalFile(NS_ConvertUTF8toUTF16(env), false,
                                     getter_AddRefs(logFile));
@@ -107,42 +119,43 @@ class ZipArchiveLogger {
       file = PR_ImportFile((PROsfd)handle);
       if (!file) return;
 #else
-      rv = logFile->OpenNSPRFileDesc(PR_WRONLY | PR_CREATE_FILE | PR_APPEND,
-                                     0644, &file);
+      rv = logFile->OpenNSPRFileDesc(
+          PR_WRONLY | PR_CREATE_FILE | PR_APPEND | PR_SYNC, 0644, &file);
       if (NS_FAILED(rv)) return;
 #endif
-      fd = file;
+      mFd = file;
     }
   }
 
-  void Write(const nsACString& zip, const char* entry) const {
-    if (fd) {
+  void Write(const nsACString& zip, const char* entry) {
+    StaticMutexAutoLock lock(sLock);
+
+    if (mFd) {
       nsCString buf(zip);
       buf.Append(' ');
       buf.Append(entry);
       buf.Append('\n');
-      PR_Write(fd, buf.get(), buf.Length());
+      PR_Write(mFd, buf.get(), buf.Length());
     }
   }
 
-  void AddRef() {
-    MOZ_ASSERT(refCnt >= 0);
-    ++refCnt;
-  }
-
   void Release() {
-    MOZ_ASSERT(refCnt > 0);
-    if ((0 == --refCnt) && fd) {
-      PR_Close(fd);
-      fd = nullptr;
+    StaticMutexAutoLock lock(sLock);
+
+    MOZ_ASSERT(mRefCnt > 0);
+    if ((0 == --mRefCnt) && mFd) {
+      PR_Close(mFd);
+      mFd = nullptr;
     }
   }
 
  private:
-  int refCnt;
-  PRFileDesc* fd;
+  static StaticMutex sLock;
+  int mRefCnt GUARDED_BY(sLock);
+  PRFileDesc* mFd GUARDED_BY(sLock);
 };
 
+StaticMutex ZipArchiveLogger::sLock;
 static ZipArchiveLogger zipLog;
 
 //***********************************************************
@@ -176,6 +189,7 @@ nsresult nsZipHandle::Init(nsIFile* file, nsZipHandle** ret, PRFileDesc** aFd) {
 #if defined(XP_WIN)
   flags |= nsIFile::OS_READAHEAD;
 #endif
+  LOG(("ZipHandle::Init %s", file->HumanReadablePath().get()));
   nsresult rv = file->OpenNSPRFileDesc(flags, 0000, &fd.rwget());
   if (NS_FAILED(rv)) return rv;
 
@@ -227,7 +241,8 @@ nsresult nsZipHandle::Init(nsZipArchive* zip, const char* entry,
   RefPtr<nsZipHandle> handle = new nsZipHandle();
   if (!handle) return NS_ERROR_OUT_OF_MEMORY;
 
-  handle->mBuf = new nsZipItemPtr<uint8_t>(zip, entry);
+  LOG(("ZipHandle::Init entry %s", entry));
+  handle->mBuf = MakeUnique<nsZipItemPtr<uint8_t>>(zip, entry);
   if (!handle->mBuf) return NS_ERROR_OUT_OF_MEMORY;
 
   if (!handle->mBuf->Buffer()) return NS_ERROR_UNEXPECTED;
@@ -244,20 +259,6 @@ nsresult nsZipHandle::Init(nsZipArchive* zip, const char* entry,
   return NS_OK;
 }
 
-nsresult nsZipHandle::Init(const uint8_t* aData, uint32_t aLen,
-                           nsZipHandle** aRet) {
-  RefPtr<nsZipHandle> handle = new nsZipHandle();
-
-  handle->mFileStart = aData;
-  handle->mTotalLen = aLen;
-  nsresult rv = handle->findDataStart();
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  handle.forget(aRet);
-  return NS_OK;
-}
-
 // This function finds the start of the ZIP data. If the file is a regular ZIP,
 // this is just the start of the file. If the file is a CRX file, the start of
 // the data is after the CRX header.
@@ -293,6 +294,7 @@ nsresult nsZipHandle::findDataStart() {
       mFileData = mFileStart + headerSize;
       return NS_OK;
     }
+    return NS_ERROR_FILE_CORRUPTED;
   }
   mLen = mTotalLen;
   mFileData = mFileStart;
@@ -333,63 +335,20 @@ nsZipHandle::~nsZipHandle() {
 //---------------------------------------------
 //  nsZipArchive::OpenArchive
 //---------------------------------------------
-nsresult nsZipArchive::OpenArchive(nsZipHandle* aZipHandle, PRFileDesc* aFd) {
-  mFd = aZipHandle;
-
-  //-- get table of contents for archive
-  nsresult rv = BuildFileList(aFd);
-  if (NS_SUCCEEDED(rv)) {
-    if (aZipHandle->mFile && XRE_IsParentProcess()) {
-      static char* env = PR_GetEnv("MOZ_JAR_LOG_FILE");
-      if (env) {
-        zipLog.Init(env);
-        // We only log accesses in jar/zip archives within the NS_GRE_DIR
-        // and/or the APK on Android. For the former, we log the archive path
-        // relative to NS_GRE_DIR, and for the latter, the nested-archive
-        // path within the APK. This makes the path match the path of the
-        // archives relative to the packaged dist/$APP_NAME directory in a
-        // build.
-        if (aZipHandle->mFile.IsZip()) {
-          // Nested archive, likely omni.ja in APK.
-          aZipHandle->mFile.GetPath(mURI);
-        } else if (nsDirectoryService::gService) {
-          // We can reach here through the initialization of Omnijar from
-          // XRE_InitCommandLine, which happens before the directory service
-          // is initialized. When that happens, it means the opened archive is
-          // the APK, and we don't care to log that one, so we just skip
-          // when the directory service is not initialized.
-          nsCOMPtr<nsIFile> dir = aZipHandle->mFile.GetBaseFile();
-          nsCOMPtr<nsIFile> gre_dir;
-          nsAutoCString path;
-          if (NS_SUCCEEDED(nsDirectoryService::gService->Get(
-                  NS_GRE_DIR, NS_GET_IID(nsIFile), getter_AddRefs(gre_dir)))) {
-            nsAutoCString leaf;
-            nsCOMPtr<nsIFile> parent;
-            while (NS_SUCCEEDED(dir->GetNativeLeafName(leaf)) &&
-                   NS_SUCCEEDED(dir->GetParent(getter_AddRefs(parent)))) {
-              if (!parent) {
-                break;
-              }
-              dir = parent;
-              if (path.Length()) {
-                path.Insert('/', 0);
-              }
-              path.Insert(leaf, 0);
-              bool equals;
-              if (NS_SUCCEEDED(dir->Equals(gre_dir, &equals)) && equals) {
-                mURI.Assign(path);
-                break;
-              }
-            }
-          }
-        }
-      }
-    }
+/* static */
+already_AddRefed<nsZipArchive> nsZipArchive::OpenArchive(
+    nsZipHandle* aZipHandle, PRFileDesc* aFd) {
+  nsresult rv;
+  RefPtr<nsZipArchive> self(new nsZipArchive(aZipHandle, aFd, rv));
+  LOG(("ZipHandle::OpenArchive[%p]", self.get()));
+  if (NS_FAILED(rv)) {
+    self = nullptr;
   }
-  return rv;
+  return self.forget();
 }
 
-nsresult nsZipArchive::OpenArchive(nsIFile* aFile) {
+/* static */
+already_AddRefed<nsZipArchive> nsZipArchive::OpenArchive(nsIFile* aFile) {
   RefPtr<nsZipHandle> handle;
 #if defined(XP_WIN)
   mozilla::AutoFDClose fd;
@@ -397,7 +356,7 @@ nsresult nsZipArchive::OpenArchive(nsIFile* aFile) {
 #else
   nsresult rv = nsZipHandle::Init(aFile, getter_AddRefs(handle));
 #endif
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) return nullptr;
 
 #if defined(XP_WIN)
   return OpenArchive(handle, fd.get());
@@ -415,7 +374,7 @@ nsresult nsZipArchive::Test(const char* aEntryName) {
   if (aEntryName)  // only test specified item
   {
     currItem = GetItem(aEntryName);
-    if (!currItem) return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
+    if (!currItem) return NS_ERROR_FILE_NOT_FOUND;
     //-- don't test (synthetic) directory items
     if (currItem->IsDirectory()) return NS_OK;
     return ExtractFile(currItem, 0, 0);
@@ -434,31 +393,13 @@ nsresult nsZipArchive::Test(const char* aEntryName) {
   return NS_OK;
 }
 
-//---------------------------------------------
-//  nsZipArchive::CloseArchive
-//---------------------------------------------
-nsresult nsZipArchive::CloseArchive() {
-  if (mFd) {
-    mArena.Clear();
-    mFd = nullptr;
-  }
-
-  // CAUTION:
-  // We don't need to delete each of the nsZipItem as the memory for
-  // the zip item and the filename it holds are both allocated from the Arena.
-  // Hence, destroying the Arena is like destroying all the memory
-  // for all the nsZipItem in one shot. But if the ~nsZipItem is doing
-  // anything more than cleaning up memory, we should start calling it.
-  // Let us also cleanup the mFiles table for re-use on the next 'open' call
-  memset(mFiles, 0, sizeof(mFiles));
-  mBuiltSynthetics = false;
-  return NS_OK;
-}
-
 //---------------------------------------------
 // nsZipArchive::GetItem
 //---------------------------------------------
 nsZipItem* nsZipArchive::GetItem(const char* aEntryName) {
+  MutexAutoLock lock(mLock);
+
+  LOG(("ZipHandle::GetItem[%p] %s", this, aEntryName));
   if (aEntryName) {
     uint32_t len = strlen(aEntryName);
     //-- If the request is for a directory, make sure that synthetic entries
@@ -475,7 +416,7 @@ nsZipItem* nsZipArchive::GetItem(const char* aEntryName) {
           (!memcmp(aEntryName, item->Name(), len))) {
         // Successful GetItem() is a good indicator that the file is about to be
         // read
-        if (mURI.Length()) {
+        if (mUseZipLog && mURI.Length()) {
           zipLog.Write(mURI, aEntryName);
         }
         return item;  //-- found it
@@ -495,6 +436,8 @@ nsZipItem* nsZipArchive::GetItem(const char* aEntryName) {
 //---------------------------------------------
 nsresult nsZipArchive::ExtractFile(nsZipItem* item, nsIFile* outFile,
                                    PRFileDesc* aFd) {
+  MutexAutoLock lock(mLock);
+  LOG(("ZipHandle::ExtractFile[%p]", this));
   if (!item) return NS_ERROR_ILLEGAL_VALUE;
   if (!mFd) return NS_ERROR_FAILURE;
 
@@ -520,7 +463,7 @@ nsresult nsZipArchive::ExtractFile(nsZipItem* item, nsIFile* outFile,
     }
 
     if (aFd && PR_Write(aFd, buf, count) < (READTYPE)count) {
-      rv = NS_ERROR_FILE_DISK_FULL;
+      rv = NS_ERROR_FILE_NO_DEVICE_SPACE;
       break;
     }
   }
@@ -542,6 +485,9 @@ nsresult nsZipArchive::ExtractFile(nsZipItem* item, nsIFile* outFile,
 nsresult nsZipArchive::FindInit(const char* aPattern, nsZipFind** aFind) {
   if (!aFind) return NS_ERROR_ILLEGAL_VALUE;
 
+  MutexAutoLock lock(mLock);
+
+  LOG(("ZipHandle::FindInit[%p]", this));
   // null out param in case an error happens
   *aFind = nullptr;
 
@@ -572,13 +518,13 @@ nsresult nsZipArchive::FindInit(const char* aPattern, nsZipFind** aFind) {
         return NS_ERROR_ILLEGAL_VALUE;
     }
 
-    pattern = PL_strdup(aPattern);
+    pattern = strdup(aPattern);
     if (!pattern) return NS_ERROR_OUT_OF_MEMORY;
   }
 
   *aFind = new nsZipFind(this, pattern, regExp);
   if (!*aFind) {
-    PL_strfree(pattern);
+    free(pattern);
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
@@ -591,6 +537,7 @@ nsresult nsZipArchive::FindInit(const char* aPattern, nsZipFind** aFind) {
 nsresult nsZipFind::FindNext(const char** aResult, uint16_t* aNameLen) {
   if (!mArchive || !aResult || !aNameLen) return NS_ERROR_ILLEGAL_VALUE;
 
+  MutexAutoLock lock(mArchive->mLock);
   *aResult = 0;
   *aNameLen = 0;
   MMAP_FAULT_HANDLER_BEGIN_HANDLE(mArchive->GetFD())
@@ -616,11 +563,13 @@ nsresult nsZipFind::FindNext(const char** aResult, uint16_t* aNameLen) {
       // Need also to return the name length, as it is NOT zero-terminatdd...
       *aResult = mItem->Name();
       *aNameLen = mItem->nameLength;
+      LOG(("ZipHandle::FindNext[%p] %s", this, *aResult));
       return NS_OK;
     }
   }
   MMAP_FAULT_HANDLER_CATCH(NS_ERROR_FAILURE)
-  return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
+  LOG(("ZipHandle::FindNext[%p] not found %s", this, mPattern));
+  return NS_ERROR_FILE_NOT_FOUND;
 }
 
 //***********************************************************
@@ -632,19 +581,26 @@ nsresult nsZipFind::FindNext(const char** aResult, uint16_t* aNameLen) {
 //---------------------------------------------
 nsZipItem* nsZipArchive::CreateZipItem() {
   // Arena allocate the nsZipItem
-  return (nsZipItem*)mArena.Allocate(sizeof(nsZipItem));
+  return (nsZipItem*)mArena.Allocate(sizeof(nsZipItem), mozilla::fallible);
 }
 
 //---------------------------------------------
 //  nsZipArchive::BuildFileList
 //---------------------------------------------
-nsresult nsZipArchive::BuildFileList(PRFileDesc* aFd) {
+nsresult nsZipArchive::BuildFileList(PRFileDesc* aFd)
+    NO_THREAD_SAFETY_ANALYSIS {
+  // We're only called from the constructor, but need to call
+  // CreateZipItem(), which touches locked data, and modify mFiles.  Turn
+  // off thread-safety, which normally doesn't apply for constructors
+  // anyways
+
   // Get archive size using end pos
   const uint8_t* buf;
   const uint8_t* startp = mFd->mFileData;
   const uint8_t* endp = startp + mFd->mLen;
   MMAP_FAULT_HANDLER_BEGIN_HANDLE(mFd)
   uint32_t centralOffset = 4;
+  LOG(("ZipHandle::BuildFileList[%p]", this));
   // Only perform readahead in the parent process. Children processes
   // don't need readahead when the file has already been readahead by
   // the parent process, and readahead only really happens for omni.ja,
@@ -713,6 +669,10 @@ nsresult nsZipArchive::BuildFileList(PRFileDesc* aFd) {
     item->isSynthetic = false;
 
     // Add item to file table
+#ifdef DEBUG
+    nsDependentCSubstring name(item->Name(), namelen);
+    LOG(("   %s", PromiseFlatCString(name).get()));
+#endif
     uint32_t hash = HashName(item->Name(), namelen);
     item->next = mFiles[hash];
     mFiles[hash] = item;
@@ -724,18 +684,6 @@ nsresult nsZipArchive::BuildFileList(PRFileDesc* aFd) {
     return NS_ERROR_FILE_CORRUPTED;
   }
 
-  // Make the comment available for consumers.
-  if ((endp >= buf) && (endp - buf >= ZIPEND_SIZE)) {
-    ZipEnd* zipend = (ZipEnd*)buf;
-
-    buf += ZIPEND_SIZE;
-    uint16_t commentlen = xtoint(zipend->commentfield_len);
-    if (endp - buf >= commentlen) {
-      mCommentPtr = (const char*)buf;
-      mCommentLen = commentlen;
-    }
-  }
-
   MMAP_FAULT_HANDLER_CATCH(NS_ERROR_FAILURE)
   return NS_OK;
 }
@@ -744,6 +692,8 @@ nsresult nsZipArchive::BuildFileList(PRFileDesc* aFd) {
 //  nsZipArchive::BuildSynthetics
 //---------------------------------------------
 nsresult nsZipArchive::BuildSynthetics() {
+  mLock.AssertCurrentThreadOwns();
+
   if (mBuiltSynthetics) return NS_OK;
   mBuiltSynthetics = true;
 
@@ -805,13 +755,14 @@ nsresult nsZipArchive::BuildSynthetics() {
   return NS_OK;
 }
 
-nsZipHandle* nsZipArchive::GetFD() {
-  if (!mFd) return nullptr;
-  return mFd.get();
-}
+//---------------------------------------------
+// nsZipArchive::GetFD
+//---------------------------------------------
+nsZipHandle* nsZipArchive::GetFD() const { return mFd.get(); }
 
 //---------------------------------------------
 // nsZipArchive::GetDataOffset
+// Returns 0 on an error; 0 is not a valid result for any success case
 //---------------------------------------------
 uint32_t nsZipArchive::GetDataOffset(nsZipItem* aItem) {
   MOZ_ASSERT(aItem);
@@ -821,9 +772,17 @@ uint32_t nsZipArchive::GetDataOffset(nsZipItem* aItem) {
   //-- read local header to get variable length values and calculate
   //-- the real data offset
   uint32_t len = mFd->mLen;
+  MOZ_DIAGNOSTIC_ASSERT(len <= UINT32_MAX, "mLen > 2GB");
   const uint8_t* data = mFd->mFileData;
   offset = aItem->LocalOffset();
-  if (len < ZIPLOCAL_SIZE || offset > len - ZIPLOCAL_SIZE) return 0;
+  if (len < ZIPLOCAL_SIZE || offset > len - ZIPLOCAL_SIZE) {
+    return 0;
+  }
+  // Check there's enough space for the signature
+  if (offset > mFd->mLen) {
+    NS_WARNING("Corrupt local offset in JAR file");
+    return 0;
+  }
 
   // -- check signature before using the structure, in case the zip file is
   // corrupt
@@ -835,8 +794,14 @@ uint32_t nsZipArchive::GetDataOffset(nsZipItem* aItem) {
   //--       the offset accurately we need the _local_ extralen.
   offset += ZIPLOCAL_SIZE + xtoint(Local->filename_len) +
             xtoint(Local->extrafield_len);
+  // Check data points inside the file.
+  if (offset > mFd->mLen) {
+    NS_WARNING("Corrupt data offset in JAR file");
+    return 0;
+  }
 
   MMAP_FAULT_HANDLER_CATCH(0)
+  // can't be 0
   return offset;
 }
 
@@ -859,14 +824,6 @@ const uint8_t* nsZipArchive::GetData(nsZipItem* aItem) {
   return mFd->mFileData + offset;
 }
 
-// nsZipArchive::GetComment
-bool nsZipArchive::GetComment(nsACString& aComment) {
-  MMAP_FAULT_HANDLER_BEGIN_BUFFER(mCommentPtr, mCommentLen)
-  aComment.Assign(mCommentPtr, mCommentLen);
-  MMAP_FAULT_HANDLER_CATCH(false)
-  return true;
-}
-
 //---------------------------------------------
 // nsZipArchive::SizeOfMapping
 //---------------------------------------------
@@ -876,24 +833,75 @@ int64_t nsZipArchive::SizeOfMapping() { return mFd ? mFd->SizeOfMapping() : 0; }
 // nsZipArchive constructor and destructor
 //------------------------------------------
 
-nsZipArchive::nsZipArchive()
-    : mRefCnt(0),
-      mCommentPtr(nullptr),
-      mCommentLen(0),
-      mBuiltSynthetics(false) {
-  zipLog.AddRef();
-
+nsZipArchive::nsZipArchive(nsZipHandle* aZipHandle, PRFileDesc* aFd,
+                           nsresult& aRv)
+    : mRefCnt(0), mFd(aZipHandle), mUseZipLog(false), mBuiltSynthetics(false) {
   // initialize the table to nullptr
   memset(mFiles, 0, sizeof(mFiles));
+
+  //-- get table of contents for archive
+  aRv = BuildFileList(aFd);
+  if (NS_FAILED(aRv)) {
+    return;  // whomever created us must destroy us in this case
+  }
+  if (aZipHandle->mFile && XRE_IsParentProcess()) {
+    static char* env = PR_GetEnv("MOZ_JAR_LOG_FILE");
+    if (env) {
+      mUseZipLog = true;
+
+      zipLog.Init(env);
+      // We only log accesses in jar/zip archives within the NS_GRE_DIR
+      // and/or the APK on Android. For the former, we log the archive path
+      // relative to NS_GRE_DIR, and for the latter, the nested-archive
+      // path within the APK. This makes the path match the path of the
+      // archives relative to the packaged dist/$APP_NAME directory in a
+      // build.
+      if (aZipHandle->mFile.IsZip()) {
+        // Nested archive, likely omni.ja in APK.
+        aZipHandle->mFile.GetPath(mURI);
+      } else if (nsDirectoryService::gService) {
+        // We can reach here through the initialization of Omnijar from
+        // XRE_InitCommandLine, which happens before the directory service
+        // is initialized. When that happens, it means the opened archive is
+        // the APK, and we don't care to log that one, so we just skip
+        // when the directory service is not initialized.
+        nsCOMPtr<nsIFile> dir = aZipHandle->mFile.GetBaseFile();
+        nsCOMPtr<nsIFile> gre_dir;
+        nsAutoCString path;
+        if (NS_SUCCEEDED(nsDirectoryService::gService->Get(
+                NS_GRE_DIR, NS_GET_IID(nsIFile), getter_AddRefs(gre_dir)))) {
+          nsAutoCString leaf;
+          nsCOMPtr<nsIFile> parent;
+          while (NS_SUCCEEDED(dir->GetNativeLeafName(leaf)) &&
+                 NS_SUCCEEDED(dir->GetParent(getter_AddRefs(parent)))) {
+            if (!parent) {
+              break;
+            }
+            dir = parent;
+            if (path.Length()) {
+              path.Insert('/', 0);
+            }
+            path.Insert(leaf, 0);
+            bool equals;
+            if (NS_SUCCEEDED(dir->Equals(gre_dir, &equals)) && equals) {
+              mURI.Assign(path);
+              break;
+            }
+          }
+        }
+      }
+    }
+  }
 }
 
 NS_IMPL_ADDREF(nsZipArchive)
 NS_IMPL_RELEASE(nsZipArchive)
 
 nsZipArchive::~nsZipArchive() {
-  CloseArchive();
-
-  zipLog.Release();
+  LOG(("Closing nsZipArchive[%p]", this));
+  if (mUseZipLog) {
+    zipLog.Release();
+  }
 }
 
 //------------------------------------------
@@ -910,7 +918,7 @@ nsZipFind::nsZipFind(nsZipArchive* aZip, char* aPattern, bool aRegExp)
 }
 
 nsZipFind::~nsZipFind() {
-  PL_strfree(mPattern);
+  free(mPattern);
 
   MOZ_COUNT_DTOR(nsZipFind);
 }
@@ -1075,12 +1083,7 @@ nsZipCursor::nsZipCursor(nsZipItem* item, nsZipArchive* aZip, uint8_t* aBuf,
     : mItem(item),
       mBuf(aBuf),
       mBufSize(aBufSize),
-      mZs()
-#ifdef MOZ_JAR_BROTLI
-      ,
-      mBrotliState(nullptr)
-#endif
-      ,
+      mZs(),
       mCRC(0),
       mDoCRC(doCRC) {
   if (mItem->Compression() == DEFLATED) {
@@ -1095,12 +1098,6 @@ nsZipCursor::nsZipCursor(nsZipItem* item, nsZipArchive* aZip, uint8_t* aBuf,
   mZs.avail_in = item->Size();
   mZs.next_in = (Bytef*)aZip->GetData(item);
 
-#ifdef MOZ_JAR_BROTLI
-  if (mItem->Compression() == MOZ_JAR_BROTLI) {
-    mBrotliState = BrotliDecoderCreateInstance(nullptr, nullptr, nullptr);
-  }
-#endif
-
   if (doCRC) mCRC = crc32(0L, Z_NULL, 0);
 }
 
@@ -1108,11 +1105,6 @@ nsZipCursor::~nsZipCursor() {
   if (mItem->Compression() == DEFLATED) {
     inflateEnd(&mZs);
   }
-#ifdef MOZ_JAR_BROTLI
-  if (mItem->Compression() == MOZ_JAR_BROTLI) {
-    BrotliDecoderDestroyInstance(mBrotliState);
-  }
-#endif
 }
 
 uint8_t* nsZipCursor::ReadOrCopy(uint32_t* aBytesRead, bool aCopy) {
@@ -1147,31 +1139,6 @@ uint8_t* nsZipCursor::ReadOrCopy(uint32_t* aBytesRead, bool aCopy) {
       *aBytesRead = mZs.next_out - buf;
       verifyCRC = (zerr == Z_STREAM_END);
       break;
-#ifdef MOZ_JAR_BROTLI
-    case MOZ_JAR_BROTLI: {
-      buf = mBuf;
-      mZs.next_out = buf;
-      /* The brotli library wants size_t, but z_stream only contains
-       * unsigned int for avail_*. So use temporary stack values. */
-      size_t avail_out = mBufSize;
-      size_t avail_in = mZs.avail_in;
-      BrotliDecoderResult result = BrotliDecoderDecompressStream(
-          mBrotliState, &avail_in,
-          const_cast<const unsigned char**>(&mZs.next_in), &avail_out,
-          &mZs.next_out, nullptr);
-      /* We don't need to update avail_out, it's not used outside this
-       * function. */
-      mZs.avail_in = avail_in;
-
-      if (result == BROTLI_DECODER_RESULT_ERROR) {
-        return nullptr;
-      }
-
-      *aBytesRead = mZs.next_out - buf;
-      verifyCRC = (result == BROTLI_DECODER_RESULT_SUCCESS);
-      break;
-    }
-#endif
     default:
       return nullptr;
   }
@@ -1195,9 +1162,6 @@ nsZipItemPtr_base::nsZipItemPtr_base(nsZipArchive* aZip, const char* aEntryName,
 
   uint32_t size = 0;
   bool compressed = (item->Compression() == DEFLATED);
-#ifdef MOZ_JAR_BROTLI
-  compressed |= (item->Compression() == MOZ_JAR_BROTLI);
-#endif
   if (compressed) {
     size = item->RealSize();
     mAutoBuf = MakeUniqueFallible<uint8_t[]>(size);
