@@ -570,7 +570,6 @@ LoadInfo::LoadInfo(const LoadInfo& rhs)
       mForcePreflight(rhs.mForcePreflight),
       mIsPreflight(rhs.mIsPreflight),
       mLoadTriggeredFromExternal(rhs.mLoadTriggeredFromExternal),
-
       mDocumentHasUserInteracted(rhs.mDocumentHasUserInteracted),
       mAllowListFutureDocumentsCreatedFromThisRedirectChain(
           rhs.mAllowListFutureDocumentsCreatedFromThisRedirectChain),
@@ -590,7 +589,8 @@ LoadInfo::LoadInfo(const LoadInfo& rhs)
       mIsMediaInitialRequest(rhs.mIsMediaInitialRequest),
       mIsFromObjectOrEmbed(rhs.mIsFromObjectOrEmbed),
       mLoadingEmbedderPolicy(rhs.mLoadingEmbedderPolicy),
-      mUnstrippedURI(rhs.mUnstrippedURI) {}
+      mUnstrippedURI(rhs.mUnstrippedURI),
+      mInterceptionInfo(rhs.mInterceptionInfo) {}
 
 LoadInfo::LoadInfo(
     nsIPrincipal* aLoadingPrincipal, nsIPrincipal* aTriggeringPrincipal,
@@ -629,7 +629,7 @@ LoadInfo::LoadInfo(
     nsILoadInfo::StoragePermissionState aStoragePermission, bool aIsMetaRefresh,
     uint32_t aRequestBlockingReason, nsINode* aLoadingContext,
     nsILoadInfo::CrossOriginEmbedderPolicy aLoadingEmbedderPolicy,
-    nsIURI* aUnstrippedURI)
+    nsIURI* aUnstrippedURI, nsIInterceptionInfo* aInterceptionInfo)
     : mLoadingPrincipal(aLoadingPrincipal),
       mTriggeringPrincipal(aTriggeringPrincipal),
       mPrincipalToInherit(aPrincipalToInherit),
@@ -694,9 +694,9 @@ LoadInfo::LoadInfo(
       mParserCreatedScript(aParserCreatedScript),
       mStoragePermission(aStoragePermission),
       mIsMetaRefresh(aIsMetaRefresh),
-
       mLoadingEmbedderPolicy(aLoadingEmbedderPolicy),
-      mUnstrippedURI(aUnstrippedURI) {
+      mUnstrippedURI(aUnstrippedURI),
+      mInterceptionInfo(aInterceptionInfo){
   // Only top level TYPE_DOCUMENT loads can have a null loadingPrincipal
   MOZ_ASSERT(mLoadingPrincipal ||
              aContentPolicyType == nsIContentPolicy::TYPE_DOCUMENT);
@@ -792,21 +792,7 @@ LoadInfo::Release() {
 
 NS_IMPL_QUERY_INTERFACE(LoadInfo, nsILoadInfo)
 
-void LoadInfo::ReleaseMembers() {
-  mCSPEventListener = nullptr;
-  mCookieJarSettings = nullptr;
-  mPerformanceStorage = nullptr;
-  mLoadingPrincipal = nullptr;
-  mTriggeringPrincipal = nullptr;
-  mPrincipalToInherit = nullptr;
-  mTopLevelPrincipal = nullptr;
-  mResultPrincipalURI = nullptr;
-  mCspToInherit = nullptr;
-  mUnstrippedURI = nullptr;
-  mAncestorPrincipals.Clear();
-}
-
-LoadInfo::~LoadInfo() { ReleaseMembers(); }
+LoadInfo::~LoadInfo() { MOZ_RELEASE_ASSERT(NS_IsMainThread()); }
 
 already_AddRefed<nsILoadInfo> LoadInfo::Clone() const {
   RefPtr<LoadInfo> copy(new LoadInfo(*this));
@@ -1800,6 +1786,20 @@ LoadInfo::SetAllowDeprecatedSystemRequests(
   return NS_OK;
 }
 
+NS_IMETHODIMP
+LoadInfo::GetIsUserTriggeredSave(bool* aIsUserTriggeredSave) {
+  *aIsUserTriggeredSave =
+      mIsUserTriggeredSave ||
+      mInternalContentPolicyType == nsIContentPolicy::TYPE_SAVEAS_DOWNLOAD;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+LoadInfo::SetIsUserTriggeredSave(bool aIsUserTriggeredSave) {
+  mIsUserTriggeredSave = aIsUserTriggeredSave;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 LoadInfo::GetIsInDevToolsContext(bool* aIsInDevToolsContext) {
   *aIsInDevToolsContext = mIsInDevToolsContext;
@@ -2139,4 +2139,10 @@ already_AddRefed<nsIContentSecurityPolicy> LoadInfo::GetCspToInherit() {
   return cspToInherit.forget();
 }
 
+nsIInterceptionInfo* LoadInfo::InterceptionInfo() { return mInterceptionInfo; }
+
+void LoadInfo::SetInterceptionInfo(nsIInterceptionInfo* aInfo) {
+  mInterceptionInfo = aInfo;
+}
+
 }  // namespace mozilla::net
