@@ -3,12 +3,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "ipc/IPCMessageUtils.h"
-
 #include "nsBufferedStreams.h"
 #include "nsStreamUtils.h"
 #include "nsNetCID.h"
 #include "nsIClassInfoImpl.h"
+#include "nsIEventTarget.h"
+#include "nsThreadUtils.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/ipc/InputStreamUtils.h"
 #include <algorithm>
@@ -39,29 +39,21 @@ static struct {
 #endif
 
 using namespace mozilla::ipc;
-using mozilla::DebugOnly;
-using mozilla::Maybe;
-using mozilla::MutexAutoLock;
-using mozilla::Nothing;
-using mozilla::Some;
+using namespace mozilla;
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsBufferedStream
 
-nsBufferedStream::nsBufferedStream()
-    : mBufferSize(0),
-      mBuffer(nullptr),
-      mBufferStartOffset(0),
-      mCursor(0),
-      mFillPoint(0),
-      mStream(nullptr),
-      mBufferDisabled(false),
-      mEOF(false),
-      mGetBufferCount(0) {}
-
 nsBufferedStream::~nsBufferedStream() { Close(); }
 
-NS_IMPL_ISUPPORTS(nsBufferedStream, nsITellableStream, nsISeekableStream)
+NS_IMPL_ADDREF(nsBufferedStream)
+NS_IMPL_RELEASE(nsBufferedStream)
+
+NS_INTERFACE_MAP_BEGIN(nsBufferedStream)
+  NS_INTERFACE_MAP_ENTRY(nsISupports)
+  NS_INTERFACE_MAP_ENTRY(nsITellableStream)
+  NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsISeekableStream, mSeekable)
+NS_INTERFACE_MAP_END
 
 nsresult nsBufferedStream::Init(nsISupports* aStream, uint32_t bufferSize) {
   NS_ASSERTION(aStream, "need to supply a stream");
@@ -70,6 +62,9 @@ nsresult nsBufferedStream::Init(nsISupports* aStream, uint32_t bufferSize) {
   mBufferSize = bufferSize;
   mBufferStartOffset = 0;
   mCursor = 0;
+  nsCOMPtr<nsISeekableStream> seekable = do_QueryInterface(mStream);
+  mSeekable = seekable;
+  RecursiveMutexAutoLock lock(mBufferMutex);
   mBuffer = new (mozilla::fallible) char[bufferSize];
   if (mBuffer == nullptr) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -80,6 +75,7 @@ nsresult nsBufferedStream::Init(nsISupports* aStream, uint32_t bufferSize) {
 void nsBufferedStream::Close() {
   // Drop the reference from nsBufferedStream::Init()
   mStream = nullptr;
+  RecursiveMutexAutoLock lock(mBufferMutex);
   if (mBuffer) {
     delete[] mBuffer;
     mBuffer = nullptr;
@@ -224,6 +220,13 @@ nsBufferedStream::Seek(int32_t whence, int64_t offset) {
                 .mNewOffset = mBufferStartOffset);
 
   mFillPoint = mCursor = 0;
+
+  // If we seeked back to the start, then don't fill the buffer
+  // right now in case this is a lazily-opened file stream.
+  // We'll fill on the first read, like we did initially.
+  if (whence == nsISeekableStream::NS_SEEK_SET && offset == 0) {
+    return NS_OK;
+  }
   return Fill();
 }
 
@@ -306,19 +309,7 @@ NS_IMPL_CI_INTERFACE_GETTER(nsBufferedInputStream, nsIInputStream,
                             nsIBufferedInputStream, nsISeekableStream,
                             nsITellableStream, nsIStreamBufferAccess)
 
-nsBufferedInputStream::nsBufferedInputStream()
-    : nsBufferedStream(),
-      mMutex("nsBufferedInputStream::mMutex"),
-      mIsIPCSerializable(true),
-      mIsAsyncInputStream(false),
-      mIsCloneableInputStream(false),
-      mIsInputStreamLength(false),
-      mIsAsyncInputStreamLength(false) {}
-
-nsresult nsBufferedInputStream::Create(nsISupports* aOuter, REFNSIID aIID,
-                                       void** aResult) {
-  NS_ENSURE_NO_AGGREGATION(aOuter);
-
+nsresult nsBufferedInputStream::Create(REFNSIID aIID, void** aResult) {
   RefPtr<nsBufferedInputStream> stream = new nsBufferedInputStream();
   return stream->QueryInterface(aIID, aResult);
 }
@@ -439,6 +430,7 @@ nsBufferedInputStream::ReadSegments(nsWriteSegmentFun writer, void* closure,
   }
 
   nsresult rv = NS_OK;
+  RecursiveMutexAutoLock lock(mBufferMutex);
   while (count > 0) {
     uint32_t amt = std::min(count, mFillPoint - mCursor);
     if (amt > 0) {
@@ -478,6 +470,8 @@ nsBufferedInputStream::Fill() {
   }
   NS_ENSURE_TRUE(mStream, NS_ERROR_NOT_INITIALIZED);
 
+  RecursiveMutexAutoLock lock(mBufferMutex);
+
   nsresult rv;
   int32_t rem = int32_t(mFillPoint - mCursor);
   if (rem > 0) {
@@ -515,6 +509,7 @@ nsBufferedInputStream::GetBuffer(uint32_t aLength, uint32_t aAlignMask) {
     return nullptr;
   }
 
+  RecursiveMutexAutoLock lock(mBufferMutex);
   char* buf = mBuffer + mCursor;
   uint32_t rem = mFillPoint - mCursor;
   if (rem == 0) {
@@ -590,26 +585,21 @@ nsBufferedInputStream::GetUnbufferedStream(nsISupports** aStream) {
   return NS_OK;
 }
 
-void nsBufferedInputStream::Serialize(
-    InputStreamParams& aParams, FileDescriptorArray& aFileDescriptors,
-    bool aDelayedStart, uint32_t aMaxSize, uint32_t* aSizeUsed,
-    mozilla::ipc::ParentToChildStreamActorManager* aManager) {
-  SerializeInternal(aParams, aFileDescriptors, aDelayedStart, aMaxSize,
-                    aSizeUsed, aManager);
+void nsBufferedInputStream::SerializedComplexity(uint32_t aMaxSize,
+                                                 uint32_t* aSizeUsed,
+                                                 uint32_t* aPipes,
+                                                 uint32_t* aTransferables) {
+  if (mStream) {
+    nsCOMPtr<nsIInputStream> stream = do_QueryInterface(mStream);
+    MOZ_ASSERT(stream);
+
+    InputStreamHelper::SerializedComplexity(stream, aMaxSize, aSizeUsed, aPipes,
+                                            aTransferables);
+  }
 }
 
-void nsBufferedInputStream::Serialize(
-    InputStreamParams& aParams, FileDescriptorArray& aFileDescriptors,
-    bool aDelayedStart, uint32_t aMaxSize, uint32_t* aSizeUsed,
-    mozilla::ipc::ChildToParentStreamActorManager* aManager) {
-  SerializeInternal(aParams, aFileDescriptors, aDelayedStart, aMaxSize,
-                    aSizeUsed, aManager);
-}
-
-template <typename M>
-void nsBufferedInputStream::SerializeInternal(
-    InputStreamParams& aParams, FileDescriptorArray& aFileDescriptors,
-    bool aDelayedStart, uint32_t aMaxSize, uint32_t* aSizeUsed, M* aManager) {
+void nsBufferedInputStream::Serialize(InputStreamParams& aParams,
+                                      uint32_t aMaxSize, uint32_t* aSizeUsed) {
   MOZ_ASSERT(aSizeUsed);
   *aSizeUsed = 0;
 
@@ -620,9 +610,8 @@ void nsBufferedInputStream::SerializeInternal(
     MOZ_ASSERT(stream);
 
     InputStreamParams wrappedParams;
-    InputStreamHelper::SerializeInputStream(stream, wrappedParams,
-                                            aFileDescriptors, aDelayedStart,
-                                            aMaxSize, aSizeUsed, aManager);
+    InputStreamHelper::SerializeInputStream(stream, wrappedParams, aMaxSize,
+                                            aSizeUsed);
 
     params.optionalStream().emplace(wrappedParams);
   }
@@ -632,9 +621,7 @@ void nsBufferedInputStream::SerializeInternal(
   aParams = params;
 }
 
-bool nsBufferedInputStream::Deserialize(
-    const InputStreamParams& aParams,
-    const FileDescriptorArray& aFileDescriptors) {
+bool nsBufferedInputStream::Deserialize(const InputStreamParams& aParams) {
   if (aParams.type() != InputStreamParams::TBufferedInputStreamParams) {
     NS_ERROR("Received unknown parameters from the other process!");
     return false;
@@ -646,8 +633,7 @@ bool nsBufferedInputStream::Deserialize(
 
   nsCOMPtr<nsIInputStream> stream;
   if (wrappedParams.isSome()) {
-    stream = InputStreamHelper::DeserializeInputStream(wrappedParams.ref(),
-                                                       aFileDescriptors);
+    stream = InputStreamHelper::DeserializeInputStream(wrappedParams.ref());
     if (!stream) {
       NS_WARNING("Failed to deserialize wrapped stream!");
       return false;
@@ -669,14 +655,28 @@ nsBufferedInputStream::AsyncWait(nsIInputStreamCallback* aCallback,
                                  nsIEventTarget* aEventTarget) {
   nsCOMPtr<nsIAsyncInputStream> stream = do_QueryInterface(mStream);
   if (!stream) {
-    return NS_ERROR_FAILURE;
+    // Stream is probably closed. Callback, if not nullptr, can be executed
+    // immediately
+    if (!aCallback) {
+      return NS_OK;
+    }
+
+    if (aEventTarget) {
+      nsCOMPtr<nsIInputStreamCallback> callable = NS_NewInputStreamReadyEvent(
+          "nsBufferedInputStream::OnInputStreamReady", aCallback, aEventTarget);
+      return callable->OnInputStreamReady(this);
+    }
+
+    aCallback->OnInputStreamReady(this);
+    return NS_OK;
   }
 
   nsCOMPtr<nsIInputStreamCallback> callback = aCallback ? this : nullptr;
   {
     MutexAutoLock lock(mMutex);
 
-    if (mAsyncWaitCallback && aCallback) {
+    if (NS_WARN_IF(mAsyncWaitCallback && aCallback &&
+                   mAsyncWaitCallback != aCallback)) {
       return NS_ERROR_FAILURE;
     }
 
@@ -719,6 +719,8 @@ NS_IMETHODIMP
 nsBufferedInputStream::GetCloneable(bool* aCloneable) {
   *aCloneable = false;
 
+  RecursiveMutexAutoLock lock(mBufferMutex);
+
   // If we don't have the buffer, the inputStream has been already closed.
   // If mBufferStartOffset is not 0, the stream has been seeked or read.
   // In both case the cloning is not supported.
@@ -736,6 +738,8 @@ nsBufferedInputStream::GetCloneable(bool* aCloneable) {
 
 NS_IMETHODIMP
 nsBufferedInputStream::Clone(nsIInputStream** aResult) {
+  RecursiveMutexAutoLock lock(mBufferMutex);
+
   if (!mBuffer || mBufferStartOffset) {
     return NS_ERROR_FAILURE;
   }
@@ -774,7 +778,22 @@ nsBufferedInputStream::AsyncLengthWait(nsIInputStreamLengthCallback* aCallback,
                                        nsIEventTarget* aEventTarget) {
   nsCOMPtr<nsIAsyncInputStreamLength> stream = do_QueryInterface(mStream);
   if (!stream) {
-    return NS_ERROR_FAILURE;
+    // Stream is probably closed. Callback, if not nullptr, can be executed
+    // immediately
+    if (aCallback) {
+      const RefPtr<nsBufferedInputStream> self = this;
+      const nsCOMPtr<nsIInputStreamLengthCallback> callback = aCallback;
+      nsCOMPtr<nsIRunnable> runnable = NS_NewRunnableFunction(
+          "nsBufferedInputStream::OnInputStreamLengthReady",
+          [self, callback] { callback->OnInputStreamLengthReady(self, -1); });
+
+      if (aEventTarget) {
+        aEventTarget->Dispatch(runnable, NS_DISPATCH_NORMAL);
+      } else {
+        runnable->Run();
+      }
+    }
+    return NS_OK;
   }
 
   nsCOMPtr<nsIInputStreamLengthCallback> callback = aCallback ? this : nullptr;
@@ -821,10 +840,7 @@ NS_INTERFACE_MAP_BEGIN(nsBufferedOutputStream)
   NS_INTERFACE_MAP_ENTRY(nsIStreamBufferAccess)
 NS_INTERFACE_MAP_END_INHERITING(nsBufferedStream)
 
-nsresult nsBufferedOutputStream::Create(nsISupports* aOuter, REFNSIID aIID,
-                                        void** aResult) {
-  NS_ENSURE_NO_AGGREGATION(aOuter);
-
+nsresult nsBufferedOutputStream::Create(REFNSIID aIID, void** aResult) {
   RefPtr<nsBufferedOutputStream> stream = new nsBufferedOutputStream();
   return stream->QueryInterface(aIID, aResult);
 }
@@ -897,6 +913,7 @@ nsBufferedOutputStream::Write(const char* buf, uint32_t count,
     return NS_BASE_STREAM_CLOSED;
   }
 
+  RecursiveMutexAutoLock lock(mBufferMutex);
   while (count > 0) {
     uint32_t amt = std::min(count, mBufferSize - mCursor);
     if (amt > 0) {
@@ -933,6 +950,7 @@ nsBufferedOutputStream::Flush() {
   if (mFillPoint == 0) {
     return NS_OK;
   }
+  RecursiveMutexAutoLock lock(mBufferMutex);
   rv = Sink()->Write(mBuffer, mFillPoint, &amt);
   if (NS_FAILED(rv)) {
     return rv;
@@ -996,17 +1014,10 @@ nsBufferedOutputStream::Finish() {
   return NS_OK;
 }
 
-static nsresult nsReadFromInputStream(nsIOutputStream* outStr, void* closure,
-                                      char* toRawSegment, uint32_t offset,
-                                      uint32_t count, uint32_t* readCount) {
-  nsIInputStream* fromStream = (nsIInputStream*)closure;
-  return fromStream->Read(toRawSegment, count, readCount);
-}
-
 NS_IMETHODIMP
 nsBufferedOutputStream::WriteFrom(nsIInputStream* inStr, uint32_t count,
                                   uint32_t* _retval) {
-  return WriteSegments(nsReadFromInputStream, inStr, count, _retval);
+  return WriteSegments(NS_CopyStreamToSegment, inStr, count, _retval);
 }
 
 NS_IMETHODIMP
@@ -1014,6 +1025,7 @@ nsBufferedOutputStream::WriteSegments(nsReadSegmentFun reader, void* closure,
                                       uint32_t count, uint32_t* _retval) {
   *_retval = 0;
   nsresult rv;
+  RecursiveMutexAutoLock lock(mBufferMutex);
   while (count > 0) {
     uint32_t left = std::min(count, mBufferSize - mCursor);
     if (left == 0) {
@@ -1058,6 +1070,7 @@ nsBufferedOutputStream::GetBuffer(uint32_t aLength, uint32_t aAlignMask) {
     return nullptr;
   }
 
+  RecursiveMutexAutoLock lock(mBufferMutex);
   char* buf = mBuffer + mCursor;
   uint32_t rem = mBufferSize - mCursor;
   if (rem == 0) {
