@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsAppDirectoryServiceDefs.h"
 #include "nsCharSeparatedTokenizer.h"
 #include "nsComponentManagerUtils.h"
 #include "nsDirectoryServiceUtils.h"
+#include "nsHttpConnectionInfo.h"
 #include "nsICaptivePortalService.h"
 #include "nsIFile.h"
 #include "nsIParentalControlsService.h"
@@ -23,14 +23,10 @@
 #include "mozilla/Telemetry.h"
 #include "mozilla/TelemetryComms.h"
 #include "mozilla/Tokenizer.h"
-#include "mozilla/net/rust_helper.h"
+#include "mozilla/net/TRRServiceChild.h"
 // Put DNSLogging.h at the end to avoid LOG being overwritten by other headers.
 #include "DNSLogging.h"
 
-#if defined(XP_WIN) && !defined(__MINGW32__)
-#  include <shlobj_core.h>  // for SHGetSpecialFolderPathA
-#endif                      // XP_WIN
-
 static const char kOpenCaptivePortalLoginEvent[] = "captive-portal-login";
 static const char kClearPrivateData[] = "clear-private-data";
 static const char kPurge[] = "browser:purge-session-history";
@@ -39,10 +35,8 @@ static const char kDisableIpv6Pref[] = "network.dns.disableIPv6";
 #define TRR_PREF_PREFIX "network.trr."
 #define TRR_PREF(x) TRR_PREF_PREFIX x
 
-namespace mozilla {
-namespace net {
+namespace mozilla::net {
 
-TRRService* gTRRService = nullptr;
 StaticRefPtr<nsIThread> sTRRBackgroundThread;
 static Atomic<TRRService*> sTRRServicePtr;
 
@@ -53,21 +47,30 @@ constexpr nsLiteralCString kTRRDomains[] = {
     "(other)"_ns,
     "mozilla.cloudflare-dns.com"_ns,
     "firefox.dns.nextdns.io"_ns,
+    "private.canadianshield.cira.ca"_ns,
     "doh.xfinity.com"_ns,  // Steered clients
+    "dns.shaw.ca"_ns, // Steered clients
     // clang-format on
 };
 
 // static
 const nsCString& TRRService::ProviderKey() { return kTRRDomains[sDomainIndex]; }
 
-NS_IMPL_ISUPPORTS(TRRService, nsIObserver, nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS_INHERITED(TRRService, TRRServiceBase, nsIObserver,
+                            nsISupportsWeakReference)
 
 NS_IMPL_ADDREF_USING_AGGREGATOR(TRRService::ConfirmationContext, OwningObject())
 NS_IMPL_RELEASE_USING_AGGREGATOR(TRRService::ConfirmationContext,
                                  OwningObject())
-NS_IMPL_QUERY_INTERFACE(TRRService::ConfirmationContext, nsITimerCallback)
+NS_IMPL_QUERY_INTERFACE(TRRService::ConfirmationContext, nsITimerCallback,
+                        nsINamed)
 
-TRRService::TRRService() { MOZ_ASSERT(NS_IsMainThread(), "wrong thread"); }
+TRRService::TRRService() : mLock("TRRService", this) {
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+}
+
+// static
+TRRService* TRRService::Get() { return sTRRServicePtr; }
 
 // static
 void TRRService::AddObserver(nsIObserver* aObserver,
@@ -112,32 +115,6 @@ bool TRRService::CheckCaptivePortalIsPassed() {
   return result;
 }
 
-static void RemoveTRRBlocklistFile() {
-  MOZ_ASSERT(NS_IsMainThread(), "Getting the profile dir on the main thread");
-
-  nsCOMPtr<nsIFile> file;
-  nsresult rv =
-      NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(file));
-  if (NS_FAILED(rv)) {
-    return;
-  }
-
-  rv = file->AppendNative("TRRBlacklist.txt"_ns);
-  if (NS_FAILED(rv)) {
-    return;
-  }
-
-  // Dispatch an async task that removes the blocklist file from the profile.
-  rv = NS_DispatchBackgroundTask(
-      NS_NewRunnableFunction("RemoveTRRBlocklistFile::Remove",
-                             [file] { file->Remove(false); }),
-      NS_DISPATCH_EVENT_MAY_BLOCK);
-  if (NS_FAILED(rv)) {
-    return;
-  }
-  Preferences::SetBool("network.trr.blocklist_cleanup_done", true);
-}
-
 static void EventTelemetryPrefChanged(const char* aPref, void* aData) {
   Telemetry::SetEventRecordingEnabled(
       "network.dns"_ns,
@@ -162,7 +139,6 @@ nsresult TRRService::Init() {
     prefBranch->AddObserver(kRolloutModePref, this, true);
   }
 
-  gTRRService = this;
   sTRRServicePtr = this;
 
   ReadPrefs(nullptr);
@@ -188,15 +164,6 @@ nsresult TRRService::Init() {
     }
 
     sTRRBackgroundThread = thread;
-
-    if (!StaticPrefs::network_trr_blocklist_cleanup_done()) {
-      // Dispatch an idle task to the main thread that gets the profile dir
-      // then attempts to delete the blocklist file on a background thread.
-      Unused << NS_DispatchToMainThreadQueue(
-          NS_NewCancelableRunnableFunction("RemoveTRRBlocklistFile::GetDir",
-                                           [] { RemoveTRRBlocklistFile(); }),
-          EventQueuePriority::Idle);
-    }
   }
 
   mODoHService = new ODoHService();
@@ -229,9 +196,9 @@ bool TRRService::GetParentalControlEnabledInternal() {
 void TRRService::SetDetectedTrrURI(const nsACString& aURI) {
   LOG(("SetDetectedTrrURI(%s", nsPromiseFlatCString(aURI).get()));
   // If the user has set a custom URI then we don't want to override that.
-  // If the URI is set via doh-rollout.uri, mURIPrefHasUserValue will be false
+  // If the URI is set via doh-rollout.uri, mURIPref will be empty
   // (see TRRServiceBase::OnTRRURIChange)
-  if (mURIPrefHasUserValue) {
+  if (!mURIPref.IsEmpty()) {
     LOG(("Already has user value. Not setting URI"));
     return;
   }
@@ -297,7 +264,7 @@ bool TRRService::MaybeSetPrivateURI(const nsACString& aURI) {
 
   ProcessURITemplate(newURI);
   {
-    MutexAutoLock lock(mLock);
+    MutexSingleWriterAutoLock lock(mLock);
     if (mPrivateURI.Equals(newURI)) {
       return false;
     }
@@ -310,12 +277,11 @@ bool TRRService::MaybeSetPrivateURI(const nsACString& aURI) {
     }
 
     nsCOMPtr<nsIURI> url;
-    nsresult rv =
-        NS_MutateURI(NS_STANDARDURLMUTATOR_CONTRACTID)
-            .Apply(NS_MutatorMethod(&nsIStandardURLMutator::Init,
-                                    nsIStandardURL::URLTYPE_STANDARD, 443,
-                                    newURI, nullptr, nullptr, nullptr))
-            .Finalize(url);
+    nsresult rv = NS_MutateURI(NS_STANDARDURLMUTATOR_CONTRACTID)
+                      .Apply(&nsIStandardURLMutator::Init,
+                             nsIStandardURL::URLTYPE_STANDARD, 443, newURI,
+                             nullptr, nullptr, nullptr)
+                      .Finalize(url);
     if (NS_FAILED(rv)) {
       LOG(("TRRService::MaybeSetPrivateURI failed to create URI!\n"));
       return false;
@@ -334,10 +300,13 @@ bool TRRService::MaybeSetPrivateURI(const nsACString& aURI) {
 
     mPrivateURI = newURI;
 
+    AsyncCreateTRRConnectionInfo(mPrivateURI);
+
     // The URI has changed. We should trigger a new confirmation immediately.
     // We must do this here because the URI could also change because of
     // steering.
-    mConfirmation.HandleEvent(ConfirmationEvent::URIChange, lock);
+    mConfirmationTriggered =
+        mConfirmation.HandleEvent(ConfirmationEvent::URIChange, lock);
   }
 
   // Clear the cache because we changed the URI
@@ -377,27 +346,19 @@ nsresult TRRService::ReadPrefs(const char* name) {
     OnTRRURIChange();
   }
   if (!name || !strcmp(name, TRR_PREF("credentials"))) {
-    MutexAutoLock lock(mLock);
+    MutexSingleWriterAutoLock lock(mLock);
     Preferences::GetCString(TRR_PREF("credentials"), mPrivateCred);
   }
   if (!name || !strcmp(name, TRR_PREF("confirmationNS"))) {
-    MutexAutoLock lock(mLock);
+    MutexSingleWriterAutoLock lock(mLock);
     Preferences::GetCString(TRR_PREF("confirmationNS"), mConfirmationNS);
     LOG(("confirmationNS = %s", mConfirmationNS.get()));
   }
   if (!name || !strcmp(name, TRR_PREF("bootstrapAddr"))) {
-    MutexAutoLock lock(mLock);
+    MutexSingleWriterAutoLock lock(mLock);
     Preferences::GetCString(TRR_PREF("bootstrapAddr"), mBootstrapAddr);
     clearEntireCache = true;
   }
-  if (!name || !strcmp(name, TRR_PREF("blacklist-duration"))) {
-    // prefs is given in number of seconds
-    uint32_t secs;
-    if (NS_SUCCEEDED(
-            Preferences::GetUint(TRR_PREF("blacklist-duration"), &secs))) {
-      mBlocklistDurationSeconds = secs;
-    }
-  }
   if (!name || !strcmp(name, kDisableIpv6Pref)) {
     bool tmp;
     if (NS_SUCCEEDED(Preferences::GetBool(kDisableIpv6Pref, &tmp))) {
@@ -406,11 +367,12 @@ nsresult TRRService::ReadPrefs(const char* name) {
   }
   if (!name || !strcmp(name, TRR_PREF("excluded-domains")) ||
       !strcmp(name, TRR_PREF("builtin-excluded-domains"))) {
-    MutexAutoLock lock(mLock);
+    MutexSingleWriterAutoLock lock(mLock);
     mExcludedDomains.Clear();
 
     auto parseExcludedDomains = [this](const char* aPrefName) {
       nsAutoCString excludedDomains;
+      mLock.AssertCurrentThreadOwns();
       Preferences::GetCString(aPrefName, excludedDomains);
       if (excludedDomains.IsEmpty()) {
         return;
@@ -453,7 +415,7 @@ void TRRService::ClearEntireCache() {
 }
 
 void TRRService::AddEtcHosts(const nsTArray<nsCString>& aArray) {
-  MutexAutoLock lock(mLock);
+  MutexSingleWriterAutoLock lock(mLock);
   for (const auto& item : aArray) {
     LOG(("Adding %s from /etc/hosts to excluded domains", item.get()));
     mEtcHostsDomains.Insert(item);
@@ -461,58 +423,26 @@ void TRRService::AddEtcHosts(const nsTArray<nsCString>& aArray) {
 }
 
 void TRRService::ReadEtcHostsFile() {
-  if (!StaticPrefs::network_trr_exclude_etc_hosts()) {
+  if (!XRE_IsParentProcess()) {
     return;
   }
 
-  auto readHostsTask = []() {
-    MOZ_ASSERT(!NS_IsMainThread(), "Must not run on the main thread");
-#if defined(XP_WIN) && !defined(__MINGW32__)
-    // Inspired by libevent/evdns.c
-    // Windows is a little coy about where it puts its configuration
-    // files.  Sure, they're _usually_ in C:\windows\system32, but
-    // there's no reason in principle they couldn't be in
-    // W:\hoboken chicken emergency
-
-    nsCString path;
-    path.SetLength(MAX_PATH + 1);
-    if (!SHGetSpecialFolderPathA(NULL, path.BeginWriting(), CSIDL_SYSTEM,
-                                 false)) {
-      LOG(("Calling SHGetSpecialFolderPathA failed"));
-      return;
+  DoReadEtcHostsFile([](const nsTArray<nsCString>* aArray) -> bool {
+    RefPtr<TRRService> service(sTRRServicePtr);
+    if (service && aArray) {
+      service->AddEtcHosts(*aArray);
     }
-
-    path.SetLength(strlen(path.get()));
-    path.Append("\\drivers\\etc\\hosts");
-#elif defined(__MINGW32__)
-    nsAutoCString path("C:\\windows\\system32\\drivers\\etc\\hosts"_ns);
-#else
-    nsAutoCString path("/etc/hosts"_ns);
-#endif
-
-    LOG(("Reading hosts file at %s", path.get()));
-    rust_parse_etc_hosts(&path, [](const nsTArray<nsCString>* aArray) -> bool {
-      RefPtr<TRRService> service(sTRRServicePtr);
-      if (service && aArray) {
-        service->AddEtcHosts(*aArray);
-      }
-      return !!service;
-    });
-  };
-
-  Unused << NS_DispatchBackgroundTask(
-      NS_NewRunnableFunction("Read /etc/hosts file", readHostsTask),
-      NS_DISPATCH_EVENT_MAY_BLOCK);
+    return !!service;
+  });
 }
 
-nsresult TRRService::GetURI(nsACString& result) {
-  MutexAutoLock lock(mLock);
+void TRRService::GetURI(nsACString& result) {
+  MutexSingleWriterAutoLock lock(mLock);
   result = mPrivateURI;
-  return NS_OK;
 }
 
 nsresult TRRService::GetCredentials(nsCString& result) {
-  MutexAutoLock lock(mLock);
+  MutexSingleWriterAutoLock lock(mLock);
   result = mPrivateCred;
   return NS_OK;
 }
@@ -522,6 +452,10 @@ uint32_t TRRService::GetRequestTimeout() {
     return StaticPrefs::network_trr_request_timeout_mode_trronly_ms();
   }
 
+  if (StaticPrefs::network_trr_strict_native_fallback()) {
+    return StaticPrefs::network_trr_strict_fallback_request_timeout_ms();
+  }
+
   return StaticPrefs::network_trr_request_timeout_ms();
 }
 
@@ -536,7 +470,6 @@ nsresult TRRService::Start() {
 TRRService::~TRRService() {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   LOG(("Exiting TRRService\n"));
-  gTRRService = nullptr;
 }
 
 nsresult TRRService::DispatchTRRRequest(TRR* aTrrRequest) {
@@ -567,7 +500,7 @@ already_AddRefed<nsIThread> TRRService::MainThreadOrTRRThread(bool aWithLock) {
 }
 
 already_AddRefed<nsIThread> TRRService::TRRThread() {
-  MutexAutoLock lock(mLock);
+  MutexSingleWriterAutoLock lock(mLock);
   return TRRThread_locked();
 }
 
@@ -579,7 +512,7 @@ already_AddRefed<nsIThread> TRRService::TRRThread_locked() {
 bool TRRService::IsOnTRRThread() {
   nsCOMPtr<nsIThread> thread;
   {
-    MutexAutoLock lock(mLock);
+    MutexSingleWriterAutoLock lock(mLock);
     thread = sTRRBackgroundThread;
   }
   if (!thread) {
@@ -595,14 +528,18 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   LOG(("TRR::Observe() topic=%s\n", aTopic));
   if (!strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
-    auto prevConf = mConfirmation.TaskAddr();
-
+    // Reset the state of whether a confirmation is triggered, so we can check
+    // if we create a new one after ReadPrefs().
+    mConfirmationTriggered = false;
     ReadPrefs(NS_ConvertUTF16toUTF8(aData).get());
-    mConfirmation.RecordEvent("pref-change");
+    {
+      MutexSingleWriterAutoLock lock(mLock);
+      mConfirmation.RecordEvent("pref-change", lock);
+    }
 
     // We should only trigger a new confirmation if reading the prefs didn't
     // already trigger one.
-    if (prevConf == mConfirmation.TaskAddr()) {
+    if (!mConfirmationTriggered) {
       mConfirmation.HandleEvent(ConfirmationEvent::PrefChange);
     }
   } else if (!strcmp(aTopic, kOpenCaptivePortalLoginEvent)) {
@@ -648,7 +585,8 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
     if (!strcmp(aTopic, NS_NETWORK_LINK_TOPIC)) {
       if (NS_ConvertUTF16toUTF8(aData).EqualsLiteral(
               NS_NETWORK_LINK_DATA_DOWN)) {
-        mConfirmation.RecordEvent("network-change");
+        MutexSingleWriterAutoLock lock(mLock);
+        mConfirmation.RecordEvent("network-change", lock);
       }
 
       if (mURISetByDetection) {
@@ -662,18 +600,19 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
       }
     }
   } else if (!strcmp(aTopic, "xpcom-shutdown-threads")) {
+    mShutdown = true;
     // If a confirmation is still in progress we record the event.
     // Since there should be no more confirmations after this, the shutdown
     // reason would not really be recorded in telemetry.
-    mConfirmation.RecordEvent("shutdown");
+    {
+      MutexSingleWriterAutoLock lock(mLock);
+      mConfirmation.RecordEvent("shutdown", lock);
+    }
 
     if (sTRRBackgroundThread) {
       nsCOMPtr<nsIThread> thread;
-      {
-        MutexAutoLock lock(mLock);
-        thread = sTRRBackgroundThread.get();
-        sTRRBackgroundThread = nullptr;
-      }
+      thread = sTRRBackgroundThread.get();
+      sTRRBackgroundThread = nullptr;
       MOZ_ALWAYS_SUCCEEDS(thread->Shutdown());
       sTRRServicePtr = nullptr;
     }
@@ -682,11 +621,11 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
 }
 
 void TRRService::RebuildSuffixList(nsTArray<nsCString>&& aSuffixList) {
-  if (!StaticPrefs::network_trr_split_horizon_mitigations()) {
+  if (!StaticPrefs::network_trr_split_horizon_mitigations() || mShutdown) {
     return;
   }
 
-  MutexAutoLock lock(mLock);
+  MutexSingleWriterAutoLock lock(mLock);
   mDNSSuffixDomains.Clear();
   for (const auto& item : aSuffixList) {
     LOG(("TRRService adding %s to suffix list", item.get()));
@@ -694,18 +633,39 @@ void TRRService::RebuildSuffixList(nsTArray<nsCString>&& aSuffixList) {
   }
 }
 
-void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent) {
-  MutexAutoLock lock(OwningObject()->mLock);
-  HandleEvent(aEvent, lock);
+void TRRService::ConfirmationContext::SetState(
+    enum ConfirmationState aNewState) {
+  mState = aNewState;
+
+  if (XRE_IsParentProcess()) {
+    return;
+  }
+
+  MOZ_ASSERT(XRE_IsSocketProcess());
+  MOZ_ASSERT(NS_IsMainThread());
+
+  TRRServiceChild* child = TRRServiceChild::GetSingleton();
+  if (child && child->CanSend()) {
+    LOG(("TRRService::SendSetConfirmationState"));
+    Unused << child->SendSetConfirmationState(mState);
+  }
 }
 
-void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
-                                                  const MutexAutoLock&) {
+bool TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent) {
+  MutexSingleWriterAutoLock lock(OwningObject()->mLock);
+  return HandleEvent(aEvent, lock);
+}
+
+// We're protected by service->mLock
+bool TRRService::ConfirmationContext::HandleEvent(
+    ConfirmationEvent aEvent, const MutexSingleWriterAutoLock&) {
+  auto prevAddr = TaskAddr();
   TRRService* service = OwningObject();
   service->mLock.AssertCurrentThreadOwns();
   nsIDNSService::ResolverMode mode = service->Mode();
 
   auto resetConfirmation = [&]() {
+    service->mLock.AssertCurrentThreadOwns();
     mTask = nullptr;
     nsCOMPtr<nsITimer> timer = std::move(mTimer);
     if (timer) {
@@ -717,29 +677,30 @@ void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
 
     if (TRR_DISABLED(mode)) {
       LOG(("TRR is disabled. mConfirmation.mState -> CONFIRM_OFF"));
-      mState = CONFIRM_OFF;
+      SetState(CONFIRM_OFF);
       return;
     }
 
     if (mode == nsIDNSService::MODE_TRRONLY) {
       LOG(("TRR_ONLY_MODE. mConfirmation.mState -> CONFIRM_DISABLED"));
-      mState = CONFIRM_DISABLED;
+      SetState(CONFIRM_DISABLED);
       return;
     }
 
     if (service->mConfirmationNS.Equals("skip"_ns)) {
       LOG((
           "mConfirmationNS == skip. mConfirmation.mState -> CONFIRM_DISABLED"));
-      mState = CONFIRM_DISABLED;
+      SetState(CONFIRM_DISABLED);
       return;
     }
 
     // The next call to maybeConfirm will transition to CONFIRM_TRYING_OK
     LOG(("mConfirmation.mState -> CONFIRM_OK"));
-    mState = CONFIRM_OK;
+    SetState(CONFIRM_OK);
   };
 
   auto maybeConfirm = [&](const char* aReason) {
+    service->mLock.AssertCurrentThreadOwns();
     if (TRR_DISABLED(mode) || mState == CONFIRM_DISABLED || mTask) {
       LOG(
           ("TRRService:MaybeConfirm(%s) mode=%d, mTask=%p "
@@ -760,10 +721,10 @@ void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
 
     if (mState == CONFIRM_FAILED) {
       LOG(("mConfirmation.mState -> CONFIRM_TRYING_FAILED"));
-      mState = CONFIRM_TRYING_FAILED;
+      SetState(CONFIRM_TRYING_FAILED);
     } else {
       LOG(("mConfirmation.mState -> CONFIRM_TRYING_OK"));
-      mState = CONFIRM_TRYING_OK;
+      SetState(CONFIRM_TRYING_OK);
     }
 
     nsCOMPtr<nsITimer> timer = std::move(mTimer);
@@ -773,8 +734,9 @@ void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
 
     MOZ_ASSERT(mode == nsIDNSService::MODE_TRRFIRST,
                "Should only confirm in TRR first mode");
-    mTask =
-        new TRR(service, service->mConfirmationNS, TRRTYPE_NS, ""_ns, false);
+    // Set aUseFreshConnection if we are in strict fallback mode.
+    mTask = new TRR(service, service->mConfirmationNS, TRRTYPE_NS, ""_ns, false,
+                    StaticPrefs::network_trr_strict_native_fallback());
     mTask->SetTimeout(StaticPrefs::network_trr_confirmation_timeout_ms());
     mTask->SetPurpose(TRR::Confirmation);
 
@@ -810,8 +772,15 @@ void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
       break;
     case ConfirmationEvent::FailedLookups:
       MOZ_ASSERT(mState == CONFIRM_OK);
+      mTrigger.Assign("failed-lookups");
+      mFailedLookups = nsDependentCSubstring(
+          mFailureReasons, mTRRFailures % ConfirmationContext::RESULTS_SIZE);
       maybeConfirm("failed-lookups");
       break;
+    case ConfirmationEvent::StrictMode:
+      MOZ_ASSERT(mState == CONFIRM_OK);
+      maybeConfirm("strict-mode");
+      break;
     case ConfirmationEvent::URIChange:
       resetConfirmation();
       maybeConfirm("uri-change");
@@ -833,13 +802,13 @@ void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
       }
       break;
     case ConfirmationEvent::ConfirmOK:
-      mState = CONFIRM_OK;
+      SetState(CONFIRM_OK);
       mTask = nullptr;
       break;
     case ConfirmationEvent::ConfirmFail:
       MOZ_ASSERT(mState == CONFIRM_TRYING_OK ||
                  mState == CONFIRM_TRYING_FAILED);
-      mState = CONFIRM_FAILED;
+      SetState(CONFIRM_FAILED);
       mTask = nullptr;
       // retry failed NS confirmation
 
@@ -853,11 +822,13 @@ void TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
     default:
       MOZ_ASSERT_UNREACHABLE("Unexpected ConfirmationEvent");
   }
+
+  return prevAddr != TaskAddr();
 }
 
 bool TRRService::MaybeBootstrap(const nsACString& aPossible,
                                 nsACString& aResult) {
-  MutexAutoLock lock(mLock);
+  MutexSingleWriterAutoLock lock(mLock);
   if (mMode == nsIDNSService::MODE_TRROFF || mBootstrapAddr.IsEmpty()) {
     return false;
   }
@@ -865,9 +836,8 @@ bool TRRService::MaybeBootstrap(const nsACString& aPossible,
   nsCOMPtr<nsIURI> url;
   nsresult rv =
       NS_MutateURI(NS_STANDARDURLMUTATOR_CONTRACTID)
-          .Apply(NS_MutatorMethod(&nsIStandardURLMutator::Init,
-                                  nsIStandardURL::URLTYPE_STANDARD, 443,
-                                  mPrivateURI, nullptr, nullptr, nullptr))
+          .Apply(&nsIStandardURLMutator::Init, nsIStandardURL::URLTYPE_STANDARD,
+                 443, mPrivateURI, nullptr, nullptr, nullptr)
           .Finalize(url);
   if (NS_FAILED(rv)) {
     LOG(("TRRService::MaybeBootstrap failed to create URI!\n"));
@@ -896,7 +866,8 @@ bool TRRService::IsDomainBlocked(const nsACString& aHost,
   // use a unified casing for the hashkey
   nsAutoCString hashkey(aHost + aOriginSuffix);
   if (auto val = bl->Lookup(hashkey)) {
-    int32_t until = *val + mBlocklistDurationSeconds;
+    int32_t until =
+        *val + int32_t(StaticPrefs::network_trr_temp_blocklist_duration_sec());
     int32_t expire = NowInSeconds();
     if (until > expire) {
       LOG(("Host [%s] is TRR blocklisted\n", nsCString(aHost).get()));
@@ -916,6 +887,11 @@ bool TRRService::IsTemporarilyBlocked(const nsACString& aHost,
                                       bool aPrivateBrowsing,
                                       bool aParentsToo)  // false if domain
 {
+  if (!StaticPrefs::network_trr_temp_blocklist()) {
+    LOG(("TRRService::IsTemporarilyBlocked temp blocklist disabled by pref"));
+    return false;
+  }
+
   if (mMode == nsIDNSService::MODE_TRRONLY) {
     return false;  // might as well try
   }
@@ -952,15 +928,13 @@ bool TRRService::IsExcludedFromTRR(const nsACString& aHost) {
   // This method may be called off the main thread. We need to lock so
   // mExcludedDomains and mDNSSuffixDomains don't change while this code
   // is running.
-  MutexAutoLock lock(mLock);
+  MutexSingleWriterAutoLock lock(mLock);
 
   return IsExcludedFromTRR_unlocked(aHost);
 }
 
 bool TRRService::IsExcludedFromTRR_unlocked(const nsACString& aHost) {
-  if (!NS_IsMainThread()) {
-    mLock.AssertCurrentThreadOwns();
-  }
+  mLock.AssertOnWritingThreadOrHeld();
 
   int32_t dot = 0;
   // iteratively check the sub-domain of |aHost|
@@ -997,6 +971,11 @@ bool TRRService::IsExcludedFromTRR_unlocked(const nsACString& aHost) {
 void TRRService::AddToBlocklist(const nsACString& aHost,
                                 const nsACString& aOriginSuffix,
                                 bool privateBrowsing, bool aParentsToo) {
+  if (!StaticPrefs::network_trr_temp_blocklist()) {
+    LOG(("TRRService::AddToBlocklist temp blocklist disabled by pref"));
+    return;
+  }
+
   LOG(("TRR blocklist %s\n", nsCString(aHost).get()));
   nsAutoCString hashkey(aHost + aOriginSuffix);
 
@@ -1026,8 +1005,8 @@ void TRRService::AddToBlocklist(const nsACString& aHost,
       LOG(("TRR: verify if '%s' resolves as NS\n", check.get()));
 
       // check if there's an NS entry for this name
-      RefPtr<TRR> trr =
-          new TRR(this, check, TRRTYPE_NS, aOriginSuffix, privateBrowsing);
+      RefPtr<TRR> trr = new TRR(this, check, TRRTYPE_NS, aOriginSuffix,
+                                privateBrowsing, false);
       trr->SetPurpose(TRR::Blocklist);
       DispatchTRRRequest(trr);
     }
@@ -1036,13 +1015,17 @@ void TRRService::AddToBlocklist(const nsACString& aHost,
 
 NS_IMETHODIMP
 TRRService::ConfirmationContext::Notify(nsITimer* aTimer) {
-  MutexAutoLock lock(OwningObject()->mLock);
+  MutexSingleWriterAutoLock lock(OwningObject()->mLock);
   if (aTimer == mTimer) {
     HandleEvent(ConfirmationEvent::Retry, lock);
-    return NS_OK;
   }
 
-  MOZ_CRASH("Unknown timer");
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+TRRService::ConfirmationContext::GetName(nsACString& aName) {
+  aName.AssignLiteral("TRRService::ConfirmationContext");
   return NS_OK;
 }
 
@@ -1085,6 +1068,13 @@ static char StatusToChar(nsresult aLookupStatus, nsresult aChannelStatus) {
   return '?';
 }
 
+void TRRService::StrictModeConfirm() {
+  if (mConfirmation.State() == CONFIRM_OK) {
+    LOG(("TRRService::StrictModeConfirm triggering confirmation"));
+    mConfirmation.HandleEvent(ConfirmationEvent::StrictMode);
+  }
+}
+
 void TRRService::RecordTRRStatus(nsresult aChannelStatus) {
   MOZ_ASSERT_IF(XRE_IsParentProcess(), NS_IsMainThread() || IsOnTRRThread());
   MOZ_ASSERT_IF(XRE_IsSocketProcess(), NS_IsMainThread());
@@ -1115,6 +1105,15 @@ void TRRService::ConfirmationContext::RecordTRRStatus(nsresult aChannelStatus) {
     return;
   }
 
+  // In strict mode, nsHostResolver will trigger Confirmation immediately
+  // upon a lookup failure, so nothing to be done here. nsHostResolver
+  // can assess the success of the lookup considering all the involved
+  // results (A, AAAA) so we let it tell us when to re-Confirm.
+  if (StaticPrefs::network_trr_strict_native_fallback()) {
+    LOG(("TRRService not counting failures in strict mode"));
+    return;
+  }
+
   mFailureReasons[mTRRFailures % ConfirmationContext::RESULTS_SIZE] =
       StatusToChar(NS_OK, aChannelStatus);
   uint32_t fails = ++mTRRFailures;
@@ -1127,17 +1126,14 @@ void TRRService::ConfirmationContext::RecordTRRStatus(nsresult aChannelStatus) {
     // Only after the confirmation fails do we finally go into CONFIRM_FAILED
     // and start skipping TRR.
 
-    mTrigger.Assign("failed-lookups");
-    mFailedLookups = nsDependentCSubstring(
-        mFailureReasons, fails % ConfirmationContext::RESULTS_SIZE);
-
     // Trigger a confirmation immediately.
     // If it fails, it will fire off a timer to start retrying again.
     HandleEvent(ConfirmationEvent::FailedLookups);
   }
 }
 
-void TRRService::ConfirmationContext::RecordEvent(const char* aReason) {
+void TRRService::ConfirmationContext::RecordEvent(
+    const char* aReason, const MutexSingleWriterAutoLock&) {
   // Reset the confirmation context attributes
   // Only resets the attributes that we keep for telemetry purposes.
   auto reset = [&]() {
@@ -1222,7 +1218,7 @@ void TRRService::ConfirmationContext::RequestCompleted(
 void TRRService::ConfirmationContext::CompleteConfirmation(nsresult aStatus,
                                                            TRR* aTRRRequest) {
   {
-    MutexAutoLock lock(OwningObject()->mLock);
+    MutexSingleWriterAutoLock lock(OwningObject()->mLock);
     // Ignore confirmations that dont match the pending task.
     if (mTask != aTRRRequest) {
       return;
@@ -1242,14 +1238,15 @@ void TRRService::ConfirmationContext::CompleteConfirmation(nsresult aStatus,
       HandleEvent(ConfirmationEvent::ConfirmFail, lock);
     }
 
+    if (State() == CONFIRM_OK) {
+      // Record event and start new confirmation context
+      RecordEvent("success", lock);
+    }
     LOG(("TRRService finishing confirmation test %s %d %X\n",
          OwningObject()->mPrivateURI.get(), State(), (unsigned int)aStatus));
   }
 
   if (State() == CONFIRM_OK) {
-    // Record event and start new confirmation context
-    RecordEvent("success");
-
     // A fresh confirmation means previous blocked entries might not
     // be valid anymore.
     auto bl = OwningObject()->mTRRBLStorage.Lock();
@@ -1301,5 +1298,30 @@ AHostResolver::LookupStatus TRRService::CompleteLookupByType(
   return LOOKUP_OK;
 }
 
-}  // namespace net
-}  // namespace mozilla
+NS_IMETHODIMP TRRService::OnProxyConfigChanged() {
+  LOG(("TRRService::OnProxyConfigChanged"));
+
+  nsAutoCString uri;
+  GetURI(uri);
+  AsyncCreateTRRConnectionInfo(uri);
+
+  return NS_OK;
+}
+
+void TRRService::InitTRRConnectionInfo() {
+  if (XRE_IsParentProcess()) {
+    TRRServiceBase::InitTRRConnectionInfo();
+    return;
+  }
+
+  MOZ_ASSERT(XRE_IsSocketProcess());
+  MOZ_ASSERT(NS_IsMainThread());
+
+  TRRServiceChild* child = TRRServiceChild::GetSingleton();
+  if (child && child->CanSend()) {
+    LOG(("TRRService::SendInitTRRConnectionInfo"));
+    Unused << child->SendInitTRRConnectionInfo();
+  }
+}
+
+}  // namespace mozilla::net
