@@ -35,6 +35,14 @@ bool DocumentChannelParent::Init(dom::CanonicalBrowsingContext* aContext,
       new nsDocShellLoadState(aArgs.loadState());
   LOG(("DocumentChannelParent Init [this=%p, uri=%s]", this,
        loadState->URI()->GetSpecOrDefault().get()));
+  if (aArgs.parentInitiatedNavigationEpoch() <
+      aContext->GetParentInitiatedNavigationEpoch()) {
+    nsresult rv = NS_BINDING_ABORTED;
+    return SendFailedAsyncOpen(rv);
+  }
+
+  ContentParent* contentParent =
+      static_cast<ContentParent*>(Manager()->Manager());
 
   RefPtr<DocumentLoadListener::OpenPromise> promise;
   if (loadState->GetChannelInitialized()) {
@@ -62,7 +70,7 @@ bool DocumentChannelParent::Init(dom::CanonicalBrowsingContext* aContext,
           loadState, aArgs.cacheKey(), Some(aArgs.channelId()),
           aArgs.asyncOpenTime(), aArgs.timing().refOr(nullptr),
           std::move(clientInfo), Some(docArgs.uriModified()),
-          Some(docArgs.isXFOError()), IProtocol::OtherPid(), &rv);
+          Some(docArgs.isXFOError()), contentParent, &rv);
     } else {
       const ObjectCreationArgs& objectArgs = aArgs.elementCreationArgs();
 
@@ -71,7 +79,7 @@ bool DocumentChannelParent::Init(dom::CanonicalBrowsingContext* aContext,
           aArgs.asyncOpenTime(), aArgs.timing().refOr(nullptr),
           std::move(clientInfo), objectArgs.embedderInnerWindowId(),
           objectArgs.loadFlags(), objectArgs.contentPolicyType(),
-          objectArgs.isUrgentStart(), IProtocol::OtherPid(),
+          objectArgs.isUrgentStart(), contentParent,
           this /* ObjectUpgradeHandler */, &rv);
     }
 
@@ -100,7 +108,7 @@ bool DocumentChannelParent::Init(dom::CanonicalBrowsingContext* aContext,
         if (self->CanSend()) {
           Unused << self->SendDisconnectChildListeners(
               aRejectValue.mStatus, aRejectValue.mLoadGroupStatus,
-              aRejectValue.mSwitchedProcess);
+              aRejectValue.mContinueNavigating);
         }
         self->mDocumentLoadListener = nullptr;
       });
