@@ -22,11 +22,17 @@
 #include "nsStringStream.h"
 #include "nsNetCID.h"
 #include "nsNetUtil.h"
+#include "nsQueryObject.h"
+#include "nsComponentManagerUtils.h"
+#include "nsServiceManagerUtils.h"
+#include "nsIPrefService.h"
 
 #include <algorithm>
 
 #define MAX_BUFFER_SIZE 512u
 
+using namespace mozilla;
+
 NS_IMPL_ISUPPORTS(nsUnknownDecoder::ConvertedStreamListener, nsIStreamListener,
                   nsIRequestObserver)
 
@@ -446,7 +452,7 @@ void nsUnknownDecoder::DetermineContentType(nsIRequest* aRequest) {
     }
     if (!decodedData.IsEmpty()) {
       testData = decodedData.get();
-      testDataLen = std::min(decodedData.Length(), MAX_BUFFER_SIZE);
+      testDataLen = std::min<uint32_t>(decodedData.Length(), MAX_BUFFER_SIZE);
     }
   }
 
@@ -541,7 +547,8 @@ bool nsUnknownDecoder::SniffForHTML(nsIRequest* aRequest) {
     end = mBuffer + mBufferLen;
   } else {
     str = mDecodedData.get();
-    end = mDecodedData.get() + std::min(mDecodedData.Length(), MAX_BUFFER_SIZE);
+    end = mDecodedData.get() +
+          std::min<uint32_t>(mDecodedData.Length(), MAX_BUFFER_SIZE);
   }
 
   // skip leading whitespace
@@ -651,7 +658,7 @@ bool nsUnknownDecoder::LastDitchSniff(nsIRequest* aRequest) {
     testDataLen = std::min<uint32_t>(mBufferLen, MAX_BUFFER_SIZE);
   } else {
     testData = mDecodedData.get();
-    testDataLen = std::min(mDecodedData.Length(), MAX_BUFFER_SIZE);
+    testDataLen = std::min<uint32_t>(mDecodedData.Length(), MAX_BUFFER_SIZE);
   }
 
   // First, check for a BOM.  If we see one, assume this is text/plain
