@@ -6,6 +6,7 @@ var EXPORTED_SYMBOLS = [
   "EngineManager",
   "SyncEngine",
   "Tracker",
+  "LegacyTracker",
   "Store",
   "Changeset",
 ];
@@ -38,7 +39,7 @@ const { SerializableSet, Svc, Utils } = ChromeUtils.import(
 XPCOMUtils.defineLazyModuleGetters(this, {
   fxAccounts: "resource://gre/modules/FxAccounts.jsm",
   OS: "resource://gre/modules/osfile.jsm",
-  PlacesSyncUtils: "resource://gre/modules/PlacesSyncUtils.jsm",
+  PlacesUtils: "resource://gre/modules/PlacesUtils.jsm",
 });
 
 function ensureDirectory(path) {
@@ -46,16 +47,15 @@ function ensureDirectory(path) {
   return OS.File.makeDir(basename, { from: OS.Constants.Path.profileDir });
 }
 
-/*
+/**
  * Trackers are associated with a single engine and deal with
  * listening for changes to their particular data type.
  *
- * There are two things they keep track of:
- * 1) A score, indicating how urgently the engine wants to sync
- * 2) A list of IDs for all the changed items that need to be synced
- * and updating their 'score', indicating how urgently they
- * want to sync.
- *
+ * The base `Tracker` only supports listening for changes, and bumping the score
+ * to indicate how urgently the engine wants to sync. It does not persist any
+ * data. Engines that track changes directly in the storage layer (like
+ * bookmarks, bridged engines, addresses, and credit cards) or only upload a
+ * single record (tabs and preferences) should subclass `Tracker`.
  */
 function Tracker(name, engine) {
   if (!engine) {
@@ -63,24 +63,28 @@ function Tracker(name, engine) {
   }
 
   name = name || "Unnamed";
-  this.name = this.file = name.toLowerCase();
+  this.name = name.toLowerCase();
   this.engine = engine;
 
   this._log = Log.repository.getLogger(`Sync.Engine.${name}.Tracker`);
 
   this._score = 0;
-  this._ignored = [];
-  this._storage = new JSONFile({
-    path: Utils.jsonFilePath("changes/" + this.file),
-    dataPostProcessor: json => this._dataPostProcessor(json),
-    beforeSave: () => this._beforeSave(),
-  });
-  this.ignoreAll = false;
 
   this.asyncObserver = Async.asyncObserver(this, this._log);
 }
 
 Tracker.prototype = {
+  // New-style trackers use change sources to filter out changes made by Sync in
+  // observer notifications, so we don't want to let the engine ignore all
+  // changes during a sync.
+  get ignoreAll() {
+    return false;
+  },
+
+  // Define an empty setter so that the engine doesn't throw a `TypeError`
+  // setting a read-only property.
+  set ignoreAll(value) {},
+
   /*
    * Score can be called as often as desired to decide which engines to sync
    *
@@ -95,16 +99,6 @@ Tracker.prototype = {
     return this._score;
   },
 
-  // Default to an empty object if the file doesn't exist.
-  _dataPostProcessor(json) {
-    return (typeof json == "object" && json) || {};
-  },
-
-  // Ensure the Weave storage directory exists before writing the file.
-  _beforeSave() {
-    return ensureDirectory(this._storage.path);
-  },
-
   set score(value) {
     this._score = value;
     Observers.notify("weave:engine:score:updated", this.name);
@@ -115,7 +109,131 @@ Tracker.prototype = {
     this._score = 0;
   },
 
-  persistChangedIDs: true,
+  // Unsupported, and throws a more descriptive error to ensure callers aren't
+  // accidentally using persistence.
+  async getChangedIDs() {
+    throw new TypeError("This tracker doesn't store changed IDs");
+  },
+
+  // Also unsupported.
+  async addChangedID(id, when) {
+    throw new TypeError("Can't add changed ID to this tracker");
+  },
+
+  // Ditto.
+  async removeChangedID(...ids) {
+    throw new TypeError("Can't remove changed IDs from this tracker");
+  },
+
+  // This method is called at various times, so we override with a no-op
+  // instead of throwing.
+  clearChangedIDs() {},
+
+  _now() {
+    return Date.now() / 1000;
+  },
+
+  _isTracking: false,
+
+  start() {
+    if (!this.engineIsEnabled()) {
+      return;
+    }
+    this._log.trace("start().");
+    if (!this._isTracking) {
+      this.onStart();
+      this._isTracking = true;
+    }
+  },
+
+  async stop() {
+    this._log.trace("stop().");
+    if (this._isTracking) {
+      await this.asyncObserver.promiseObserversComplete();
+      this.onStop();
+      this._isTracking = false;
+    }
+  },
+
+  // Override these in your subclasses.
+  onStart() {},
+  onStop() {},
+  async observe(subject, topic, data) {},
+
+  engineIsEnabled() {
+    if (!this.engine) {
+      // Can't tell -- we must be running in a test!
+      return true;
+    }
+    return this.engine.enabled;
+  },
+
+  /**
+   * Starts or stops listening for changes depending on the associated engine's
+   * enabled state.
+   *
+   * @param {Boolean} engineEnabled Whether the engine was enabled.
+   */
+  async onEngineEnabledChanged(engineEnabled) {
+    if (engineEnabled == this._isTracking) {
+      return;
+    }
+
+    if (engineEnabled) {
+      this.start();
+    } else {
+      await this.stop();
+      this.clearChangedIDs();
+    }
+  },
+
+  async finalize() {
+    await this.stop();
+  },
+};
+
+/*
+ * A tracker that persists a list of IDs for all changed items that need to be
+ * synced. This is 🚨 _extremely deprecated_ 🚨 and only kept around for current
+ * engines. ⚠️ Please **don't use it** for new engines! ⚠️
+ *
+ * Why is this kind of external change tracking deprecated? Because it causes
+ * consistency issues due to missed notifications, interrupted syncs, and the
+ * tracker's view of what changed diverging from the data store's.
+ */
+function LegacyTracker(name, engine) {
+  Tracker.call(this, name, engine);
+
+  this._ignored = [];
+  this.file = this.name;
+  this._storage = new JSONFile({
+    path: Utils.jsonFilePath("changes", this.file),
+    dataPostProcessor: json => this._dataPostProcessor(json),
+    beforeSave: () => this._beforeSave(),
+  });
+  this._ignoreAll = false;
+}
+
+LegacyTracker.prototype = {
+  __proto__: Tracker.prototype,
+
+  get ignoreAll() {
+    return this._ignoreAll;
+  },
+
+  set ignoreAll(value) {
+    this._ignoreAll = value;
+  },
+
+  // Default to an empty object if the file doesn't exist.
+  _dataPostProcessor(json) {
+    return (typeof json == "object" && json) || {};
+  },
+
+  // Ensure the Weave storage directory exists before writing the file.
+  _beforeSave() {
+    return ensureDirectory(this._storage.path);
+  },
 
   async getChangedIDs() {
     await this._storage.load();
@@ -123,10 +241,6 @@ Tracker.prototype = {
   },
 
   _saveChangedIDs() {
-    if (!this.persistChangedIDs) {
-      this._log.debug("Not saving changedIDs.");
-      return;
-    }
     this._storage.saveSoon();
   },
 
@@ -196,72 +310,20 @@ Tracker.prototype = {
         delete changedIDs[id];
       }
     }
-    await this._saveChangedIDs();
+    this._saveChangedIDs();
     return true;
   },
 
-  async clearChangedIDs() {
+  clearChangedIDs() {
     this._log.trace("Clearing changed ID list");
     this._storage.data = {};
-    await this._saveChangedIDs();
-  },
-
-  _now() {
-    return Date.now() / 1000;
-  },
-
-  _isTracking: false,
-
-  start() {
-    if (!this.engineIsEnabled()) {
-      return;
-    }
-    this._log.trace("start().");
-    if (!this._isTracking) {
-      this.onStart();
-      this._isTracking = true;
-    }
-  },
-
-  async stop() {
-    this._log.trace("stop().");
-    if (this._isTracking) {
-      await this.asyncObserver.promiseObserversComplete();
-      this.onStop();
-      this._isTracking = false;
-    }
-  },
-
-  // Override these in your subclasses.
-  onStart() {},
-  onStop() {},
-  async observe(subject, topic, data) {},
-
-  engineIsEnabled() {
-    if (!this.engine) {
-      // Can't tell -- we must be running in a test!
-      return true;
-    }
-    return this.engine.enabled;
-  },
-
-  async onEngineEnabledChanged(engineEnabled) {
-    if (engineEnabled == this._isTracking) {
-      return;
-    }
-
-    if (engineEnabled) {
-      this.start();
-    } else {
-      await this.stop();
-      await this.clearChangedIDs();
-    }
+    this._saveChangedIDs();
   },
 
   async finalize() {
     // Persist all pending tracked changes to disk, and wait for the final write
     // to finish.
-    await this.stop();
+    await super.finalize();
     this._saveChangedIDs();
     await this._storage.finalize();
   },
@@ -535,9 +597,7 @@ EngineManager.prototype = {
       }
       // Unregister the old engine, register the new one.
       this._log.info(
-        `Switching ${name} engine ("${info.pref}" went from ${
-          info.lastValue
-        } => ${prefValue})`
+        `Switching ${name} engine ("${info.pref}" went from ${info.lastValue} => ${prefValue})`
       );
       try {
         await this._removeAndFinalize(name);
@@ -554,9 +614,7 @@ EngineManager.prototype = {
         this._log.info(`Switched the ${name} engine to use ${engineType.name}`);
       } catch (e) {
         this._log.warn(
-          `Switching the ${name} engine to use ${
-            engineType.name
-          } failed (couldn't register)`,
+          `Switching the ${name} engine to use ${engineType.name} failed (couldn't register)`,
           e
         );
       }
@@ -625,21 +683,6 @@ EngineManager.prototype = {
     this.persistDeclined();
   },
 
-  /**
-   * Mark any non-enabled engines as declined.
-   *
-   * This is useful after initial customization during setup.
-   */
-  declineDisabled() {
-    for (let e of this.getAll()) {
-      if (!e.enabled) {
-        this._log.debug("Declining disabled engine " + e.name);
-        this._declined.add(e.name);
-      }
-    }
-    this.persistDeclined();
-  },
-
   /**
    * Register an Engine to the service. Alternatively, give an array of engine
    * objects to register.
@@ -724,13 +767,13 @@ function SyncEngine(name, service) {
   this._log.debug("Engine constructed");
 
   this._toFetchStorage = new JSONFile({
-    path: Utils.jsonFilePath("toFetch/" + this.name),
+    path: Utils.jsonFilePath("toFetch", this.name),
     dataPostProcessor: json => this._metadataPostProcessor(json),
     beforeSave: () => this._beforeSaveMetadata(),
   });
 
   this._previousFailedStorage = new JSONFile({
-    path: Utils.jsonFilePath("failed/" + this.name),
+    path: Utils.jsonFilePath("failed", this.name),
     dataPostProcessor: json => this._metadataPostProcessor(json),
     beforeSave: () => this._beforeSaveMetadata(),
   });
@@ -739,10 +782,7 @@ function SyncEngine(name, service) {
     this,
     "_enabled",
     `services.sync.engine.${this.prefName}`,
-    false,
-    (data, previous, latest) =>
-      // We do not await on the promise onEngineEnabledChanged returns.
-      this._tracker.onEngineEnabledChanged(latest)
+    false
   );
   XPCOMUtils.defineLazyPreferenceGetter(
     this,
@@ -760,33 +800,7 @@ function SyncEngine(name, service) {
   );
   // Async initializations can be made in the initialize() method.
 
-  // The map of ids => metadata for records needing a weak upload.
-  //
-  // Currently the "metadata" fields are:
-  //
-  // - forceTombstone: whether or not we should ignore the local information
-  //   about the record, and write a tombstone for it anyway -- e.g. in the case
-  //   of records that should exist locally, but should never be uploaded to the
-  //   server (note that not all sync engines support tombstones)
-  //
-  // The difference between this and a "normal" upload is that these records
-  // are only tracked in memory, and if the upload attempt fails (shutdown,
-  // 412, etc), we abort uploading the "weak" set (by clearing the map).
-  //
-  // The rationale here is for the cases where we receive a record from the
-  // server that we know is wrong in some (small) way. For example, the
-  // dateAdded field on bookmarks -- maybe we have a better date, or the server
-  // record is entirely missing the date, etc.
-  //
-  // In these cases, we fix our local copy of the record, and mark it for
-  // weak upload. A normal ("strong") upload is problematic here because
-  // in the case of a conflict from the server, there's a window where our
-  // record would be marked as modified more recently than a change that occurs
-  // on another device change, and we lose data from the user.
-  //
-  // Additionally, we use this as the set of items to upload for bookmark
-  // repair reponse, which has similar constraints.
-  this._needWeakUpload = new Map();
+  this.asyncObserver = Async.asyncObserver(this, this._log);
 }
 
 // Enumeration to define approaches to handling bad records.
@@ -859,6 +873,7 @@ SyncEngine.prototype = {
   async initialize() {
     await this._toFetchStorage.load();
     await this._previousFailedStorage.load();
+    Svc.Prefs.observe(`engine.${this.prefName}`, this.asyncObserver);
     this._log.debug("SyncEngine initialized", this.name);
   },
 
@@ -917,6 +932,18 @@ SyncEngine.prototype = {
     return this._tracker.stop();
   },
 
+  // Listens for engine enabled state changes, and updates the tracker's state.
+  // This is an async observer because the tracker waits on all its async
+  // observers to finish when it's stopped.
+  async observe(subject, topic, data) {
+    if (
+      topic == "nsPref:changed" &&
+      data == `services.sync.engine.${this.prefName}`
+    ) {
+      await this._tracker.onEngineEnabledChanged(this._enabled);
+    }
+  },
+
   async sync() {
     if (!this.enabled) {
       return false;
@@ -996,6 +1023,17 @@ SyncEngine.prototype = {
     return this.ensureCurrentSyncID(Utils.makeGUID());
   },
 
+  /**
+   * Allows overriding scheduler logic -- added to help reduce kinto server
+   * getting hammered because our scheduler never got tuned for it.
+   *
+   * Note: Overriding engines must take resyncs into account -- score will not
+   * be cleared.
+   */
+  shouldSkipSync(syncReason) {
+    return false;
+  },
+
   /*
    * lastSync is a timestamp in server time.
    */
@@ -1074,10 +1112,6 @@ SyncEngine.prototype = {
     return tombstone;
   },
 
-  addForWeakUpload(id, { forceTombstone = false } = {}) {
-    this._needWeakUpload.set(id, { forceTombstone });
-  },
-
   // Any setup that needs to happen at the beginning of each sync.
   async _syncStartup() {
     // Determine if we need to wipe on outdated versions
@@ -1125,13 +1159,9 @@ SyncEngine.prototype = {
     this._modified.replace(initialChanges);
     // Clear the tracker now. If the sync fails we'll add the ones we failed
     // to upload back.
-    await this._tracker.clearChangedIDs();
+    this._tracker.clearChangedIDs();
     this._tracker.resetScore();
 
-    this._log.info(
-      this._modified.count() + " outgoing items pre-reconciliation"
-    );
-
     // Keep track of what to delete at the end of sync
     this._delete = {};
   },
@@ -1310,52 +1340,54 @@ SyncEngine.prototype = {
 
     // `getBatched` includes the list of IDs as a query parameter, so we need to fetch
     // records in chunks to avoid exceeding URI length limits.
-    for (let [, ids] of PlacesSyncUtils.chunkArray(
-      idsToBackfill,
-      this.guidFetchBatchSize
-    )) {
-      backfilledItems.ids = ids;
+    if (this.guidFetchBatchSize) {
+      for (let ids of PlacesUtils.chunkArray(
+        idsToBackfill,
+        this.guidFetchBatchSize
+      )) {
+        backfilledItems.ids = ids;
 
-      let { response, records } = await backfilledItems.getBatched(
-        this.downloadBatchSize
-      );
-      if (!response.success) {
-        response.failureCode = ENGINE_DOWNLOAD_FAIL;
-        throw response;
-      }
-
-      let backfilledRecordsToApply = [];
-      let failedInBackfill = [];
-
-      await Async.yieldingForEach(records, async record => {
-        let { shouldApply, error } = await this._maybeReconcile(record);
-        if (error) {
-          failedInBackfill.push(record.id);
-          count.failed++;
-          return;
+        let { response, records } = await backfilledItems.getBatched(
+          this.downloadBatchSize
+        );
+        if (!response.success) {
+          response.failureCode = ENGINE_DOWNLOAD_FAIL;
+          throw response;
         }
-        if (!shouldApply) {
-          count.reconciled++;
-          return;
+
+        let backfilledRecordsToApply = [];
+        let failedInBackfill = [];
+
+        await Async.yieldingForEach(records, async record => {
+          let { shouldApply, error } = await this._maybeReconcile(record);
+          if (error) {
+            failedInBackfill.push(record.id);
+            count.failed++;
+            return;
+          }
+          if (!shouldApply) {
+            count.reconciled++;
+            return;
+          }
+          backfilledRecordsToApply.push(record);
+        });
+
+        let failedToApply = await this._applyRecords(backfilledRecordsToApply);
+        failedInBackfill.push(...failedToApply);
+
+        count.failed += failedToApply.length;
+        count.applied += backfilledRecordsToApply.length;
+
+        this.toFetch = Utils.setDeleteAll(this.toFetch, ids);
+        this.previousFailed = Utils.setAddAll(
+          this.previousFailed,
+          failedInBackfill
+        );
+
+        if (lastSync < this.lastModified) {
+          lastSync = this.lastModified;
+          await this.setLastSync(lastSync);
         }
-        backfilledRecordsToApply.push(record);
-      });
-
-      let failedToApply = await this._applyRecords(backfilledRecordsToApply);
-      failedInBackfill.push(...failedToApply);
-
-      count.failed += failedToApply.length;
-      count.applied += backfilledRecordsToApply.length;
-
-      this.toFetch = Utils.setDeleteAll(this.toFetch, ids);
-      this.previousFailed = Utils.setAddAll(
-        this.previousFailed,
-        failedInBackfill
-      );
-
-      if (lastSync < this.lastModified) {
-        lastSync = this.lastModified;
-        await this.setLastSync(lastSync);
       }
     }
 
@@ -1767,13 +1799,9 @@ SyncEngine.prototype = {
     // collection we'll upload
     let up = new Collection(this.engineURL, null, this.service);
     let modifiedIDs = new Set(this._modified.ids());
-    for (let id of this._needWeakUpload.keys()) {
-      modifiedIDs.add(id);
-    }
     let counts = { failed: 0, sent: 0 };
+    this._log.info(`Uploading ${modifiedIDs.size} outgoing records`);
     if (modifiedIDs.size) {
-      this._log.trace("Preparing " + modifiedIDs.size + " outgoing records");
-
       counts.sent = modifiedIDs.size;
 
       let failed = [];
@@ -1784,7 +1812,7 @@ SyncEngine.prototype = {
         // the batch is complete, however we want to remember success/failure
         // indicators for when that happens.
         if (!resp.success) {
-          this._log.debug("Uploading records failed: " + resp);
+          this._log.debug(`Uploading records failed: ${resp.status}`);
           resp.failureCode =
             resp.status == 412 ? ENGINE_BATCH_INTERRUPTED : ENGINE_UPLOAD_FAIL;
           throw resp;
@@ -1836,12 +1864,7 @@ SyncEngine.prototype = {
         let out;
         let ok = false;
         try {
-          let { forceTombstone = false } = this._needWeakUpload.get(id) || {};
-          if (forceTombstone) {
-            out = await this._createTombstone(id);
-          } else {
-            out = await this._createRecord(id);
-          }
+          out = await this._createRecord(id);
           if (this._log.level <= Log.Level.Trace) {
             this._log.trace("Outgoing: " + out);
           }
@@ -1883,7 +1906,6 @@ SyncEngine.prototype = {
       }
       await postQueue.flush(true);
     }
-    this._needWeakUpload.clear();
 
     if (counts.sent || counts.failed) {
       Observers.notify("weave:engine:sync:uploaded", counts, this.name);
@@ -1927,11 +1949,6 @@ SyncEngine.prototype = {
   },
 
   async _syncCleanup() {
-    this._needWeakUpload.clear();
-    if (!this._modified) {
-      return;
-    }
-
     try {
       // Mark failed WBOs as changed again so they are reuploaded next time.
       await this.trackRemainingChanges();
@@ -2103,7 +2120,6 @@ SyncEngine.prototype = {
     this.hasSyncedThisSession = false;
     this.previousFailed = new SerializableSet();
     this.toFetch = new SerializableSet();
-    this._needWeakUpload.clear();
   },
 
   /**
@@ -2119,7 +2135,7 @@ SyncEngine.prototype = {
     this._tracker.ignoreAll = true;
     await this._store.wipe();
     this._tracker.ignoreAll = false;
-    await this._tracker.clearChangedIDs();
+    this._tracker.clearChangedIDs();
   },
 
   /**
@@ -2132,10 +2148,17 @@ SyncEngine.prototype = {
   },
 
   async finalize() {
+    Svc.Prefs.ignore(`engine.${this.prefName}`, this.asyncObserver);
+    await this.asyncObserver.promiseObserversComplete();
     await this._tracker.finalize();
     await this._toFetchStorage.finalize();
     await this._previousFailedStorage.finalize();
   },
+
+  // Returns a new watchdog. Exposed for tests.
+  _newWatchdog() {
+    return Async.watchdog();
+  },
 };
 
 /**
