@@ -56,7 +56,7 @@ ChromeUtils.defineModuleGetter(
   "resource://gre/modules/AddonManager.jsm"
 );
 
-add_task(async function head_setup() {
+add_setup(async function head_setup() {
   // Initialize logging. This will sometimes be reset by a pref reset,
   // so it's also called as part of SyncTestingInfrastructure().
   syncTestLogging();
@@ -67,23 +67,19 @@ add_task(async function head_setup() {
 });
 
 XPCOMUtils.defineLazyGetter(this, "SyncPingSchema", function() {
-  let ns = {};
-  ChromeUtils.import("resource://gre/modules/FileUtils.jsm", ns);
-  ChromeUtils.import("resource://gre/modules/NetUtil.jsm", ns);
+  let { FileUtils } = ChromeUtils.import(
+    "resource://gre/modules/FileUtils.jsm"
+  );
+  let { NetUtil } = ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
   let stream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(
     Ci.nsIFileInputStream
   );
   let schema;
   try {
     let schemaFile = do_get_file("sync_ping_schema.json");
-    stream.init(
-      schemaFile,
-      ns.FileUtils.MODE_RDONLY,
-      ns.FileUtils.PERMS_FILE,
-      0
-    );
+    stream.init(schemaFile, FileUtils.MODE_RDONLY, FileUtils.PERMS_FILE, 0);
 
-    let bytes = ns.NetUtil.readInputStream(stream, stream.available());
+    let bytes = NetUtil.readInputStream(stream, stream.available());
     schema = JSON.parse(new TextDecoder().decode(bytes));
   } finally {
     stream.close();
@@ -96,10 +92,10 @@ XPCOMUtils.defineLazyGetter(this, "SyncPingSchema", function() {
 });
 
 XPCOMUtils.defineLazyGetter(this, "SyncPingValidator", function() {
-  let ns = {};
-  ChromeUtils.import("resource://testing-common/ajv-4.1.1.js", ns);
-  let ajv = new ns.Ajv({ async: "co*" });
-  return ajv.compile(SyncPingSchema);
+  const { JsonSchema } = ChromeUtils.import(
+    "resource://gre/modules/JsonSchema.jsm"
+  );
+  return new JsonSchema.Validator(SyncPingSchema);
 });
 
 // This is needed for loadAddonTestFunctions().
@@ -275,24 +271,31 @@ function mockGetWindowEnumerator(url, numWindows, numTabs, indexes, moreURLs) {
 // Helper function to get the sync telemetry and add the typically used test
 // engine names to its list of allowed engines.
 function get_sync_test_telemetry() {
-  let ns = {};
-  ChromeUtils.import("resource://services-sync/telemetry.js", ns);
-  let testEngines = ["rotary", "steam", "sterling", "catapult"];
+  let { SyncTelemetry } = ChromeUtils.import(
+    "resource://services-sync/telemetry.js"
+  );
+  SyncTelemetry.tryRefreshDevices = function() {};
+  let testEngines = ["rotary", "steam", "sterling", "catapult", "nineties"];
   for (let engineName of testEngines) {
-    ns.SyncTelemetry.allowedEngines.add(engineName);
+    SyncTelemetry.allowedEngines.add(engineName);
   }
-  ns.SyncTelemetry.submissionInterval = -1;
-  return ns.SyncTelemetry;
+  SyncTelemetry.submissionInterval = -1;
+  return SyncTelemetry;
 }
 
 function assert_valid_ping(record) {
+  // Our JSON validator does not like `undefined` values, even though they will
+  // be skipped when we serialize to JSON.
+  record = JSON.parse(JSON.stringify(record));
+
   // This is called as the test harness tears down due to shutdown. This
   // will typically have no recorded syncs, and the validator complains about
   // it. So ignore such records (but only ignore when *both* shutdown and
   // no Syncs - either of them not being true might be an actual problem)
   if (record && (record.why != "shutdown" || record.syncs.length != 0)) {
-    if (!SyncPingValidator(record)) {
-      if (SyncPingValidator.errors.length) {
+    const result = SyncPingValidator.validate(record);
+    if (!result.valid) {
+      if (result.errors.length) {
         // validation failed - using a simple |deepEqual([], errors)| tends to
         // truncate the validation errors in the output and doesn't show that
         // the ping actually was - so be helpful.
@@ -300,7 +303,7 @@ function assert_valid_ping(record) {
         info("the ping data is: " + JSON.stringify(record, undefined, 2));
         info(
           "the validation failures: " +
-            JSON.stringify(SyncPingValidator.errors, undefined, 2)
+            JSON.stringify(result.errors, undefined, 2)
         );
         ok(
           false,
@@ -311,14 +314,6 @@ function assert_valid_ping(record) {
     equal(record.version, 1);
     record.syncs.forEach(p => {
       lessOrEqual(p.when, Date.now());
-      if (p.devices) {
-        ok(!p.devices.some(device => device.id == record.deviceID));
-        equal(
-          new Set(p.devices.map(device => device.id)).size,
-          p.devices.length,
-          "Duplicate device ids in ping devices list"
-        );
-      }
     });
   }
 }
@@ -390,9 +385,47 @@ async function wait_for_ping(callback, allowErrorPings, getFullPing = false) {
   return record.syncs[0];
 }
 
-// Short helper for wait_for_ping
-function sync_and_validate_telem(allowErrorPings, getFullPing = false) {
-  return wait_for_ping(() => Service.sync(), allowErrorPings, getFullPing);
+// Perform a sync and validate all telemetry caused by the sync. If fnValidate
+// is null, we just check the ping records success. If fnValidate is specified,
+// then the sync must have recorded just a single sync, and that sync will be
+// passed to the function to be checked.
+async function sync_and_validate_telem(
+  fnValidate = null,
+  wantFullPing = false
+) {
+  let numErrors = 0;
+  let telem = get_sync_test_telemetry();
+  let oldSubmit = telem.submit;
+  try {
+    telem.submit = function(record) {
+      // This is called via an observer, so failures here don't cause the test
+      // to fail :(
+      try {
+        // All pings must be valid.
+        assert_valid_ping(record);
+        if (fnValidate) {
+          // for historical reasons most of these callbacks expect a "sync"
+          // record, not the entire ping.
+          if (wantFullPing) {
+            fnValidate(record);
+          } else {
+            Assert.equal(record.syncs.length, 1);
+            fnValidate(record.syncs[0]);
+          }
+        } else {
+          // no validation function means it must be a "success" ping.
+          assert_success_ping(record);
+        }
+      } catch (ex) {
+        print("Failure in ping validation callback", ex, "\n", ex.stack);
+        numErrors += 1;
+      }
+    };
+    await Service.sync();
+    Assert.ok(numErrors == 0, "There were telemetry validation errors");
+  } finally {
+    telem.submit = oldSubmit;
+  }
 }
 
 // Used for the (many) cases where we do a 'partial' sync, where only a single
@@ -402,16 +435,16 @@ function sync_and_validate_telem(allowErrorPings, getFullPing = false) {
 async function sync_engine_and_validate_telem(
   engine,
   allowErrorPings,
-  onError
+  onError,
+  wantFullPing = false
 ) {
   let telem = get_sync_test_telemetry();
   let caughtError = null;
   // Clear out status, so failures from previous syncs won't show up in the
   // telemetry ping.
-  let ns = {};
-  ChromeUtils.import("resource://services-sync/status.js", ns);
-  ns.Status._engines = {};
-  ns.Status.partial = false;
+  let { Status } = ChromeUtils.import("resource://services-sync/status.js");
+  Status._engines = {};
+  Status.partial = false;
   // Ideally we'd clear these out like we do with engines, (probably via
   // Status.resetSync()), but this causes *numerous* tests to fail, so we just
   // assume that if no failureReason or engine failures are set, and the
@@ -419,8 +452,8 @@ async function sync_engine_and_validate_telem(
   // a leftover.
   // This is only an issue since we're triggering the sync of just one engine,
   // without doing any other parts of the sync.
-  let initialServiceStatus = ns.Status._service;
-  let initialSyncStatus = ns.Status._sync;
+  let initialServiceStatus = Status._service;
+  let initialSyncStatus = Status._sync;
 
   let oldSubmit = telem.submit;
   let submitPromise = new Promise((resolve, reject) => {
@@ -469,6 +502,8 @@ async function sync_engine_and_validate_telem(
           onError(ping.syncs[0], ping);
         }
         reject(caughtError);
+      } else if (wantFullPing) {
+        resolve(ping);
       } else {
         resolve(ping.syncs[0]);
       }
@@ -509,15 +544,6 @@ function promiseOneObserver(topic, callback) {
   });
 }
 
-// Avoid an issue where `client.name2` containing unicode characters causes
-// a number of tests to fail, due to them assuming that we do not need to utf-8
-// encode or decode data sent through the mocked server (see bug 1268912).
-// We stash away the original implementation so test_utils_misc.js can test it.
-Utils._orig_getDefaultDeviceName = Utils.getDefaultDeviceName;
-Utils.getDefaultDeviceName = function() {
-  return "Test device name";
-};
-
 async function registerRotaryEngine() {
   let { RotaryEngine } = ChromeUtils.import(
     "resource://testing-common/services/sync/rotaryengine.js"
@@ -592,9 +618,8 @@ async function promiseVisit(expectedType, expectedURI) {
   return new Promise(resolve => {
     function done(type, uri) {
       if (uri == expectedURI.spec && type == expectedType) {
-        PlacesUtils.history.removeObserver(observer);
         PlacesObservers.removeListener(
-          ["page-visited"],
+          ["page-visited", "page-removed"],
           observer.handlePlacesEvents
         );
         resolve();
@@ -603,23 +628,19 @@ async function promiseVisit(expectedType, expectedURI) {
     let observer = {
       handlePlacesEvents(events) {
         Assert.equal(events.length, 1);
-        Assert.equal(events[0].type, "page-visited");
-        done("added", events[0].url);
+
+        if (events[0].type === "page-visited") {
+          done("added", events[0].url);
+        } else if (events[0].type === "page-removed") {
+          Assert.ok(events[0].isRemovedFromStore);
+          done("removed", events[0].url);
+        }
       },
-      onBeginUpdateBatch() {},
-      onEndUpdateBatch() {},
-      onTitleChanged() {},
-      onFrecencyChanged() {},
-      onManyFrecenciesChanged() {},
-      onDeleteURI(uri) {
-        done("removed", uri.spec);
-      },
-      onClearHistory() {},
-      onPageChanged() {},
-      onDeleteVisits() {},
     };
-    PlacesUtils.history.addObserver(observer, false);
-    PlacesObservers.addListener(["page-visited"], observer.handlePlacesEvents);
+    PlacesObservers.addListener(
+      ["page-visited", "page-removed"],
+      observer.handlePlacesEvents
+    );
   });
 }
 
@@ -653,18 +674,14 @@ function bookmarkNodesToInfos(nodes) {
     if (node.children) {
       info.children = bookmarkNodesToInfos(node.children);
     }
-    if (node.annos) {
-      let orphanAnno = node.annos.find(anno => anno.name == "sync/parent");
-      if (orphanAnno) {
-        info.requestedParent = orphanAnno.value;
-      }
-    }
     return info;
   });
 }
 
 async function assertBookmarksTreeMatches(rootGuid, expected, message) {
-  let root = await PlacesUtils.promiseBookmarksTree(rootGuid);
+  let root = await PlacesUtils.promiseBookmarksTree(rootGuid, {
+    includeItemIds: true,
+  });
   let actual = bookmarkNodesToInfos(root.children);
 
   if (!ObjectUtils.deepEqual(actual, expected)) {
@@ -674,9 +691,20 @@ async function assertBookmarksTreeMatches(rootGuid, expected, message) {
   }
 }
 
-function bufferedBookmarksEnabled() {
-  return Services.prefs.getBoolPref(
-    "services.sync.engine.bookmarks.buffer",
-    false
+function add_bookmark_test(task) {
+  const { BookmarksEngine } = ChromeUtils.import(
+    "resource://services-sync/engines/bookmarks.js"
   );
+
+  add_task(async function() {
+    _(`Running bookmarks test ${task.name}`);
+    let engine = new BookmarksEngine(Service);
+    await engine.initialize();
+    await engine._resetClient();
+    try {
+      await task(engine);
+    } finally {
+      await engine.finalize();
+    }
+  });
 }
