@@ -12,6 +12,7 @@
 #include "mozilla/dom/BrowsingContext.h"
 #include "mozilla/dom/CanonicalBrowsingContext.h"
 #include "mozilla/net/CookieJarSettings.h"
+#include "mozilla/LoadInfo.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/WindowGlobalParent.h"
 #include "mozilla/dom/WindowContext.h"
@@ -152,6 +153,69 @@ bool AntiTrackingUtils::IsStorageAccessPermission(nsIPermission* aPermission,
   return StringBeginsWith(type, permissionKey);
 }
 
+// static
+Maybe<size_t> AntiTrackingUtils::CountSitesAllowStorageAccess(
+    nsIPrincipal* aPrincipal) {
+  PermissionManager* permManager = PermissionManager::GetInstance();
+  if (NS_WARN_IF(!permManager)) {
+    return Nothing();
+  }
+
+  nsAutoCString prefix;
+  AntiTrackingUtils::CreateStoragePermissionKey(aPrincipal, prefix);
+
+  using Permissions = nsTArray<RefPtr<nsIPermission>>;
+  Permissions perms;
+  nsresult rv = permManager->GetAllWithTypePrefix(prefix, perms);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return Nothing();
+  }
+
+  // Create a set of unique origins
+  using Origins = nsTArray<nsCString>;
+  Origins origins;
+
+  // Iterate over all permissions that have a prefix equal to the expected
+  // permission we are looking for. This includes two things we need to remove:
+  // duplicates and origin strings that have a prefix of aPrincipal's origin
+  // string, e.g. https://example.company when aPrincipal is
+  // https://example.com.
+  for (const auto& perm : perms) {
+    nsAutoCString type;
+    rv = perm->GetType(type);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return Nothing();
+    }
+    // Let's make sure that we're not looking at a permission for
+    // https://exampletracker.company when we mean to look for the
+    // permission for https://exampletracker.com!
+    if (type != prefix) {
+      continue;
+    }
+
+    nsCOMPtr<nsIPrincipal> principal;
+    rv = perm->GetPrincipal(getter_AddRefs(principal));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return Nothing();
+    }
+
+    nsAutoCString origin;
+    rv = principal->GetOrigin(origin);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return Nothing();
+    }
+
+    ToLowerCase(origin);
+
+    // Append if it would not be a duplicate
+    if (origins.IndexOf(origin) == Origins::NoIndex) {
+      origins.AppendElement(origin);
+    }
+  }
+
+  return Some(origins.Length());
+}
+
 // static
 bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
                                                const nsAutoCString& aType,
@@ -255,8 +319,9 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
   return true;
 }
 
-/* static */ bool AntiTrackingUtils::HasStoragePermissionInParent(
-    nsIChannel* aChannel) {
+/* static */
+nsILoadInfo::StoragePermissionState
+AntiTrackingUtils::GetStoragePermissionStateInParent(nsIChannel* aChannel) {
   MOZ_ASSERT(aChannel);
   MOZ_DIAGNOSTIC_ASSERT(XRE_IsParentProcess());
 
@@ -269,13 +334,13 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
   // window should always has 'hasStoragePermission' flag as false. So, we can
   // return here directly.
   if (policyType == ExtContentPolicy::TYPE_DOCUMENT) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
   }
 
   nsresult rv =
       loadInfo->GetCookieJarSettings(getter_AddRefs(cookieJarSettings));
   if (NS_WARN_IF(NS_FAILED(rv))) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
   }
 
   int32_t cookieBehavior = cookieJarSettings->GetCookieBehavior();
@@ -286,13 +351,13 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
   // update or check the storage permission if the cookie behavior is not
   // belongs to these three.
   if (!net::CookieJarSettings::IsRejectThirdPartyContexts(cookieBehavior)) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
   }
 
   RefPtr<BrowsingContext> bc;
   rv = loadInfo->GetTargetBrowsingContext(getter_AddRefs(bc));
   if (NS_WARN_IF(NS_FAILED(rv)) || !bc) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
   }
 
   uint64_t targetWindowId = GetTopLevelAntiTrackingWindowId(bc);
@@ -303,7 +368,7 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
         WindowGlobalParent::GetByInnerWindowId(targetWindowId);
 
     if (NS_WARN_IF(!wgp)) {
-      return false;
+      return nsILoadInfo::NoStoragePermission;
     }
 
     targetPrincipal = wgp->DocumentPrincipal();
@@ -336,24 +401,30 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
 
   // Cannot get the target principal, bail out.
   if (NS_WARN_IF(!targetPrincipal)) {
-    return false;
-  }
-
-  nsAutoCString targetOrigin;
-  if (NS_FAILED(targetPrincipal->GetAsciiOrigin(targetOrigin))) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
   }
 
   nsCOMPtr<nsIURI> trackingURI;
   rv = aChannel->GetURI(getter_AddRefs(trackingURI));
   if (NS_WARN_IF(NS_FAILED(rv))) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
   }
 
   nsAutoCString trackingOrigin;
   rv = nsContentUtils::GetASCIIOrigin(trackingURI, trackingOrigin);
   if (NS_WARN_IF(NS_FAILED(rv))) {
-    return false;
+    return nsILoadInfo::NoStoragePermission;
+  }
+
+  if (IsThirdPartyChannel(aChannel)) {
+    nsAutoCString targetOrigin;
+    if (NS_FAILED(targetPrincipal->GetAsciiOrigin(targetOrigin))) {
+      return nsILoadInfo::NoStoragePermission;
+    }
+
+    if (PartitioningExceptionList::Check(targetOrigin, trackingOrigin)) {
+      return nsILoadInfo::StoragePermissionAllowListed;
+    }
   }
 
   nsAutoCString type;
@@ -361,13 +432,13 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
 
   uint32_t unusedReason = 0;
 
-  if (PartitioningExceptionList::Check(targetOrigin, trackingOrigin)) {
-    return true;
+  if (AntiTrackingUtils::CheckStoragePermission(targetPrincipal, type,
+                                                NS_UsePrivateBrowsing(aChannel),
+                                                &unusedReason, unusedReason)) {
+    return nsILoadInfo::HasStoragePermission;
   }
 
-  return AntiTrackingUtils::CheckStoragePermission(
-      targetPrincipal, type, NS_UsePrivateBrowsing(aChannel), &unusedReason,
-      unusedReason);
+  return nsILoadInfo::NoStoragePermission;
 }
 
 uint64_t AntiTrackingUtils::GetTopLevelAntiTrackingWindowId(
@@ -547,8 +618,20 @@ void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
   Unused << aChannel->GetURI(getter_AddRefs(uri));
 
   // In some cases we don't have a browsingContext. For example, in xpcshell
-  // tests or channels that are used to download images.
+  // tests, channels that are used to download images and channels for loading
+  // worker script.
   if (!bc) {
+    // If the flag was set before, we don't need to compute again. This could
+    // happen for the channels used to load worker scripts.
+    //
+    // Note that we cannot stop computing the flag in general even it has set
+    // before because sometimes we need to get the up-to-date flag, e.g.
+    // redirects.
+    if (static_cast<net::LoadInfo*>(loadInfo.get())
+            ->HasIsThirdPartyContextToTopWindowSet()) {
+      return;
+    }
+
     // We turn to check the loading principal if there is no browsing context.
     auto* loadingPrincipal =
         BasePrincipal::Cast(loadInfo->GetLoadingPrincipal());
@@ -577,11 +660,11 @@ void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
     auto* basePrin = BasePrincipal::Cast(topWindowPrincipal);
     bool isThirdParty = true;
 
-    // For about:blank, we can't just compare uri to determine whether the page
-    // is third-party, so we use channel result principal instead. By doing
-    // this, an about:blank inherits the principal from its parent is considered
-    // not a third-party.
-    if (NS_IsAboutBlank(uri)) {
+    // For about:blank and about:srcdoc, we can't just compare uri to determine
+    // whether the page is third-party, so we use channel result principal
+    // instead. By doing this, an the resource inherits the principal from
+    // its parent is considered not a third-party.
+    if (NS_IsAboutBlank(uri) || NS_IsAboutSrcdoc(uri)) {
       nsIScriptSecurityManager* ssm = nsContentUtils::GetSecurityManager();
       if (NS_WARN_IF(!ssm)) {
         return;
@@ -617,13 +700,13 @@ bool AntiTrackingUtils::IsThirdPartyChannel(nsIChannel* aChannel) {
 bool AntiTrackingUtils::IsThirdPartyWindow(nsPIDOMWindowInner* aWindow,
                                            nsIURI* aURI) {
   MOZ_ASSERT(aWindow);
-  MOZ_ASSERT(aURI);
 
   // We assume that the window is foreign to the URI by default.
   bool thirdParty = true;
 
-  // This is to comply with ThirdPartyUtil::IsThirdPartyWindow API.
-  if (aURI && !NS_IsAboutBlank(aURI)) {
+  // We will skip checking URIs for about:blank and about:srcdoc because they
+  // have no domain. So, comparing them will always fail.
+  if (aURI && !NS_IsAboutBlank(aURI) && !NS_IsAboutSrcdoc(aURI)) {
     nsCOMPtr<nsIScriptObjectPrincipal> scriptObjPrin =
         do_QueryInterface(aWindow);
     if (!scriptObjPrin) {
@@ -647,9 +730,9 @@ bool AntiTrackingUtils::IsThirdPartyWindow(nsPIDOMWindowInner* aWindow,
   }
 
   RefPtr<Document> doc = aWindow->GetDoc();
-  if (!doc || !doc->GetChannel()) {
-    // If we can't get channel from the window, ex, about:blank, fallback to use
-    // IsThirdPartyWindow check that examine the whole hierarchy.
+  if (!doc) {
+    // If we can't get the document from the window, ex, about:blank, fallback
+    // to use IsThirdPartyWindow check that examine the whole hierarchy.
     nsCOMPtr<mozIThirdPartyUtil> thirdPartyUtil =
         components::ThirdPartyUtil::Service();
     Unused << thirdPartyUtil->IsThirdPartyWindow(aWindow->GetOuterWindow(),
@@ -657,12 +740,69 @@ bool AntiTrackingUtils::IsThirdPartyWindow(nsPIDOMWindowInner* aWindow,
     return thirdParty;
   }
 
+  return IsThirdPartyDocument(doc);
+}
+
+/* static */
+bool AntiTrackingUtils::IsThirdPartyDocument(Document* aDocument) {
+  MOZ_ASSERT(aDocument);
+  if (!aDocument->GetChannel()) {
+    // If we can't get the channel from the document, i.e. initial about:blank
+    // page, we use the browsingContext of the document to check if it's in the
+    // third-party context. If the browsing context is still not available, we
+    // will treat the window as third-party.
+    RefPtr<BrowsingContext> bc = aDocument->GetBrowsingContext();
+    return bc ? IsThirdPartyContext(bc) : true;
+  }
+
   // We only care whether the channel is 3rd-party with respect to
   // the top-level.
-  nsCOMPtr<nsILoadInfo> loadInfo = doc->GetChannel()->LoadInfo();
+  nsCOMPtr<nsILoadInfo> loadInfo = aDocument->GetChannel()->LoadInfo();
   return loadInfo->GetIsThirdPartyContextToTopWindow();
 }
 
+/* static */
+bool AntiTrackingUtils::IsThirdPartyContext(BrowsingContext* aBrowsingContext) {
+  MOZ_ASSERT(aBrowsingContext);
+  MOZ_ASSERT(aBrowsingContext->IsInProcess());
+
+  if (aBrowsingContext->IsTopContent()) {
+    return false;
+  }
+
+  // If the top browsing context is not in the same process, it's cross-origin.
+  if (!aBrowsingContext->Top()->IsInProcess()) {
+    return true;
+  }
+
+  nsIDocShell* docShell = aBrowsingContext->GetDocShell();
+  if (!docShell) {
+    return true;
+  }
+  Document* doc = docShell->GetExtantDocument();
+  if (!doc) {
+    return true;
+  }
+  nsIPrincipal* principal = doc->NodePrincipal();
+
+  nsIDocShell* topDocShell = aBrowsingContext->Top()->GetDocShell();
+  if (!topDocShell) {
+    return true;
+  }
+  Document* topDoc = topDocShell->GetDocument();
+  if (!topDoc) {
+    return true;
+  }
+  nsIPrincipal* topPrincipal = topDoc->NodePrincipal();
+
+  auto* topBasePrin = BasePrincipal::Cast(topPrincipal);
+  bool isThirdParty = true;
+
+  topBasePrin->IsThirdPartyPrincipal(principal, &isThirdParty);
+
+  return isThirdParty;
+}
+
 /* static */
 nsCString AntiTrackingUtils::GrantedReasonToString(
     ContentBlockingNotifier::StorageAccessPermissionGrantedReason aReason) {
@@ -686,12 +826,12 @@ void AntiTrackingUtils::UpdateAntiTrackingInfoForChannel(nsIChannel* aChannel) {
 
   MOZ_DIAGNOSTIC_ASSERT(XRE_IsParentProcess());
 
+  AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(aChannel);
+
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
 
-  Unused << loadInfo->SetHasStoragePermission(
-      AntiTrackingUtils::HasStoragePermissionInParent(aChannel));
-
-  AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(aChannel);
+  Unused << loadInfo->SetStoragePermission(
+      AntiTrackingUtils::GetStoragePermissionStateInParent(aChannel));
 
   // We only update the IsOnContentBlockingAllowList flag and the partition key
   // for the top-level http channel.
