@@ -7,7 +7,7 @@
 #ifndef mozilla_RejectForeignAllowList_h
 #define mozilla_RejectForeignAllowList_h
 
-#include "nsIUrlClassifierSkipListService.h"
+#include "nsIUrlClassifierExceptionListService.h"
 
 class nsIHttpChannel;
 class nsIURI;
@@ -18,13 +18,15 @@ namespace dom {
 class Document;
 }
 
-class RejectForeignAllowList final : public nsIUrlClassifierSkipListObserver {
+class RejectForeignAllowList final
+    : public nsIUrlClassifierExceptionListObserver {
  public:
   NS_DECL_ISUPPORTS
-  NS_DECL_NSIURLCLASSIFIERSKIPLISTOBSERVER
+  NS_DECL_NSIURLCLASSIFIEREXCEPTIONLISTOBSERVER
 
   static bool Check(dom::Document* aDocument);
   static bool Check(nsIHttpChannel* aChannel);
+  static bool Check(nsIPrincipal* aPrincipal);
 
  private:
   static RejectForeignAllowList* GetOrCreate();
@@ -33,6 +35,7 @@ class RejectForeignAllowList final : public nsIUrlClassifierSkipListObserver {
   ~RejectForeignAllowList();
 
   bool CheckInternal(nsIURI* aURI);
+  bool CheckInternal(nsIPrincipal* aPrincipal);
 
   nsCString mList;
 };
