@@ -510,25 +510,33 @@ this.AntiTracking = {
     if (extraPrefs && Array.isArray(extraPrefs) && extraPrefs.length) {
       await SpecialPowers.pushPrefEnv({ set: extraPrefs });
 
-      for (let item of extraPrefs) {
-        // When setting up exception URLs, we need to wait to ensure our prefs
-        // actually take effect.  In order to do this, we set up a exception
-        // list observer and wait until it calls us back.
-        if (item[0] == "urlclassifier.trackingAnnotationSkipURLs") {
-          info("Waiting for the exception list service to initialize...");
-          let classifier = Cc[
-            "@mozilla.org/url-classifier/dbservice;1"
-          ].getService(Ci.nsIURIClassifier);
-          let feature = classifier.getFeatureByName("tracking-annotation");
-          await TestUtils.waitForCondition(() => {
-            for (let x of item[1].toLowerCase().split(",")) {
-              if (feature.exceptionHostList.split(",").includes(x)) {
-                return true;
+      let enableWebcompat = Services.prefs.getBoolPref(
+        "privacy.antitracking.enableWebcompat"
+      );
+
+      // If the skip list is disabled by pref, it will always return an empty
+      // list.
+      if (enableWebcompat) {
+        for (let item of extraPrefs) {
+          // When setting up exception URLs, we need to wait to ensure our prefs
+          // actually take effect.  In order to do this, we set up a exception
+          // list observer and wait until it calls us back.
+          if (item[0] == "urlclassifier.trackingAnnotationSkipURLs") {
+            info("Waiting for the exception list service to initialize...");
+            let classifier = Cc[
+              "@mozilla.org/url-classifier/dbservice;1"
+            ].getService(Ci.nsIURIClassifier);
+            let feature = classifier.getFeatureByName("tracking-annotation");
+            await TestUtils.waitForCondition(() => {
+              for (let x of item[1].toLowerCase().split(",")) {
+                if (feature.exceptionHostList.split(",").includes(x)) {
+                  return true;
+                }
               }
-            }
-            return false;
-          }, "Exception list service initialized");
-          break;
+              return false;
+            }, "Exception list service initialized");
+            break;
+          }
         }
       }
     }
