@@ -12,6 +12,7 @@
 #include "nsUnicharUtils.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIObserverService.h"
+#include "nsServiceManagerUtils.h"
 #include "mozilla/Services.h"
 #include "mozilla/Unused.h"
 #include "mozilla/dom/KeyboardEventBinding.h"
@@ -59,14 +60,10 @@ nsAutoCompleteController::nsAutoCompleteController()
 
 nsAutoCompleteController::~nsAutoCompleteController() { SetInput(nullptr); }
 
-void nsAutoCompleteController::SetValueOfInputTo(const nsString& aValue,
-                                                 uint16_t aReason) {
+void nsAutoCompleteController::SetValueOfInputTo(const nsString& aValue) {
   mSetValue = aValue;
   nsCOMPtr<nsIAutoCompleteInput> input(mInput);
-  nsresult rv = input->SetTextValueWithReason(aValue, aReason);
-  if (NS_FAILED(rv)) {
-    input->SetTextValue(aValue);
-  }
+  input->SetTextValue(aValue);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -460,14 +457,10 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
                              nsCaseInsensitiveStringComparator)) {
               start = mSearchString.Length();
               value = mPlaceholderCompletionString;
-              SetValueOfInputTo(
-                  value,
-                  nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+              SetValueOfInputTo(value);
             } else {
               start = value.Length();
-              SetValueOfInputTo(
-                  value,
-                  nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETESELECTED);
+              SetValueOfInputTo(value);
             }
 
             input->SelectTextRange(start, value.Length());
@@ -475,8 +468,7 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
           mCompletedSelectionIndex = selectedIndex;
         } else {
           // Nothing is selected, so fill in the last typed value
-          SetValueOfInputTo(mSearchString,
-                            nsIAutoCompleteInput::TEXTVALUE_REASON_REVERT);
+          SetValueOfInputTo(mSearchString);
           input->SelectTextRange(mSearchString.Length(),
                                  mSearchString.Length());
           mCompletedSelectionIndex = -1;
@@ -613,8 +605,7 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
           }
 
           if (value.Equals(suggestedValue, nsCaseInsensitiveStringComparator)) {
-            SetValueOfInputTo(
-                value, nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+            SetValueOfInputTo(value);
             input->SelectTextRange(value.Length(), value.Length());
           }
         }
@@ -623,8 +614,7 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
         // The pop-up is open and has a selection, take its value
         nsAutoString value;
         if (NS_SUCCEEDED(GetResultValueAt(selectedIndex, false, value))) {
-          SetValueOfInputTo(
-              value, nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETESELECTED);
+          SetValueOfInputTo(value);
           input->SelectTextRange(value.Length(), value.Length());
         }
       }
@@ -677,6 +667,14 @@ nsAutoCompleteController::HandleDelete(bool* _retval) {
   nsIAutoCompleteResult* result = mResults.SafeObjectAt(searchIndex);
   NS_ENSURE_TRUE(result, NS_ERROR_FAILURE);
 
+  bool removable;
+  nsresult rv = result->IsRemovableAt(matchIndex, &removable);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!removable) {
+    return NS_OK;
+  }
+
   nsAutoString search;
   input->GetSearchParam(search);
 
@@ -903,11 +901,16 @@ nsresult nsAutoCompleteController::BeforeSearches() {
   mSearchStatus = nsIAutoCompleteController::STATUS_SEARCHING;
   mDefaultIndexCompleted = false;
 
-  // ClearResults will clear the mResults array, but we should pass the previous
-  // result to each search to allow reusing it.  So we temporarily cache the
-  // current results until AfterSearches().
-  if (!mResultCache.AppendObjects(mResults)) {
-    return NS_ERROR_OUT_OF_MEMORY;
+  bool invalidatePreviousResult = false;
+  mInput->GetInvalidatePreviousResult(&invalidatePreviousResult);
+
+  if (!invalidatePreviousResult) {
+    // ClearResults will clear the mResults array, but we should pass the
+    // previous result to each search to allow reusing it.  So we temporarily
+    // cache the current results until AfterSearches().
+    if (!mResultCache.AppendObjects(mResults)) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
   }
   ClearResults(true);
   mSearchesOngoing = mSearches.Length();
@@ -970,7 +973,8 @@ nsresult nsAutoCompleteController::StartSearch(uint16_t aSearchType) {
     }
 
     rv = search->StartSearch(mSearchString, searchParam, result,
-                             static_cast<nsIAutoCompleteObserver*>(this));
+                             static_cast<nsIAutoCompleteObserver*>(this),
+                             nullptr);
     if (NS_FAILED(rv)) {
       ++mSearchesFailed;
       MOZ_ASSERT(mSearchesOngoing > 0);
@@ -1167,6 +1171,8 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
 
   // Ask the popup if it wants to enter a special value into the textbox
   nsAutoString value;
+  nsAutoString comment;
+
   popup->GetOverrideValue(value);
   if (value.IsEmpty()) {
     bool shouldComplete;
@@ -1177,6 +1183,7 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
     if (selectedIndex >= 0) {
       nsAutoString inputValue;
       input->GetTextValue(inputValue);
+      GetCommentAt(selectedIndex, comment);
       if (aIsPopupSelection || !completeSelection) {
         // We need to fill-in the value if:
         //  * completeselectedindex is false
@@ -1268,12 +1275,16 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
     }
   }
 
+  if (comment.IsEmpty()) {
+    comment.Assign(u"{}");
+  }
+
   nsCOMPtr<nsIObserverService> obsSvc = services::GetObserverService();
   NS_ENSURE_STATE(obsSvc);
-  obsSvc->NotifyObservers(input, "autocomplete-will-enter-text", nullptr);
+  obsSvc->NotifyObservers(input, "autocomplete-will-enter-text", comment.get());
 
   if (!value.IsEmpty()) {
-    SetValueOfInputTo(value, nsIAutoCompleteInput::TEXTVALUE_REASON_ENTERMATCH);
+    SetValueOfInputTo(value);
     input->SelectTextRange(value.Length(), value.Length());
     SetSearchStringInternal(value);
   }
@@ -1319,8 +1330,7 @@ nsresult nsAutoCompleteController::RevertTextValue() {
     // events. NOTE: how can |RevertTextValue| be called with inputValue !=
     // oldValue?
     if (mSearchString != currentValue) {
-      SetValueOfInputTo(mSearchString,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_REVERT);
+      SetValueOfInputTo(mSearchString);
     }
 
     obsSvc->NotifyObservers(input, "autocomplete-did-revert-text", nullptr);
@@ -1500,8 +1510,7 @@ nsresult nsAutoCompleteController::CompleteDefaultIndex(int32_t aResultIndex) {
     nsAutoString inputValue;
     input->GetTextValue(inputValue);
     if (!inputValue.Equals(mSearchString)) {
-      SetValueOfInputTo(mSearchString,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_REVERT);
+      SetValueOfInputTo(mSearchString);
       input->SelectTextRange(mSearchString.Length(), mSearchString.Length());
     }
     mPlaceholderCompletionString.Truncate();
@@ -1629,8 +1638,7 @@ nsresult nsAutoCompleteController::CompleteValue(nsString& aValue)
     // matches the beginning of aValue.  In either case we can simply
     // autocomplete to aValue.
     mPlaceholderCompletionString = aValue;
-    SetValueOfInputTo(aValue,
-                      nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+    SetValueOfInputTo(aValue);
   } else {
     nsresult rv;
     nsCOMPtr<nsIIOService> ios = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
@@ -1654,16 +1662,14 @@ nsresult nsAutoCompleteController::CompleteValue(nsString& aValue)
       mPlaceholderCompletionString =
           mSearchString +
           Substring(aValue, mSearchStringLength + findIndex, endSelect);
-      SetValueOfInputTo(mPlaceholderCompletionString,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+      SetValueOfInputTo(mPlaceholderCompletionString);
 
       endSelect -= findIndex;  // We're skipping this many characters of aValue.
     } else {
       // Autocompleting something other than a URI from the middle.
       // Use the format "searchstring >> full string" to indicate to the user
       // what we are going to replace their search string with.
-      SetValueOfInputTo(mSearchString + NS_LITERAL_STRING(" >> ") + aValue,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+      SetValueOfInputTo(mSearchString + u" >> "_ns + aValue);
 
       endSelect = mSearchString.Length() + 4 + aValue.Length();
 
