@@ -10,9 +10,14 @@
  * for the current user.
  */
 add_task(async function test_private_temp() {
+  Services.prefs.setBoolPref(
+    "browser.download.improvements_to_download_panel",
+    false
+  );
 
   let download = await promiseStartExternalHelperAppServiceDownload(
-                                                         httpUrl("empty.txt"));
+    httpUrl("empty.txt")
+  );
 
   await promiseDownloadStopped(download);
 
@@ -21,4 +26,9 @@ add_task(async function test_private_temp() {
 
   // 488 is the decimal value of 0o700.
   equal(targetFile.parent.permissions, 448);
+
+  Services.prefs.setBoolPref(
+    "browser.download.improvements_to_download_panel",
+    true
+  );
 });
