@@ -11,7 +11,7 @@ var { ExtensionPreferencesManager } = ChromeUtils.import(
 );
 
 var { ExtensionError } = ExtensionUtils;
-var { getSettingsAPI } = ExtensionPreferencesManager;
+var { getSettingsAPI, getPrimedSettingsListener } = ExtensionPreferencesManager;
 
 const cookieSvc = Ci.nsICookieService;
 
@@ -64,6 +64,31 @@ ExtensionPreferencesManager.addSetting("network.networkPredictionEnabled", {
       "network.prefetch-next": value,
     };
   },
+
+  getCallback() {
+    return (
+      getBoolPref("network.predictor.enabled") &&
+      getBoolPref("network.prefetch-next") &&
+      getIntPref("network.http.speculative-parallel-limit") > 0 &&
+      !getBoolPref("network.dns.disablePrefetch")
+    );
+  },
+});
+
+ExtensionPreferencesManager.addSetting("network.globalPrivacyControl", {
+  permission: "privacy",
+  prefNames: ["privacy.globalprivacycontrol.enabled"],
+  readOnly: true,
+
+  setCallback(value) {
+    return {
+      "privacy.globalprivacycontrol.enabled": value,
+    };
+  },
+
+  getCallback() {
+    return getBoolPref("privacy.globalprivacycontrol.enabled");
+  },
 });
 
 ExtensionPreferencesManager.addSetting("network.httpsOnlyMode", {
@@ -72,6 +97,7 @@ ExtensionPreferencesManager.addSetting("network.httpsOnlyMode", {
     "dom.security.https_only_mode",
     "dom.security.https_only_mode_pbm",
   ],
+  readOnly: true,
 
   setCallback(value) {
     let prefs = {
@@ -94,6 +120,16 @@ ExtensionPreferencesManager.addSetting("network.httpsOnlyMode", {
 
     return prefs;
   },
+
+  getCallback() {
+    if (getBoolPref("dom.security.https_only_mode")) {
+      return "always";
+    }
+    if (getBoolPref("dom.security.https_only_mode_pbm")) {
+      return "private_browsing";
+    }
+    return "never";
+  },
 });
 
 ExtensionPreferencesManager.addSetting("network.peerConnectionEnabled", {
@@ -103,6 +139,10 @@ ExtensionPreferencesManager.addSetting("network.peerConnectionEnabled", {
   setCallback(value) {
     return { [this.prefNames[0]]: value };
   },
+
+  getCallback() {
+    return getBoolPref("media.peerconnection.enabled");
+  },
 });
 
 ExtensionPreferencesManager.addSetting("network.webRTCIPHandlingPolicy", {
@@ -142,6 +182,30 @@ ExtensionPreferencesManager.addSetting("network.webRTCIPHandlingPolicy", {
     }
     return prefs;
   },
+
+  getCallback() {
+    if (getBoolPref("media.peerconnection.ice.proxy_only")) {
+      return "proxy_only";
+    }
+
+    let default_address_only = getBoolPref(
+      "media.peerconnection.ice.default_address_only"
+    );
+    if (default_address_only) {
+      let no_host = getBoolPref("media.peerconnection.ice.no_host");
+      if (no_host) {
+        if (
+          getBoolPref("media.peerconnection.ice.proxy_only_if_behind_proxy")
+        ) {
+          return "disable_non_proxied_udp";
+        }
+        return "default_public_interface_only";
+      }
+      return "default_public_and_private_interfaces";
+    }
+
+    return "default";
+  },
 });
 
 ExtensionPreferencesManager.addSetting("services.passwordSavingEnabled", {
@@ -151,11 +215,15 @@ ExtensionPreferencesManager.addSetting("services.passwordSavingEnabled", {
   setCallback(value) {
     return { [this.prefNames[0]]: value };
   },
+
+  getCallback() {
+    return getBoolPref("signon.rememberSignons");
+  },
 });
 
 ExtensionPreferencesManager.addSetting("websites.cookieConfig", {
   permission: "privacy",
-  prefNames: ["network.cookie.cookieBehavior", "network.cookie.lifetimePolicy"],
+  prefNames: ["network.cookie.cookieBehavior"],
 
   setCallback(value) {
     const cookieBehavior = cookieBehaviorValues.get(value.behavior);
@@ -173,11 +241,26 @@ ExtensionPreferencesManager.addSetting("websites.cookieConfig", {
         `Invalid cookieConfig '${value.behavior}' when firstPartyIsolate is enabled`
       );
     }
+
+    if (typeof value.nonPersistentCookies === "boolean") {
+      Cu.reportError(
+        "'nonPersistentCookies' has been deprecated and it has no effect anymore."
+      );
+    }
+
     return {
       "network.cookie.cookieBehavior": cookieBehavior,
-      "network.cookie.lifetimePolicy": value.nonPersistentCookies
-        ? cookieSvc.ACCEPT_SESSION
-        : cookieSvc.ACCEPT_NORMALLY,
+    };
+  },
+
+  getCallback() {
+    let prefValue = getIntPref("network.cookie.cookieBehavior");
+    return {
+      behavior: Array.from(cookieBehaviorValues.entries()).find(
+        entry => entry[1] === prefValue
+      )[0],
+      // Bug 1754924 - this property is now deprecated.
+      nonPersistentCookies: false,
     };
   },
 });
@@ -207,6 +290,10 @@ ExtensionPreferencesManager.addSetting("websites.firstPartyIsolate", {
 
     return { [this.prefNames[0]]: value };
   },
+
+  getCallback() {
+    return getBoolPref("privacy.firstparty.isolate");
+  },
 });
 
 ExtensionPreferencesManager.addSetting("websites.hyperlinkAuditingEnabled", {
@@ -216,6 +303,10 @@ ExtensionPreferencesManager.addSetting("websites.hyperlinkAuditingEnabled", {
   setCallback(value) {
     return { [this.prefNames[0]]: value };
   },
+
+  getCallback() {
+    return getBoolPref("browser.send_pings");
+  },
 });
 
 ExtensionPreferencesManager.addSetting("websites.referrersEnabled", {
@@ -228,6 +319,10 @@ ExtensionPreferencesManager.addSetting("websites.referrersEnabled", {
   setCallback(value) {
     return { [this.prefNames[0]]: value ? 2 : 0 };
   },
+
+  getCallback() {
+    return getIntPref("network.http.sendRefererHeader") !== 0;
+  },
 });
 
 ExtensionPreferencesManager.addSetting("websites.resistFingerprinting", {
@@ -237,6 +332,10 @@ ExtensionPreferencesManager.addSetting("websites.resistFingerprinting", {
   setCallback(value) {
     return { [this.prefNames[0]]: value };
   },
+
+  getCallback() {
+    return getBoolPref("privacy.resistFingerprinting");
+  },
 });
 
 ExtensionPreferencesManager.addSetting("websites.trackingProtectionMode", {
@@ -268,6 +367,15 @@ ExtensionPreferencesManager.addSetting("websites.trackingProtectionMode", {
 
     return prefs;
   },
+
+  getCallback() {
+    if (getBoolPref("privacy.trackingprotection.enabled")) {
+      return "always";
+    } else if (getBoolPref("privacy.trackingprotection.pbmode.enabled")) {
+      return "private_browsing";
+    }
+    return "never";
+  },
 });
 
 ExtensionPreferencesManager.addSetting("network.tlsVersionRestriction", {
@@ -318,171 +426,89 @@ ExtensionPreferencesManager.addSetting("network.tlsVersionRestriction", {
 
     return prefs;
   },
+
+  getCallback() {
+    function tlsVersionToString(pref) {
+      const value = getIntPref(pref);
+      const version = TLS_VERSIONS.find(a => a.version === value);
+      if (version) {
+        return version.name;
+      }
+      return "unknown";
+    }
+
+    return {
+      minimum: tlsVersionToString(TLS_MIN_PREF),
+      maximum: tlsVersionToString(TLS_MAX_PREF),
+    };
+  },
+
+  validate(extension) {
+    if (!extension.isPrivileged) {
+      throw new ExtensionError(
+        "tlsVersionRestriction can be set by privileged extensions only."
+      );
+    }
+  },
 });
 
 this.privacy = class extends ExtensionAPI {
+  primeListener(event, fire) {
+    let { extension } = this;
+    let listener = getPrimedSettingsListener({
+      extension,
+      name: event,
+    });
+    return listener(fire);
+  }
+
   getAPI(context) {
+    function makeSettingsAPI(name) {
+      return getSettingsAPI({
+        context,
+        module: "privacy",
+        name,
+      });
+    }
+
     return {
       privacy: {
         network: {
-          networkPredictionEnabled: getSettingsAPI({
-            context,
-            name: "network.networkPredictionEnabled",
-            callback() {
-              return (
-                getBoolPref("network.predictor.enabled") &&
-                getBoolPref("network.prefetch-next") &&
-                getIntPref("network.http.speculative-parallel-limit") > 0 &&
-                !getBoolPref("network.dns.disablePrefetch")
-              );
-            },
-          }),
-          httpsOnlyMode: getSettingsAPI({
-            context,
-            name: "network.httpsOnlyMode",
-            callback() {
-              if (getBoolPref("dom.security.https_only_mode")) {
-                return "always";
-              }
-              if (getBoolPref("dom.security.https_only_mode_pbm")) {
-                return "private_browsing";
-              }
-              return "never";
-            },
-            readOnly: true,
-          }),
-          peerConnectionEnabled: getSettingsAPI({
-            context,
-            name: "network.peerConnectionEnabled",
-            callback() {
-              return getBoolPref("media.peerconnection.enabled");
-            },
-          }),
-          webRTCIPHandlingPolicy: getSettingsAPI({
-            context,
-            name: "network.webRTCIPHandlingPolicy",
-            callback() {
-              if (getBoolPref("media.peerconnection.ice.proxy_only")) {
-                return "proxy_only";
-              }
-
-              let default_address_only = getBoolPref(
-                "media.peerconnection.ice.default_address_only"
-              );
-              if (default_address_only) {
-                let no_host = getBoolPref("media.peerconnection.ice.no_host");
-                if (no_host) {
-                  if (
-                    getBoolPref(
-                      "media.peerconnection.ice.proxy_only_if_behind_proxy"
-                    )
-                  ) {
-                    return "disable_non_proxied_udp";
-                  }
-                  return "default_public_interface_only";
-                }
-                return "default_public_and_private_interfaces";
-              }
-
-              return "default";
-            },
-          }),
-          tlsVersionRestriction: getSettingsAPI({
-            context,
-            name: "network.tlsVersionRestriction",
-            callback() {
-              function tlsVersionToString(pref) {
-                const value = getIntPref(pref);
-                const version = TLS_VERSIONS.find(a => a.version === value);
-                if (version) {
-                  return version.name;
-                }
-                return "unknown";
-              }
-
-              return {
-                minimum: tlsVersionToString(TLS_MIN_PREF),
-                maximum: tlsVersionToString(TLS_MAX_PREF),
-              };
-            },
-            validate() {
-              if (!context.extension.isPrivileged) {
-                throw new ExtensionError(
-                  "tlsVersionRestriction can be set by privileged extensions only."
-                );
-              }
-            },
-          }),
+          networkPredictionEnabled: makeSettingsAPI(
+            "network.networkPredictionEnabled"
+          ),
+          globalPrivacyControl: makeSettingsAPI("network.globalPrivacyControl"),
+          httpsOnlyMode: makeSettingsAPI("network.httpsOnlyMode"),
+          peerConnectionEnabled: makeSettingsAPI(
+            "network.peerConnectionEnabled"
+          ),
+          webRTCIPHandlingPolicy: makeSettingsAPI(
+            "network.webRTCIPHandlingPolicy"
+          ),
+          tlsVersionRestriction: makeSettingsAPI(
+            "network.tlsVersionRestriction"
+          ),
         },
 
         services: {
-          passwordSavingEnabled: getSettingsAPI({
-            context,
-            name: "services.passwordSavingEnabled",
-            callback() {
-              return getBoolPref("signon.rememberSignons");
-            },
-          }),
+          passwordSavingEnabled: makeSettingsAPI(
+            "services.passwordSavingEnabled"
+          ),
         },
 
         websites: {
-          cookieConfig: getSettingsAPI({
-            context,
-            name: "websites.cookieConfig",
-            callback() {
-              let prefValue = getIntPref("network.cookie.cookieBehavior");
-              return {
-                behavior: Array.from(cookieBehaviorValues.entries()).find(
-                  entry => entry[1] === prefValue
-                )[0],
-                nonPersistentCookies:
-                  getIntPref("network.cookie.lifetimePolicy") ===
-                  cookieSvc.ACCEPT_SESSION,
-              };
-            },
-          }),
-          firstPartyIsolate: getSettingsAPI({
-            context,
-            name: "websites.firstPartyIsolate",
-            callback() {
-              return getBoolPref("privacy.firstparty.isolate");
-            },
-          }),
-          hyperlinkAuditingEnabled: getSettingsAPI({
-            context,
-            name: "websites.hyperlinkAuditingEnabled",
-            callback() {
-              return getBoolPref("browser.send_pings");
-            },
-          }),
-          referrersEnabled: getSettingsAPI({
-            context,
-            name: "websites.referrersEnabled",
-            callback() {
-              return getIntPref("network.http.sendRefererHeader") !== 0;
-            },
-          }),
-          resistFingerprinting: getSettingsAPI({
-            context,
-            name: "websites.resistFingerprinting",
-            callback() {
-              return getBoolPref("privacy.resistFingerprinting");
-            },
-          }),
-          trackingProtectionMode: getSettingsAPI({
-            context,
-            name: "websites.trackingProtectionMode",
-            callback() {
-              if (getBoolPref("privacy.trackingprotection.enabled")) {
-                return "always";
-              } else if (
-                getBoolPref("privacy.trackingprotection.pbmode.enabled")
-              ) {
-                return "private_browsing";
-              }
-              return "never";
-            },
-          }),
+          cookieConfig: makeSettingsAPI("websites.cookieConfig"),
+          firstPartyIsolate: makeSettingsAPI("websites.firstPartyIsolate"),
+          hyperlinkAuditingEnabled: makeSettingsAPI(
+            "websites.hyperlinkAuditingEnabled"
+          ),
+          referrersEnabled: makeSettingsAPI("websites.referrersEnabled"),
+          resistFingerprinting: makeSettingsAPI(
+            "websites.resistFingerprinting"
+          ),
+          trackingProtectionMode: makeSettingsAPI(
+            "websites.trackingProtectionMode"
+          ),
         },
       },
     };
