@@ -27,3 +27,9 @@ add_task(function test_bytes_in_use() {
     test_background_storage_area_with_bytes_in_use("sync", true)
   );
 });
+
+add_task(function test_storage_onChanged_event_page() {
+  return runWithPrefs([[STORAGE_SYNC_PREF, true]], () =>
+    test_storage_change_event_page("sync")
+  );
+});
