@@ -12,10 +12,6 @@
 #include "nsCOMPtr.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsINode.h"
-#include "mozilla/intl/WordBreaker.h"
-
-class nsIContent;
-class nsRange;
 
 #define NS_FIND_CONTRACTID "@mozilla.org/embedcomp/rangefind;1"
 
@@ -26,27 +22,23 @@ class nsRange;
     }                                                \
   }
 
-class nsFindContentIterator;
-
 class nsFind : public nsIFind {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_NSIFIND
   NS_DECL_CYCLE_COLLECTION_CLASS(nsFind)
 
-  nsFind();
-
  protected:
-  virtual ~nsFind();
+  virtual ~nsFind() = default;
 
   // Parameters set from the interface:
-  bool mFindBackward;
-  bool mCaseSensitive;
-  bool mMatchDiacritics;
+  bool mFindBackward = false;
+  bool mCaseSensitive = false;
+  bool mMatchDiacritics = false;
 
-  // Use "find entire words" mode by setting to a word breaker or null, to
+  // Use "find entire words" mode by setting mEntireWord to true; or false to
   // disable "entire words" mode.
-  RefPtr<mozilla::intl::WordBreaker> mWordBreaker;
+  bool mEntireWord = false;
 
   struct State;
   class StateRestorer;
@@ -64,7 +56,7 @@ class nsFind : public nsIFind {
   // Get the first character from the next node (last if mFindBackward).
   //
   // This will mutate the state, but then restore it afterwards.
-  char32_t PeekNextChar(State&) const;
+  char32_t PeekNextChar(State&, bool aAlreadyMatching) const;
 };
 
 #endif  // nsFind_h__
