@@ -17,7 +17,7 @@ namespace mozilla::glean {
 class Category final : public nsISupports, public nsWrapperCache {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(Category)
+  NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(Category)
 
   JSObject* WrapObject(JSContext* aCx,
                        JS::Handle<JSObject*> aGivenProto) override;
