@@ -51,7 +51,7 @@ let login2B = new nsLoginInfo(
 
 requestLongerTimeout(2);
 
-add_task(async function setup() {
+add_setup(async function() {
   // Load recipes for this test.
   let recipeParent = await LoginManagerParent.recipeParentPromise;
   await recipeParent.load({
@@ -330,11 +330,7 @@ add_task(async function test_pwOnlyNewLoginMatchesUPForm() {
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "checking for notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(
-      notif.message,
-      "Would you like to add a username to the saved password?",
-      "Check message"
-    );
+    is(notif.message, "Add username to saved password?", "Check message");
 
     let { panel } = PopupNotifications;
     let passwordVisiblityToggle = panel.querySelector(
@@ -385,11 +381,7 @@ add_task(async function test_pwOnlyOldLoginMatchesUPForm() {
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "checking for notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(
-      notif.message,
-      "Would you like to add a username to the saved password?",
-      "Check message"
-    );
+    is(notif.message, "Add username to saved password?", "Check message");
 
     let { panel } = PopupNotifications;
     let passwordVisiblityToggle = panel.querySelector(
@@ -477,7 +469,7 @@ add_task(async function test_changeUPLoginOnUPForm_dont() {
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "got notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(notif.message, "Would you like to update this login?", "Check message");
+    is(notif.message, "Update login for example.com?", "Check message");
 
     await checkDoorhangerUsernamePassword("notifyu1", "pass2");
     clickDoorhangerButton(notif, DONT_CHANGE_BUTTON);
@@ -498,28 +490,25 @@ add_task(async function test_changeUPLoginOnUPForm_remove() {
   Services.logins.addLogin(login1);
 
   await testSubmittingLoginForm("subtst_notifications_8.html", async function(
-    fieldValues
+    fieldValues,
+    browser
   ) {
     is(fieldValues.username, "notifyu1", "Checking submitted username");
     is(fieldValues.password, "pass2", "Checking submitted password");
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "got notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(notif.message, "Would you like to update this login?", "Check message");
+    is(notif.message, "Update login for example.com?", "Check message");
 
     await checkDoorhangerUsernamePassword("notifyu1", "pass2");
     clickDoorhangerButton(notif, REMOVE_LOGIN_MENUITEM);
-  });
 
-  // Let the hint hide itself
-  const forceClosePopup = false;
-  // Make sure confirmation hint was shown
-  info("waiting for verifyConfirmationHint");
-  await verifyConfirmationHint(
-    gBrowser.selectedBrowser,
-    forceClosePopup,
-    "identity-icon"
-  );
+    // Let the hint hide itself
+    const forceClosePopup = false;
+    // Make sure confirmation hint was shown
+    info("waiting for verifyConfirmationHint");
+    await verifyConfirmationHint(browser, forceClosePopup, "identity-icon");
+  });
 
   let logins = Services.logins.getAllLogins();
   is(logins.length, 0, "Should have 0 logins");
@@ -537,7 +526,7 @@ add_task(async function test_changeUPLoginOnUPForm_change() {
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "got notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(notif.message, "Would you like to update this login?", "Check message");
+    is(notif.message, "Update login for example.com?", "Check message");
 
     await checkDoorhangerUsernamePassword("notifyu1", "pass2");
     let promiseLoginUpdateSaved = TestUtils.topicObserved(
@@ -577,7 +566,7 @@ add_task(async function test_changePLoginOnUPForm() {
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "got notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(notif.message, "Would you like to update this password?", "Check msg");
+    is(notif.message, "Update password for example.com?", "Check msg");
 
     await checkDoorhangerUsernamePassword("", "pass2");
     clickDoorhangerButton(notif, CHANGE_BUTTON);
@@ -606,7 +595,7 @@ add_task(async function test_changePLoginOnPForm() {
     let notif = await getCaptureDoorhangerThatMayOpen("password-change");
     ok(notif, "got notification popup");
     ok(!notif.dismissed, "doorhanger is not dismissed");
-    is(notif.message, "Would you like to update this password?", "Check msg");
+    is(notif.message, "Update password for example.com?", "Check msg");
 
     await checkDoorhangerUsernamePassword("", "notifyp1");
     clickDoorhangerButton(notif, CHANGE_BUTTON);
@@ -637,11 +626,8 @@ add_task(async function test_checkUPSaveText() {
     ok(notif, "got notification popup");
     // Check the text, which comes from the localized saveLoginMsg string.
     let notificationText = notif.message;
-    let expectedText =
-      "Would you like " +
-      BRAND_SHORT_NAME +
-      " to save this login for example.com?";
-    is(expectedText, notificationText, "Checking text: " + notificationText);
+    let expectedText = "Save login for example.com?";
+    is(notificationText, expectedText, "Checking text: " + notificationText);
     await cleanupDoorhanger(notif);
   });
 
@@ -665,11 +651,8 @@ add_task(async function test_checkPSaveText() {
     ok(notif, "got notification popup");
     // Check the text, which comes from the localized saveLoginMsgNoUser string.
     let notificationText = notif.message;
-    let expectedText =
-      "Would you like " +
-      BRAND_SHORT_NAME +
-      " to save this password for example.com?";
-    is(expectedText, notificationText, "Checking text: " + notificationText);
+    let expectedText = "Save password for example.com?";
+    is(notificationText, expectedText, "Checking text: " + notificationText);
     await cleanupDoorhanger(notif);
   });
 
@@ -886,7 +869,7 @@ add_task(async function test_recipeCaptureFields_ExistingLogin() {
   is(login.password, "notifyp1", "Check the password unchanged");
   is(login.timesUsed, 2, "Check times used incremented");
 
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
 });
 
 add_task(async function test_saveUsingEnter() {
@@ -925,7 +908,7 @@ add_task(async function test_saveUsingEnter() {
     is(login.password, "notifyp1", "Check the password used on the new entry");
     is(login.timesUsed, 1, "Check times used on new entry");
 
-    Services.logins.removeAllLogins();
+    Services.logins.removeAllUserFacingLogins();
   }
 
   await testWithTextboxSelector("#password-notification-password");
