@@ -5,8 +5,15 @@ const RELATIVE_DIR = "toolkit/components/pdfjs/test/";
 const TESTROOT = "http://example.com/browser/" + RELATIVE_DIR;
 
 function test() {
+  // When the always ask pref is disabled, we expect the PDF to be simply
+  // downloaded without a prompt, so ensure the pref is enabled here.
+  Services.prefs.setBoolPref(
+    "browser.download.always_ask_before_handling_new_types",
+    true
+  );
   var oldAction = changeMimeHandler(Ci.nsIHandlerInfo.useSystemDefault, true);
   var tab = BrowserTestUtils.addTab(gBrowser, TESTROOT + "file_pdfjs_test.pdf");
+
   // Test: "Open with" dialog comes up when pdf.js is not selected as the default
   // handler.
   addWindowListener(
@@ -16,6 +23,9 @@ function test() {
 
   waitForExplicitFinish();
   registerCleanupFunction(function() {
+    Services.prefs.clearUserPref(
+      "browser.download.always_ask_before_handling_new_types"
+    );
     changeMimeHandler(oldAction[0], oldAction[1]);
     gBrowser.removeTab(tab);
   });
