@@ -24,6 +24,7 @@
       ", url_hash INTEGER DEFAULT 0 NOT NULL "         \
       ", description TEXT"                             \
       ", preview_image_url TEXT"                       \
+      ", site_name TEXT"                               \
       ", origin_id INTEGER REFERENCES moz_origins(id)" \
       ")")
 
@@ -305,4 +306,82 @@
       "terms TEXT NOT NULL UNIQUE "                                      \
       ")")
 
+#define CREATE_MOZ_PLACES_METADATA_SNAPSHOTS                                  \
+  nsLiteralCString(                                                           \
+      "CREATE TABLE IF NOT EXISTS moz_places_metadata_snapshots ( "           \
+      "  place_id INTEGER PRIMARY KEY, "                                      \
+      "  created_at INTEGER NOT NULL, "                                       \
+      "  removed_at INTEGER, "                                                \
+      "  first_interaction_at INTEGER NOT NULL, "                             \
+      "  last_interaction_at INTEGER NOT NULL, "                              \
+      "  document_type INTEGER NOT NULL DEFAULT 0, "                          \
+      "  user_persisted INTEGER NOT NULL DEFAULT 0, "                         \
+      "  title TEXT, "                                                        \
+      "  FOREIGN KEY (place_id) REFERENCES moz_places(id) ON DELETE CASCADE " \
+      ")")
+
+#define CREATE_MOZ_PLACES_METADATA_SNAPSHOTS_EXTRA                        \
+  nsLiteralCString(                                                       \
+      "CREATE TABLE IF NOT EXISTS moz_places_metadata_snapshots_extra ( " \
+      "  place_id INTEGER NOT NULL, "                                     \
+      "  type INTEGER NOT NULL DEFAULT 0, "                               \
+      "  data TEXT NOT NULL, "                                            \
+      "  PRIMARY KEY (place_id, type), "                                  \
+      "  FOREIGN KEY (place_id) REFERENCES "                              \
+      "moz_places_metadata_snapshots(place_id) ON DELETE CASCADE "        \
+      ") WITHOUT ROWID ")
+
+#define CREATE_MOZ_PLACES_METADATA_SNAPSHOTS_GROUPS                        \
+  nsLiteralCString(                                                        \
+      "CREATE TABLE IF NOT EXISTS moz_places_metadata_snapshots_groups ( " \
+      "  id INTEGER PRIMARY KEY, "                                         \
+      "  title TEXT, "                                                     \
+      "  hidden INTEGER DEFAULT 0 NOT NULL, "                              \
+      "  builder TEXT NOT NULL, "                                          \
+      "  builder_data TEXT "                                               \
+      ")")
+
+// Note: if adding/removing columns here, consider updating
+// SnapshotGroups.updateUrls as well.
+#define CREATE_MOZ_PLACES_METADATA_GROUPS_TO_SNAPSHOTS                        \
+  nsLiteralCString(                                                           \
+      "CREATE TABLE IF NOT EXISTS moz_places_metadata_groups_to_snapshots ( " \
+      "  group_id INTEGER NOT NULL, "                                         \
+      "  place_id INTEGER NOT NULL, "                                         \
+      "  hidden INTEGER DEFAULT 0 NOT NULL, "                                 \
+      "  PRIMARY KEY (group_id, place_id), "                                  \
+      "  FOREIGN KEY (group_id) REFERENCES "                                  \
+      "    moz_places_metadata_snapshots_groups(id) ON DELETE CASCADE, "      \
+      "  FOREIGN KEY (place_id) REFERENCES "                                  \
+      "    moz_places_metadata_snapshots(place_id) ON DELETE CASCADE "        \
+      ") WITHOUT ROWID")
+
+#define CREATE_MOZ_SESSION_METADATA                        \
+  nsLiteralCString(                                        \
+      "CREATE TABLE IF NOT EXISTS moz_session_metadata ( " \
+      "  id INTEGER PRIMARY KEY, "                         \
+      "  guid TEXT NOT NULL UNIQUE, "                      \
+      "  last_saved_at INTEGER NOT NULL DEFAULT 0, "       \
+      "  data TEXT "                                       \
+      ")")
+
+#define CREATE_MOZ_SESSION_TO_PLACES                                       \
+  nsLiteralCString(                                                        \
+      "CREATE TABLE IF NOT EXISTS moz_session_to_places ( "                \
+      "  session_id INTEGER NOT NULL, "                                    \
+      "  place_id INTEGER NOT NULL , "                                     \
+      "  position INTEGER, "                                               \
+      "  PRIMARY KEY (session_id, place_id), "                             \
+      "  FOREIGN KEY (place_id) REFERENCES moz_places(id) ON DELETE "      \
+      "CASCADE "                                                           \
+      "  FOREIGN KEY (session_id) REFERENCES moz_session_metadata(id) ON " \
+      "DELETE CASCADE "                                                    \
+      ") WITHOUT ROWID")
+
+#define CREATE_MOZ_PREVIEWS_TOMBSTONES                        \
+  nsLiteralCString(                                           \
+      "CREATE TABLE IF NOT EXISTS moz_previews_tombstones ( " \
+      "  hash TEXT PRIMARY KEY "                              \
+      ") WITHOUT ROWID")
+
 #endif  // __nsPlacesTables_h__
