@@ -14,13 +14,11 @@ add_task(async function() {
     url,
   });
 
-  let file = OS.Path.join(
-    OS.Constants.Path.profileDir,
+  let file = PathUtils.join(
+    PathUtils.profileDir,
     "bookmarks.exported.997030.html"
   );
-  if (await OS.File.exists(file)) {
-    await OS.File.remove(file);
-  }
+  await IOUtils.remove(file, { ignoreAbsent: true });
   await BookmarkHTMLUtils.exportToFile(file);
 
   // Remove the bookmarks, then restore the backup.
