@@ -3,12 +3,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-// It is expected that the test files importing this file define Cu etc.
-/* global Cu, Ci, Cc, Cr */
-
-const CURRENT_SCHEMA_VERSION = 43;
-const FIRST_UPGRADABLE_SCHEMA_VERSION = 30;
-
 const NS_APP_USER_PROFILE_50_DIR = "ProfD";
 
 // Shortcuts to transitions type.
@@ -17,48 +11,56 @@ const TRANSITION_TYPED = Ci.nsINavHistoryService.TRANSITION_TYPED;
 const TRANSITION_BOOKMARK = Ci.nsINavHistoryService.TRANSITION_BOOKMARK;
 const TRANSITION_EMBED = Ci.nsINavHistoryService.TRANSITION_EMBED;
 const TRANSITION_FRAMED_LINK = Ci.nsINavHistoryService.TRANSITION_FRAMED_LINK;
-const TRANSITION_REDIRECT_PERMANENT = Ci.nsINavHistoryService.TRANSITION_REDIRECT_PERMANENT;
-const TRANSITION_REDIRECT_TEMPORARY = Ci.nsINavHistoryService.TRANSITION_REDIRECT_TEMPORARY;
+const TRANSITION_REDIRECT_PERMANENT =
+  Ci.nsINavHistoryService.TRANSITION_REDIRECT_PERMANENT;
+const TRANSITION_REDIRECT_TEMPORARY =
+  Ci.nsINavHistoryService.TRANSITION_REDIRECT_TEMPORARY;
 const TRANSITION_DOWNLOAD = Ci.nsINavHistoryService.TRANSITION_DOWNLOAD;
 const TRANSITION_RELOAD = Ci.nsINavHistoryService.TRANSITION_RELOAD;
 
 const TITLE_LENGTH_MAX = 4096;
 
-Cu.importGlobalProperties(["URL"]);
-
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+var { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { PlacesSyncUtils } = ChromeUtils.import(
+  "resource://gre/modules/PlacesSyncUtils.jsm"
+);
 XPCOMUtils.defineLazyModuleGetters(this, {
   FileUtils: "resource://gre/modules/FileUtils.jsm",
   NetUtil: "resource://gre/modules/NetUtil.jsm",
   PromiseUtils: "resource://gre/modules/PromiseUtils.jsm",
-  Services: "resource://gre/modules/Services.jsm",
   BookmarkJSONUtils: "resource://gre/modules/BookmarkJSONUtils.jsm",
   BookmarkHTMLUtils: "resource://gre/modules/BookmarkHTMLUtils.jsm",
   PlacesBackups: "resource://gre/modules/PlacesBackups.jsm",
-  PlacesSyncUtils: "resource://gre/modules/PlacesSyncUtils.jsm",
   PlacesTestUtils: "resource://testing-common/PlacesTestUtils.jsm",
   PlacesTransactions: "resource://gre/modules/PlacesTransactions.jsm",
-  OS: "resource://gre/modules/osfile.jsm",
   Sqlite: "resource://gre/modules/Sqlite.jsm",
   TestUtils: "resource://testing-common/TestUtils.jsm",
   AppConstants: "resource://gre/modules/AppConstants.jsm",
   PlacesUtils: "resource://gre/modules/PlacesUtils.jsm",
+  PlacesDBUtils: "resource://gre/modules/PlacesDBUtils.jsm",
 });
 
 XPCOMUtils.defineLazyGetter(this, "SMALLPNG_DATA_URI", function() {
   return NetUtil.newURI(
-         "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAA" +
-         "AAAA6fptVAAAACklEQVQI12NgAAAAAgAB4iG8MwAAAABJRU5ErkJggg==");
+    "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAA" +
+      "AAAA6fptVAAAACklEQVQI12NgAAAAAgAB4iG8MwAAAABJRU5ErkJggg=="
+  );
 });
+const SMALLPNG_DATA_LEN = 67;
+
 XPCOMUtils.defineLazyGetter(this, "SMALLSVG_DATA_URI", function() {
   return NetUtil.newURI(
-         "data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy5" +
-         "3My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAxMDAgMTAwIiBmaWxs" +
-         "PSIjNDI0ZTVhIj4NCiAgPGNpcmNsZSBjeD0iNTAiIGN5PSI1MCIgcj0iN" +
-         "DQiIHN0cm9rZT0iIzQyNGU1YSIgc3Ryb2tlLXdpZHRoPSIxMSIgZmlsbD" +
-         "0ibm9uZSIvPg0KICA8Y2lyY2xlIGN4PSI1MCIgY3k9IjI0LjYiIHI9IjY" +
-         "uNCIvPg0KICA8cmVjdCB4PSI0NSIgeT0iMzkuOSIgd2lkdGg9IjEwLjEi" +
-         "IGhlaWdodD0iNDEuOCIvPg0KPC9zdmc%2BDQo%3D");
+    "data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy5" +
+      "3My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAxMDAgMTAwIiBmaWxs" +
+      "PSIjNDI0ZTVhIj4NCiAgPGNpcmNsZSBjeD0iNTAiIGN5PSI1MCIgcj0iN" +
+      "DQiIHN0cm9rZT0iIzQyNGU1YSIgc3Ryb2tlLXdpZHRoPSIxMSIgZmlsbD" +
+      "0ibm9uZSIvPg0KICA8Y2lyY2xlIGN4PSI1MCIgY3k9IjI0LjYiIHI9IjY" +
+      "uNCIvPg0KICA8cmVjdCB4PSI0NSIgeT0iMzkuOSIgd2lkdGg9IjEwLjEi" +
+      "IGhlaWdodD0iNDEuOCIvPg0KPC9zdmc%2BDQo%3D"
+  );
 });
 
 var gTestDir = do_get_cwd();
@@ -68,7 +70,8 @@ var gProfD = do_get_profile(true);
 
 Services.prefs.setBoolPref("browser.urlbar.usepreloadedtopurls.enabled", false);
 registerCleanupFunction(() =>
-  Services.prefs.clearUserPref("browser.urlbar.usepreloadedtopurls.enabled"));
+  Services.prefs.clearUserPref("browser.urlbar.usepreloadedtopurls.enabled")
+);
 
 // Remove any old database.
 clearDB();
@@ -83,7 +86,6 @@ function uri(aSpec) {
   return NetUtil.newURI(aSpec);
 }
 
-
 /**
  * Gets the database connection.  If the Places connection is invalid it will
  * try to create a new connection.
@@ -98,20 +100,22 @@ function uri(aSpec) {
 var gDBConn;
 function DBConn(aForceNewConnection) {
   if (!aForceNewConnection) {
-    let db = PlacesUtils.history.QueryInterface(Ci.nsPIPlacesDatabase)
-                                .DBConnection;
-    if (db.connectionReady)
+    let db = PlacesUtils.history.DBConnection;
+    if (db.connectionReady) {
       return db;
+    }
   }
 
   // If the Places database connection has been closed, create a new connection.
   if (!gDBConn || aForceNewConnection) {
     let file = Services.dirsvc.get("ProfD", Ci.nsIFile);
     file.append("places.sqlite");
-    let dbConn = gDBConn = Services.storage.openDatabase(file);
+    let dbConn = (gDBConn = Services.storage.openDatabase(file));
 
     // Be sure to cleanly close this connection.
-    promiseTopicObserved("profile-before-change").then(() => dbConn.asyncClose());
+    promiseTopicObserved("profile-before-change").then(() =>
+      dbConn.asyncClose()
+    );
   }
 
   return gDBConn.connectionReady ? gDBConn : null;
@@ -123,8 +127,9 @@ function DBConn(aForceNewConnection) {
  * @return an array of bytes.
  */
 function readInputStreamData(aStream) {
-  let bistream = Cc["@mozilla.org/binaryinputstream;1"].
-                 createInstance(Ci.nsIBinaryInputStream);
+  let bistream = Cc["@mozilla.org/binaryinputstream;1"].createInstance(
+    Ci.nsIBinaryInputStream
+  );
   try {
     bistream.setInputStream(aStream);
     let expectedData = [];
@@ -146,16 +151,17 @@ function readInputStreamData(aStream) {
  * @return an array of bytes.
  */
 function readFileData(aFile) {
-  let inputStream = Cc["@mozilla.org/network/file-input-stream;1"].
-                    createInstance(Ci.nsIFileInputStream);
+  let inputStream = Cc[
+    "@mozilla.org/network/file-input-stream;1"
+  ].createInstance(Ci.nsIFileInputStream);
   // init the stream as RD_ONLY, -1 == default permissions.
   inputStream.init(aFile, 0x01, -1, null);
 
   // Check the returned size versus the expected size.
-  let size  = inputStream.available();
+  let size = inputStream.available();
   let bytes = readInputStreamData(inputStream);
   if (size != bytes.length) {
-    throw "Didn't read expected number of bytes";
+    throw new Error("Didn't read expected number of bytes");
   }
   return bytes;
 }
@@ -176,7 +182,6 @@ function readFileOfLength(aFileName, aExpectedLength) {
   return data;
 }
 
-
 /**
  * Returns the base64-encoded version of the given string.  This function is
  * similar to window.btoa, but is available to xpcshell tests also.
@@ -188,15 +193,16 @@ function readFileOfLength(aFileName, aExpectedLength) {
  * @return The base64-encoded string.
  */
 function base64EncodeString(aString) {
-  var stream = Cc["@mozilla.org/io/string-input-stream;1"]
-               .createInstance(Ci.nsIStringInputStream);
+  var stream = Cc["@mozilla.org/io/string-input-stream;1"].createInstance(
+    Ci.nsIStringInputStream
+  );
   stream.setData(aString, aString.length);
-  var encoder = Cc["@mozilla.org/scriptablebase64encoder;1"]
-                .createInstance(Ci.nsIScriptableBase64Encoder);
+  var encoder = Cc["@mozilla.org/scriptablebase64encoder;1"].createInstance(
+    Ci.nsIScriptableBase64Encoder
+  );
   return encoder.encodeToString(stream, aString.length);
 }
 
-
 /**
  * Compares two arrays, and returns true if they are equal.
  *
@@ -213,8 +219,16 @@ function compareArrays(aArray1, aArray2) {
 
   for (let i = 0; i < aArray1.length; i++) {
     if (aArray1[i] != aArray2[i]) {
-      print("compareArrays: arrays differ at index " + i + ": " +
-            "(" + aArray1[i] + ") != (" + aArray2[i] + ")\n");
+      print(
+        "compareArrays: arrays differ at index " +
+          i +
+          ": " +
+          "(" +
+          aArray1[i] +
+          ") != (" +
+          aArray2[i] +
+          ")\n"
+      );
       return false;
     }
   }
@@ -222,7 +236,6 @@ function compareArrays(aArray1, aArray2) {
   return true;
 }
 
-
 /**
  * Deletes a previously created sqlite file from the profile folder.
  */
@@ -230,20 +243,25 @@ function clearDB() {
   try {
     let file = Services.dirsvc.get("ProfD", Ci.nsIFile);
     file.append("places.sqlite");
-    if (file.exists())
+    if (file.exists()) {
       file.remove(false);
-  } catch (ex) { dump("Exception: " + ex); }
+    }
+  } catch (ex) {
+    dump("Exception: " + ex);
+  }
 }
 
-
 /**
  * Dumps the rows of a table out to the console.
  *
  * @param aName
  *        The name of the table or view to output.
  */
-function dump_table(aName) {
-  let stmt = DBConn().createStatement("SELECT * FROM " + aName);
+function dump_table(aName, dbConn) {
+  if (!dbConn) {
+    dbConn = DBConn();
+  }
+  let stmt = dbConn.createStatement("SELECT * FROM " + aName);
 
   print("\n*** Printing data from " + aName);
   let count = 0;
@@ -252,8 +270,9 @@ function dump_table(aName) {
 
     if (count == 0) {
       // Print the column names.
-      for (let i = 0; i < columns; i++)
+      for (let i = 0; i < columns; i++) {
         dump(stmt.getColumnName(i) + "\t");
+      }
       dump("\n");
     }
 
@@ -283,7 +302,6 @@ function dump_table(aName) {
   stmt.finalize();
 }
 
-
 /**
  * Checks if an address is found in the database.
  * @param aURI
@@ -297,8 +315,9 @@ function page_in_database(aURI) {
   );
   stmt.params.url = url;
   try {
-    if (!stmt.executeStep())
+    if (!stmt.executeStep()) {
       return 0;
+    }
     return stmt.getInt64(0);
   } finally {
     stmt.finalize();
@@ -320,34 +339,15 @@ function visits_in_database(aURI) {
   );
   stmt.params.url = url;
   try {
-    if (!stmt.executeStep())
+    if (!stmt.executeStep()) {
       return 0;
+    }
     return stmt.getInt64(0);
   } finally {
     stmt.finalize();
   }
 }
 
-/**
- * Checks that we don't have any bookmark
- */
-function check_no_bookmarks() {
-  let query = PlacesUtils.history.getNewQuery();
-  let folders = [
-    PlacesUtils.bookmarks.toolbarFolder,
-    PlacesUtils.bookmarks.bookmarksMenuFolder,
-    PlacesUtils.bookmarks.unfiledBookmarksFolder,
-  ];
-  query.setFolders(folders, 3);
-  let options = PlacesUtils.history.getNewQueryOptions();
-  options.queryType = Ci.nsINavHistoryQueryOptions.QUERY_TYPE_BOOKMARKS;
-  let root = PlacesUtils.history.executeQuery(query, options).root;
-  root.containerOpen = true;
-  if (root.childCount != 0)
-    do_throw("Unable to remove all bookmarks");
-  root.containerOpen = false;
-}
-
 /**
  * Allows waiting for an observer notification once.
  *
@@ -360,10 +360,15 @@ function check_no_bookmarks() {
  */
 function promiseTopicObserved(aTopic) {
   return new Promise(resolve => {
-    Services.obs.addObserver(function observe(aObsSubject, aObsTopic, aObsData) {
+    Services.obs.addObserver(function observe(
+      aObsSubject,
+      aObsTopic,
+      aObsData
+    ) {
       Services.obs.removeObserver(observe, aObsTopic);
       resolve([aObsSubject, aObsData]);
-    }, aTopic);
+    },
+    aTopic);
   });
 }
 
@@ -386,8 +391,8 @@ var shutdownPlaces = function() {
 };
 
 const FILENAME_BOOKMARKS_HTML = "bookmarks.html";
-const FILENAME_BOOKMARKS_JSON = "bookmarks-" +
-  (PlacesBackups.toISODateString(new Date())) + ".json";
+const FILENAME_BOOKMARKS_JSON =
+  "bookmarks-" + PlacesBackups.toISODateString(new Date()) + ".json";
 
 /**
  * Creates a bookmarks.html file in the profile folder from a given source file.
@@ -399,8 +404,9 @@ const FILENAME_BOOKMARKS_JSON = "bookmarks-" +
  * @return nsIFile object for the file.
  */
 function create_bookmarks_html(aFilename) {
-  if (!aFilename)
+  if (!aFilename) {
     do_throw("you must pass a filename to create_bookmarks_html function");
+  }
   remove_bookmarks_html();
   let bookmarksHTMLFile = gTestDir.clone();
   bookmarksHTMLFile.append(aFilename);
@@ -412,7 +418,6 @@ function create_bookmarks_html(aFilename) {
   return profileBookmarksHTMLFile;
 }
 
-
 /**
  * Remove bookmarks.html file from the profile folder.
  */
@@ -425,7 +430,6 @@ function remove_bookmarks_html() {
   }
 }
 
-
 /**
  * Check bookmarks.html file exists in the profile folder.
  *
@@ -438,7 +442,6 @@ function check_bookmarks_html() {
   return profileBookmarksHTMLFile;
 }
 
-
 /**
  * Creates a JSON backup in the profile folder folder from a given source file.
  *
@@ -449,8 +452,9 @@ function check_bookmarks_html() {
  * @return nsIFile object for the file.
  */
 function create_JSON_backup(aFilename) {
-  if (!aFilename)
+  if (!aFilename) {
     do_throw("you must pass a filename to create_JSON_backup function");
+  }
   let bookmarksBackupDir = gProfD.clone();
   bookmarksBackupDir.append("bookmarkbackups");
   if (!bookmarksBackupDir.exists()) {
@@ -472,7 +476,6 @@ function create_JSON_backup(aFilename) {
   return profileBookmarksJSONFile;
 }
 
-
 /**
  * Remove bookmarksbackup dir and all backups from the profile folder.
  */
@@ -485,7 +488,6 @@ function remove_all_JSON_backups() {
   }
 }
 
-
 /**
  * Check a JSON backup file for today exists in the profile folder.
  *
@@ -500,7 +502,7 @@ function check_JSON_backup(aIsAutomaticBackup) {
     bookmarksBackupDir.append("bookmarkbackups");
     let files = bookmarksBackupDir.directoryEntries;
     while (files.hasMoreElements()) {
-      let entry = files.getNext().QueryInterface(Ci.nsIFile);
+      let entry = files.nextFile;
       if (PlacesBackups.filenamesRegex.test(entry.leafName)) {
         profileBookmarksJSONFile = entry;
         break;
@@ -556,8 +558,9 @@ function isUrlHidden(aURI) {
     "SELECT hidden FROM moz_places WHERE url_hash = hash(?1) AND url = ?1"
   );
   stmt.bindByIndex(0, url);
-  if (!stmt.executeStep())
+  if (!stmt.executeStep()) {
     throw new Error("No result for hidden.");
+  }
   let hidden = stmt.getInt32(0);
   stmt.finalize();
 
@@ -577,7 +580,7 @@ function is_time_ordered(before, after) {
   // Windows has an estimated 16ms timers precision, since Date.now() and
   // PR_Now() use different code atm, the results can be unordered by this
   // amount of time.  See bug 558745 and bug 557406.
-  let isWindows = ("@mozilla.org/windows-registry-key;1" in Cc);
+  let isWindows = "@mozilla.org/windows-registry-key;1" in Cc;
   // Just to be safe we consider 20ms.
   let skew = isWindows ? 20000000 : 0;
   return after - before > -skew;
@@ -602,12 +605,8 @@ function waitForConnectionClosed(aCallback) {
  * @param [optional] aStack
  *        The stack frame used to report the error.
  */
-function do_check_valid_places_guid(aGuid,
-                                    aStack) {
-  if (!aStack) {
-    aStack = Components.stack.caller;
-  }
-  Assert.ok(/^[a-zA-Z0-9\-_]{12}$/.test(aGuid), aStack);
+function do_check_valid_places_guid(aGuid) {
+  Assert.ok(/^[a-zA-Z0-9\-_]{12}$/.test(aGuid), "Should be a valid GUID");
 }
 
 /**
@@ -619,21 +618,17 @@ function do_check_valid_places_guid(aGuid,
  *        The stack frame used to report the error.
  * @return the associated the guid.
  */
-function do_get_guid_for_uri(aURI,
-                             aStack) {
-  if (!aStack) {
-    aStack = Components.stack.caller;
-  }
+function do_get_guid_for_uri(aURI) {
   let stmt = DBConn().createStatement(
     `SELECT guid
      FROM moz_places
      WHERE url_hash = hash(:url) AND url = :url`
   );
   stmt.params.url = aURI.spec;
-  Assert.ok(stmt.executeStep(), aStack);
+  Assert.ok(stmt.executeStep(), "GUID for URI statement should succeed");
   let guid = stmt.row.guid;
   stmt.finalize();
-  do_check_valid_places_guid(guid, aStack);
+  do_check_valid_places_guid(guid);
   return guid;
 }
 
@@ -645,13 +640,11 @@ function do_get_guid_for_uri(aURI,
  * @param [optional] aGUID
  *        The expected guid in the database.
  */
-function do_check_guid_for_uri(aURI,
-                               aGUID) {
-  let caller = Components.stack.caller;
-  let guid = do_get_guid_for_uri(aURI, caller);
+function do_check_guid_for_uri(aURI, aGUID) {
+  let guid = do_get_guid_for_uri(aURI);
   if (aGUID) {
-    do_check_valid_places_guid(aGUID, caller);
-    Assert.equal(guid, aGUID, caller);
+    do_check_valid_places_guid(aGUID);
+    Assert.equal(guid, aGUID, "Should have a guid in moz_places for the URI");
   }
 }
 
@@ -664,21 +657,17 @@ function do_check_guid_for_uri(aURI,
  *        The stack frame used to report the error.
  * @return the associated the guid.
  */
-function do_get_guid_for_bookmark(aId,
-                                  aStack) {
-  if (!aStack) {
-    aStack = Components.stack.caller;
-  }
+function do_get_guid_for_bookmark(aId) {
   let stmt = DBConn().createStatement(
     `SELECT guid
      FROM moz_bookmarks
      WHERE id = :item_id`
   );
   stmt.params.item_id = aId;
-  Assert.ok(stmt.executeStep(), aStack);
+  Assert.ok(stmt.executeStep(), "Should succeed executing the SQL statement");
   let guid = stmt.row.guid;
   stmt.finalize();
-  do_check_valid_places_guid(guid, aStack);
+  do_check_valid_places_guid(guid);
   return guid;
 }
 
@@ -690,13 +679,11 @@ function do_get_guid_for_bookmark(aId,
  * @param [optional] aGUID
  *        The expected guid in the database.
  */
-function do_check_guid_for_bookmark(aId,
-                                    aGUID) {
-  let caller = Components.stack.caller;
-  let guid = do_get_guid_for_bookmark(aId, caller);
+function do_check_guid_for_bookmark(aId, aGUID) {
+  let guid = do_get_guid_for_bookmark(aId);
   if (aGUID) {
-    do_check_valid_places_guid(aGUID, caller);
-    Assert.equal(guid, aGUID, caller);
+    do_check_valid_places_guid(aGUID);
+    Assert.equal(guid, aGUID, "Should have the correct GUID for the bookmark");
   }
 }
 
@@ -712,14 +699,17 @@ function do_check_guid_for_bookmark(aId,
  * @return true if the arrays contain the same elements, false otherwise.
  */
 function do_compare_arrays(a1, a2, sorted) {
-  if (a1.length != a2.length)
+  if (a1.length != a2.length) {
     return false;
+  }
 
   if (sorted) {
     return a1.every((e, i) => e == a2[i]);
   }
-  return a1.filter(e => !a2.includes(e)).length == 0 &&
-         a2.filter(e => !a1.includes(e)).length == 0;
+  return (
+    !a1.filter(e => !a2.includes(e)).length &&
+    !a2.filter(e => !a1.includes(e)).length
+  );
 }
 
 /**
@@ -729,36 +719,9 @@ function do_compare_arrays(a1, a2, sorted) {
 function NavBookmarkObserver() {}
 
 NavBookmarkObserver.prototype = {
-  onBeginUpdateBatch() {},
-  onEndUpdateBatch() {},
-  onItemAdded() {},
   onItemRemoved() {},
   onItemChanged() {},
-  onItemVisited() {},
-  onItemMoved() {},
-  QueryInterface: XPCOMUtils.generateQI([
-    Ci.nsINavBookmarkObserver,
-  ])
-};
-
-/**
- * Generic nsINavHistoryObserver that doesn't implement anything, but provides
- * dummy methods to prevent errors about an object not having a certain method.
- */
-function NavHistoryObserver() {}
-
-NavHistoryObserver.prototype = {
-  onBeginUpdateBatch() {},
-  onEndUpdateBatch() {},
-  onVisits() {},
-  onTitleChanged() {},
-  onDeleteURI() {},
-  onClearHistory() {},
-  onPageChanged() {},
-  onDeleteVisits() {},
-  QueryInterface: XPCOMUtils.generateQI([
-    Ci.nsINavHistoryObserver,
-  ])
+  QueryInterface: ChromeUtils.generateQI(["nsINavBookmarkObserver"]),
 };
 
 /**
@@ -772,7 +735,6 @@ NavHistoryResultObserver.prototype = {
   batching() {},
   containerStateChanged() {},
   invalidateContainer() {},
-  nodeAnnotationChanged() {},
   nodeDateAddedChanged() {},
   nodeHistoryDetailsChanged() {},
   nodeIconChanged() {},
@@ -785,18 +747,25 @@ NavHistoryResultObserver.prototype = {
   nodeTitleChanged() {},
   nodeURIChanged() {},
   sortingChanged() {},
-  QueryInterface: XPCOMUtils.generateQI([
-    Ci.nsINavHistoryResultObserver,
-  ])
+  QueryInterface: ChromeUtils.generateQI(["nsINavHistoryResultObserver"]),
 };
 
 function checkBookmarkObject(info) {
   do_check_valid_places_guid(info.guid);
   do_check_valid_places_guid(info.parentGuid);
   Assert.ok(typeof info.index == "number", "index should be a number");
-  Assert.ok(info.dateAdded.constructor.name == "Date", "dateAdded should be a Date");
-  Assert.ok(info.lastModified.constructor.name == "Date", "lastModified should be a Date");
-  Assert.ok(info.lastModified >= info.dateAdded, "lastModified should never be smaller than dateAdded");
+  Assert.ok(
+    info.dateAdded.constructor.name == "Date",
+    "dateAdded should be a Date"
+  );
+  Assert.ok(
+    info.lastModified.constructor.name == "Date",
+    "lastModified should be a Date"
+  );
+  Assert.ok(
+    info.lastModified >= info.dateAdded,
+    "lastModified should never be smaller than dateAdded"
+  );
   Assert.ok(typeof info.type == "number", "type should be a number");
 }
 
@@ -804,14 +773,17 @@ function checkBookmarkObject(info) {
  * Reads foreign_count value for a given url.
  */
 async function foreign_count(url) {
-  if (url instanceof Ci.nsIURI)
+  if (url instanceof Ci.nsIURI) {
     url = url.spec;
+  }
   let db = await PlacesUtils.promiseDBConnection();
   let rows = await db.executeCached(
     `SELECT foreign_count FROM moz_places
      WHERE url_hash = hash(:url) AND url = :url
-    `, { url });
-  return rows.length == 0 ? 0 : rows[0].getResultByName("foreign_count");
+    `,
+    { url }
+  );
+  return !rows.length ? 0 : rows[0].getResultByName("foreign_count");
 }
 
 function compareAscending(a, b) {
@@ -838,13 +810,15 @@ function sortBy(array, prop) {
  *        Whether to enforce reloading the icon.
  */
 function setFaviconForPage(page, icon, forceReload = true) {
-  let pageURI = page instanceof Ci.nsIURI ? page
-                                          : NetUtil.newURI(new URL(page).href);
-  let iconURI = icon instanceof Ci.nsIURI ? icon
-                                          : NetUtil.newURI(new URL(icon).href);
+  let pageURI =
+    page instanceof Ci.nsIURI ? page : NetUtil.newURI(new URL(page).href);
+  let iconURI =
+    icon instanceof Ci.nsIURI ? icon : NetUtil.newURI(new URL(icon).href);
   return new Promise(resolve => {
     PlacesUtils.favicons.setAndFetchFaviconForPage(
-      pageURI, iconURI, forceReload,
+      pageURI,
+      iconURI,
+      forceReload,
       PlacesUtils.favicons.FAVICON_LOAD_NON_PRIVATE,
       resolve,
       Services.scriptSecurityManager.getSystemPrincipal()
@@ -853,25 +827,34 @@ function setFaviconForPage(page, icon, forceReload = true) {
 }
 
 function getFaviconUrlForPage(page, width = 0) {
-  let pageURI = page instanceof Ci.nsIURI ? page
-                                          : NetUtil.newURI(new URL(page).href);
+  let pageURI =
+    page instanceof Ci.nsIURI ? page : NetUtil.newURI(new URL(page).href);
   return new Promise((resolve, reject) => {
-    PlacesUtils.favicons.getFaviconURLForPage(pageURI, iconURI => {
-      if (iconURI)
-        resolve(iconURI.spec);
-      else
-        reject("Unable to find an icon for " + pageURI.spec);
-    }, width);
+    PlacesUtils.favicons.getFaviconURLForPage(
+      pageURI,
+      iconURI => {
+        if (iconURI) {
+          resolve(iconURI.spec);
+        } else {
+          reject("Unable to find an icon for " + pageURI.spec);
+        }
+      },
+      width
+    );
   });
 }
 
 function getFaviconDataForPage(page, width = 0) {
-  let pageURI = page instanceof Ci.nsIURI ? page
-                                          : NetUtil.newURI(new URL(page).href);
+  let pageURI =
+    page instanceof Ci.nsIURI ? page : NetUtil.newURI(new URL(page).href);
   return new Promise(resolve => {
-    PlacesUtils.favicons.getFaviconDataForPage(pageURI, (iconUri, len, data, mimeType) => {
-      resolve({ data, mimeType });
-    }, width);
+    PlacesUtils.favicons.getFaviconDataForPage(
+      pageURI,
+      (iconUri, len, data, mimeType) => {
+        resolve({ data, mimeType });
+      },
+      width
+    );
   });
 }
 
@@ -884,46 +867,131 @@ async function compareFavicons(icon1, icon2, msg) {
 
   function getIconData(icon) {
     return new Promise((resolve, reject) => {
-      NetUtil.asyncFetch({
-        uri: icon.href, loadUsingSystemPrincipal: true,
-        contentPolicyType: Ci.nsIContentPolicy.TYPE_INTERNAL_IMAGE_FAVICON
-      }, function(inputStream, status) {
-          if (!Components.isSuccessCode(status))
+      NetUtil.asyncFetch(
+        {
+          uri: icon.href,
+          loadUsingSystemPrincipal: true,
+          contentPolicyType: Ci.nsIContentPolicy.TYPE_INTERNAL_IMAGE_FAVICON,
+        },
+        function(inputStream, status) {
+          if (!Components.isSuccessCode(status)) {
             reject();
+          }
           let size = inputStream.available();
           resolve(NetUtil.readInputStreamToString(inputStream, size));
-      });
+        }
+      );
     });
   }
 
   let data1 = await getIconData(icon1);
-  Assert.ok(data1.length > 0, "Should fetch icon data");
+  Assert.ok(!!data1.length, "Should fetch icon data");
   let data2 = await getIconData(icon2);
-  Assert.ok(data2.length > 0, "Should fetch icon data");
+  Assert.ok(!!data2.length, "Should fetch icon data");
   Assert.deepEqual(data1, data2, msg);
 }
 
 /**
- * Get the internal "root" folder name for an item, specified by its itemId.
- * If the itemId does not point to a root folder, null is returned.
+ * Get the internal "root" folder name for an item, specified by its itemGuid.
+ * If the itemGuid does not point to a root folder, null is returned.
  *
- * @param aItemId
- *        the item id.
- * @return the internal-root name for the root folder, if aItemId points
+ * @param itemGuid
+ *        the item guid.
+ * @return the internal-root name for the root folder, if itemGuid points
  * to such folder, null otherwise.
  */
-function mapItemIdToInternalRootName(aItemId) {
-  switch (aItemId) {
-    case PlacesUtils.placesRootId:
+function mapItemGuidToInternalRootName(itemGuid) {
+  switch (itemGuid) {
+    case PlacesUtils.bookmarks.rootGuid:
       return "placesRoot";
-    case PlacesUtils.bookmarksMenuFolderId:
+    case PlacesUtils.bookmarks.menuGuid:
       return "bookmarksMenuFolder";
-    case PlacesUtils.toolbarFolderId:
+    case PlacesUtils.bookmarks.toolbarGuid:
       return "toolbarFolder";
-    case PlacesUtils.unfiledBookmarksFolderId:
+    case PlacesUtils.bookmarks.unfiledGuid:
       return "unfiledBookmarksFolder";
-    case PlacesUtils.mobileFolderId:
+    case PlacesUtils.bookmarks.mobileGuid:
       return "mobileFolder";
   }
   return null;
 }
+
+const DB_FILENAME = "places.sqlite";
+
+/**
+ * Sets the database to use for the given test.  This should be the very first
+ * thing in the test, otherwise this database will not be used!
+ *
+ * @param {string|string[]} path
+ *        A filename or path to a database. The database must exist.
+ *        If this is a string, then this is assumed to be a filename in the
+ *        directory where the test calling this is located.
+ *        If this is an array, this is assumed to be a path relative to the
+ *        directory that this file, head_common.js, is located.
+ * @param {string} destFileName
+ *        The destination filename to copy the database to.
+ * @return {Promise} the final path to the database
+ */
+async function setupPlacesDatabase(path, destFileName = DB_FILENAME) {
+  let currentDir = do_get_cwd().path;
+
+  if (typeof path == "string") {
+    path = [path];
+  } else {
+    currentDir = PathUtils.parent(currentDir);
+  }
+  let src = PathUtils.join(currentDir, ...path);
+  Assert.ok(await IOUtils.exists(src), "Database file found");
+
+  // Ensure that our database doesn't already exist.
+  let dest = PathUtils.join(PathUtils.profileDir, destFileName);
+  Assert.ok(
+    !(await IOUtils.exists(dest)),
+    "Database file should not exist yet"
+  );
+
+  await IOUtils.copy(src, dest);
+  return dest;
+}
+
+/**
+ * Gets the URLs of pages that have a particular annotation.
+ *
+ * @param {String} name The name of the annotation to search for.
+ * @return An array of URLs found.
+ */
+function getPagesWithAnnotation(name) {
+  return PlacesUtils.promiseDBConnection().then(async db => {
+    let rows = await db.execute(
+      `
+      SELECT h.url FROM moz_anno_attributes n
+      JOIN moz_annos a ON n.id = a.anno_attribute_id
+      JOIN moz_places h ON h.id = a.place_id
+      WHERE n.name = :name
+    `,
+      { name }
+    );
+
+    return rows.map(row => row.getResultByName("url"));
+  });
+}
+
+/**
+ * Checks there are no orphan page annotations in the database, and no
+ * orphan anno attribute names.
+ */
+async function assertNoOrphanPageAnnotations() {
+  let db = await PlacesUtils.promiseDBConnection();
+
+  let rows = await db.execute(`
+    SELECT place_id FROM moz_annos
+    WHERE place_id NOT IN (SELECT id FROM moz_places)
+  `);
+
+  Assert.equal(rows.length, 0, "Should not have any orphan page annotations");
+
+  rows = await db.execute(`
+    SELECT id FROM moz_anno_attributes
+    WHERE id NOT IN (SELECT anno_attribute_id FROM moz_annos) AND
+          id NOT IN (SELECT anno_attribute_id FROM moz_items_annos)`);
+}
