@@ -161,7 +161,7 @@ add_task(async function test_update_frecencies() {
       id: "bookmarkDDDD",
       parentid: "unfiled",
       type: "bookmark",
-      title: "D",
+      title: null,
       bmkUri: "http://example.com/d",
     },
     {
@@ -286,7 +286,7 @@ async function setupLocalTree(localTimeSeconds) {
       },
       {
         guid: "bookmarkDDDD",
-        title: "D",
+        title: null,
         url: "http://example.com/d",
         dateAdded,
         lastModified,
@@ -433,21 +433,6 @@ add_task(async function test_apply_then_revert() {
   ]);
 
   observer.check([
-    {
-      name: "onItemChanged",
-      params: {
-        itemId: localItemIds.get("bookmarkEEEE"),
-        property: "guid",
-        isAnnoProperty: false,
-        newValue: "bookmarkEEEE",
-        type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
-        parentId: PlacesUtils.bookmarksMenuFolderId,
-        guid: "bookmarkEEEE",
-        parentGuid: PlacesUtils.bookmarks.menuGuid,
-        oldValue: "bookmarkEEE1",
-        source: PlacesUtils.bookmarks.SOURCES.SYNC,
-      },
-    },
     {
       name: "bookmark-removed",
       params: {
@@ -456,11 +441,24 @@ add_task(async function test_apply_then_revert() {
         index: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         urlHref: "http://example.com/d",
+        title: "", // null titles get turned into empty strings.
         guid: "bookmarkDDDD",
         parentGuid: PlacesUtils.bookmarks.menuGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
       },
     },
+    {
+      name: "bookmark-guid-changed",
+      params: {
+        itemId: localItemIds.get("bookmarkEEEE"),
+        type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+        urlHref: "",
+        guid: "bookmarkEEEE",
+        parentGuid: PlacesUtils.bookmarks.menuGuid,
+        source: PlacesUtils.bookmarks.SOURCES.SYNC,
+        isTagging: false,
+      },
+    },
     {
       name: "bookmark-added",
       params: {
@@ -536,33 +534,24 @@ add_task(async function test_apply_then_revert() {
       },
     },
     {
-      name: "onItemChanged",
+      name: "bookmark-title-changed",
       params: {
         itemId: localItemIds.get("folderAAAAAA"),
-        property: "title",
-        isAnnoProperty: false,
-        newValue: "A (remote)",
-        type: PlacesUtils.bookmarks.TYPE_FOLDER,
-        parentId: PlacesUtils.toolbarFolderId,
+        title: "A (remote)",
         guid: "folderAAAAAA",
         parentGuid: PlacesUtils.bookmarks.toolbarGuid,
-        oldValue: "A",
-        source: PlacesUtils.bookmarks.SOURCES.SYNC,
       },
     },
     {
-      name: "onItemChanged",
+      name: "bookmark-url-changed",
       params: {
         itemId: localItemIds.get("bookmarkBBBB"),
-        property: "uri",
-        isAnnoProperty: false,
-        newValue: "http://example.com/b-remote",
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
-        parentId: localItemIds.get("folderAAAAAA"),
+        urlHref: "http://example.com/b-remote",
         guid: "bookmarkBBBB",
         parentGuid: "folderAAAAAA",
-        oldValue: "http://example.com/b",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
+        isTagging: false,
       },
     },
   ]);
