@@ -55,7 +55,7 @@ add_task(async function test_submission_url_matching() {
 
   for (let [name, searchURL, searchParams] of SUBMISSION_YES) {
     engine = await addAndMakeDefault(name, searchURL, searchParams);
-    engineInfo = await Services.search.getDefaultEngineInfo();
+    engineInfo = Services.search.getDefaultEngineInfo();
     Assert.equal(
       engineInfo.defaultSearchEngineData.submissionURL,
       (searchURL + "?" + searchParams).replace("{searchTerms}", "")
@@ -65,7 +65,7 @@ add_task(async function test_submission_url_matching() {
 
   for (let [name, searchURL, searchParams] of SUBMISSION_NO) {
     engine = await addAndMakeDefault(name, searchURL, searchParams);
-    engineInfo = await Services.search.getDefaultEngineInfo();
+    engineInfo = Services.search.getDefaultEngineInfo();
     Assert.equal(engineInfo.defaultSearchEngineData.submissionURL, null);
     await Services.search.removeEngine(engine);
   }
@@ -75,7 +75,7 @@ add_task(async function test_submission_url_built_in() {
   const engine = await Services.search.getEngineByName("engine1");
   await Services.search.setDefault(engine);
 
-  const engineInfo = await Services.search.getDefaultEngineInfo();
+  const engineInfo = Services.search.getDefaultEngineInfo();
   Assert.equal(
     engineInfo.defaultSearchEngineData.submissionURL,
     "https://1.example.com/search?q=",
