@@ -8,10 +8,9 @@ import logging
 import os
 import sys
 
-from mach.decorators import CommandProvider, Command
+from mach.decorators import Command
 
 from mozbuild.base import (
-    MachCommandBase,
     MachCommandConditions as conditions,
     BinaryNotFoundException,
 )
@@ -55,6 +54,9 @@ def run_telemetry(tests, binary=None, topsrcdir=None, **kwargs):
 
     os.environ["MOZ_IGNORE_NSS_SHUTDOWN_LEAKS"] = "1"
 
+    # Causes Firefox to crash when using non-local connections.
+    os.environ["MOZ_DISABLE_NONLOCAL_CONNECTIONS"] = "1"
+
     if not args.logger:
         args.logger = commandline.setup_logging(
             "Telemetry Client Tests", args, {"mach": sys.stdout}
@@ -65,37 +67,35 @@ def run_telemetry(tests, binary=None, topsrcdir=None, **kwargs):
     return 0
 
 
-@CommandProvider
-class TelemetryTest(MachCommandBase):
-    @Command(
-        "telemetry-tests-client",
-        category="testing",
-        description="Run tests specifically for the Telemetry client",
-        conditions=[conditions.is_firefox_or_android],
-        parser=create_parser_tests,
-    )
-    def telemetry_test(self, command_context, tests, **kwargs):
-        if "test_objects" in kwargs:
-            tests = []
-            for obj in kwargs["test_objects"]:
-                tests.append(obj["file_relpath"])
-            del kwargs["test_objects"]
-        if not kwargs.get("binary") and conditions.is_firefox(self):
-            try:
-                kwargs["binary"] = self.get_binary_path("app")
-            except BinaryNotFoundException as e:
-                self.log(
-                    logging.ERROR,
-                    "telemetry-tests-client",
-                    {"error": str(e)},
-                    "ERROR: {error}",
-                )
-                self.log(
-                    logging.INFO, "telemetry-tests-client", {"help": e.help()}, "{help}"
-                )
-                return 1
-        if not kwargs.get("server_root"):
-            kwargs[
-                "server_root"
-            ] = "toolkit/components/telemetry/tests/marionette/harness/www"
-        return run_telemetry(tests, topsrcdir=self.topsrcdir, **kwargs)
+@Command(
+    "telemetry-tests-client",
+    category="testing",
+    description="Run tests specifically for the Telemetry client",
+    conditions=[conditions.is_firefox_or_android],
+    parser=create_parser_tests,
+)
+def telemetry_test(command_context, tests, **kwargs):
+    if "test_objects" in kwargs:
+        tests = []
+        for obj in kwargs["test_objects"]:
+            tests.append(obj["file_relpath"])
+        del kwargs["test_objects"]
+    if not kwargs.get("binary") and conditions.is_firefox(command_context):
+        try:
+            kwargs["binary"] = command_context.get_binary_path("app")
+        except BinaryNotFoundException as e:
+            command_context.log(
+                logging.ERROR,
+                "telemetry-tests-client",
+                {"error": str(e)},
+                "ERROR: {error}",
+            )
+            command_context.log(
+                logging.INFO, "telemetry-tests-client", {"help": e.help()}, "{help}"
+            )
+            return 1
+    if not kwargs.get("server_root"):
+        kwargs[
+            "server_root"
+        ] = "toolkit/components/telemetry/tests/marionette/harness/www"
+    return run_telemetry(tests, topsrcdir=command_context.topsrcdir, **kwargs)
