@@ -15,41 +15,41 @@
  * process as fast as possible, without any cleanup.
  */
 
+#include "mozilla/ShutdownPhase.h"
 #include "nsTerminator.h"
 
 #include "prthread.h"
 #include "prmon.h"
-#include "plstr.h"
 #include "prio.h"
 
 #include "nsString.h"
-#include "nsServiceManagerUtils.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsAppDirectoryServiceDefs.h"
 
-#include "nsIObserverService.h"
-#include "nsIPrefService.h"
 #include "nsExceptionHandler.h"
 #include "GeckoProfiler.h"
 #include "nsThreadUtils.h"
+#include "nsXULAppAPI.h"
 
 #if defined(XP_WIN)
-#include <windows.h>
+#  include <windows.h>
 #else
-#include <unistd.h>
+#  include <unistd.h>
 #endif
 
+#include "mozilla/AppShutdown.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Atomics.h"
-#include "mozilla/Attributes.h"
 #include "mozilla/DebugOnly.h"
+#include "mozilla/IntentionalCrash.h"
 #include "mozilla/MemoryChecking.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Services.h"
+#include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
 #include "mozilla/Telemetry.h"
 
+#include "mozilla/dom/IOUtils.h"
 #include "mozilla/dom/workerinternals/RuntimeService.h"
 
 // Normally, the number of milliseconds that AsyncShutdown waits until
@@ -62,6 +62,8 @@
 // forcefully.
 #define ADDITIONAL_WAIT_BEFORE_CRASH_MS 3000
 
+#define HEARTBEAT_INTERVAL_MS 100
+
 namespace mozilla {
 
 namespace {
@@ -74,22 +76,34 @@ namespace {
  * ticks between the time we receive a notification and the next one.
  */
 struct ShutdownStep {
-  char const* const mTopic;
-  int mTicks;
+  mozilla::ShutdownPhase mPhase;
+  Atomic<int> mTicks;
 
-  constexpr explicit ShutdownStep(const char* const topic)
-      : mTopic(topic), mTicks(-1) {}
+  constexpr explicit ShutdownStep(mozilla::ShutdownPhase aPhase)
+      : mPhase(aPhase), mTicks(-1) {}
 };
 
 static ShutdownStep sShutdownSteps[] = {
-    ShutdownStep("quit-application"),
-    ShutdownStep("profile-change-teardown"),
-    ShutdownStep("profile-before-change"),
-    ShutdownStep("xpcom-will-shutdown"),
-    ShutdownStep("xpcom-shutdown"),
+    ShutdownStep(mozilla::ShutdownPhase::AppShutdownConfirmed),
+    ShutdownStep(mozilla::ShutdownPhase::AppShutdownNetTeardown),
+    ShutdownStep(mozilla::ShutdownPhase::AppShutdownTeardown),
+    ShutdownStep(mozilla::ShutdownPhase::AppShutdown),
+    ShutdownStep(mozilla::ShutdownPhase::AppShutdownQM),
+    ShutdownStep(mozilla::ShutdownPhase::XPCOMWillShutdown),
+    ShutdownStep(mozilla::ShutdownPhase::XPCOMShutdown),
+    ShutdownStep(mozilla::ShutdownPhase::XPCOMShutdownThreads),
+    ShutdownStep(mozilla::ShutdownPhase::XPCOMShutdownFinal),
+    ShutdownStep(mozilla::ShutdownPhase::CCPostLastCycleCollection),
 };
 
-Atomic<bool> sShutdownNotified;
+int GetStepForPhase(mozilla::ShutdownPhase aPhase) {
+  for (size_t i = 0; i < std::size(sShutdownSteps); i++) {
+    if (sShutdownSteps[i].mPhase >= aPhase) {
+      return (int)i;
+    }
+  }
+  return -1;
+}
 
 // Utility function: create a thread that is non-joinable,
 // does not prevent the process from terminating, is never
@@ -157,60 +171,77 @@ void RunWatchdog(void* arg) {
 
   const uint32_t timeToLive = crashAfterTicks;
   while (true) {
-  //
-  // We do not want to sleep for the entire duration,
-  // as putting the computer to sleep would suddenly
-  // cause us to timeout on wakeup.
-  //
-  // Rather, we prefer sleeping for at most 1 second
-  // at a time. If the computer sleeps then wakes up,
-  // we have lost at most one second, which is much
-  // more reasonable.
-  //
+    //
+    // We do not want to sleep for the entire duration,
+    // as putting the computer to sleep would suddenly
+    // cause us to timeout on wakeup.
+    //
+    // Rather, we prefer sleeping for at most 1 second
+    // at a time. If the computer sleeps then wakes up,
+    // we have lost at most one second, which is much
+    // more reasonable.
+    //
 #if defined(XP_WIN)
-    Sleep(1000 /* ms */);
+    Sleep(HEARTBEAT_INTERVAL_MS /* ms */);
 #else
-    usleep(1000000 /* usec */);
+    usleep(HEARTBEAT_INTERVAL_MS * 1000 /* usec */);
 #endif
 
     if (gHeartbeat++ < timeToLive) {
       continue;
     }
 
-    // The shutdown steps are not completed yet. Let's report the last one.
-    if (!sShutdownNotified) {
-      const char* lastStep = nullptr;
-      for (size_t i = 0; i < ArrayLength(sShutdownSteps); ++i) {
-        if (sShutdownSteps[i].mTicks == -1) {
-          break;
-        }
-        lastStep = sShutdownSteps[i].mTopic;
-      }
+    // Arrived here we know we will crash in a way or another.
+    NoteIntentionalCrash(XRE_GetProcessTypeString());
 
-      if (lastStep) {
-        nsCString msg;
-        msg.AppendPrintf(
-            "Shutdown hanging at step %s. "
-            "Something is blocking the main-thread.",
-            lastStep);
-        // This string will be leaked.
-        MOZ_CRASH_UNSAFE_OOL(strdup(msg.BeginReading()));
-      }
+    // Until we have general log output for crash annotations in treeherder
+    // (bug 1728721) we manually spit out our nested event loop stack.
+    // XXX: Remove once bug 1728721 is fixed.
+    nsCString stack;
+    AutoNestedEventLoopAnnotation::CopyCurrentStack(stack);
+    printf_stderr(
+        "RunWatchdog: Mainthread nested event loops during hang: \n --- %s\n",
+        stack.get());
 
-      MOZ_CRASH("Shutdown hanging before starting.");
+    // Let's find the last known shutdown phase.
+    mozilla::ShutdownPhase lastPhase = mozilla::ShutdownPhase::NotInShutdown;
+    // Looping inverse here to make the search more robust in case
+    // the observer that triggers UpdateHeartbeat was not called
+    // at all or in the expected order on some step. This should
+    // give us always the last known ShutdownStep.
+    for (int i = ArrayLength(sShutdownSteps) - 1; i >= 0; --i) {
+      if (sShutdownSteps[i].mTicks > -1) {
+        lastPhase = sShutdownSteps[i].mPhase;
+        break;
+      }
     }
 
-    // Maybe some workers are blocking the shutdown.
+    if (lastPhase == mozilla::ShutdownPhase::NotInShutdown) {
+      // This is not something we expect to ever happen, but still.
+      CrashReporter::SetMinidumpAnalysisAllThreads();
+      MOZ_CRASH("Shutdown hanging before starting any known phase.");
+    }
+
+    // First check if worker shutdown started and is incomplete, in case
+    // report running workers.
     mozilla::dom::workerinternals::RuntimeService* runtimeService =
         mozilla::dom::workerinternals::RuntimeService::GetService();
     if (runtimeService) {
+      // CrashIfHanging will check if we actually ever asked for worker
+      // shutdown, so calling it before is a no-op.
       runtimeService->CrashIfHanging();
     }
 
-    // Shutdown is apparently dead. Crash the process.
-    CrashReporter::SetMinidumpAnalysisAllThreads();
+    // Otherwise just report our shutdown phase.
+    // This string will be leaked.
+    nsCString msg;
+    msg.AppendPrintf(
+        "Shutdown hanging at step %s. "
+        "Something is blocking the main-thread.",
+        mozilla::AppShutdown::GetShutdownPhaseName(lastPhase));
 
-    MOZ_CRASH("Shutdown too long, probably frozen, causing a crash.");
+    CrashReporter::SetMinidumpAnalysisAllThreads();
+    MOZ_CRASH_UNSAFE(strdup(msg.BeginReading()));
   }
 }
 
@@ -224,16 +255,6 @@ void RunWatchdog(void* arg) {
 // threads.
 //
 
-// Utility class, used by UniquePtr<> to close nspr files.
-class PR_CloseDelete {
- public:
-  constexpr PR_CloseDelete() = default;
-
-  PR_CloseDelete(const PR_CloseDelete& aOther) = default;
-
-  void operator()(PRFileDesc* aPtr) const { PR_Close(aPtr); }
-};
-
 //
 // Communication between the main thread and the writer thread.
 //
@@ -339,6 +360,7 @@ void RunWriter(void* arg) {
     // will be written correctly, but, again, we don't care enough
     // about the data to make more efforts.
     //
+    Unused << PR_Delete(destinationPath.get());
     if (PR_Rename(tmpFilePath.get(), destinationPath.get()) != PR_SUCCESS) {
       break;
     }
@@ -351,25 +373,11 @@ NS_IMPL_ISUPPORTS(nsTerminator, nsIObserver)
 
 nsTerminator::nsTerminator() : mInitialized(false), mCurrentStep(-1) {}
 
-// During startup, register as an observer for all interesting topics.
-nsresult nsTerminator::SelfInit() {
-  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
-  if (!os) {
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  for (auto& shutdownStep : sShutdownSteps) {
-    DebugOnly<nsresult> rv = os->AddObserver(this, shutdownStep.mTopic, false);
-    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "AddObserver failed");
-  }
-
-  return NS_OK;
-}
-
 // Actually launch these threads. This takes place at the first sign of
 // shutdown.
 void nsTerminator::Start() {
   MOZ_ASSERT(!mInitialized);
+
   StartWatchdog();
 #if !defined(NS_FREE_PERMANENT_DATA)
   // Only allow nsTerminator to write on non-leak-checked builds so we don't
@@ -378,7 +386,39 @@ void nsTerminator::Start() {
   StartWriter();
 #endif  // !defined(NS_FREE_PERMANENT_DATA)
   mInitialized = true;
-  sShutdownNotified = false;
+}
+
+NS_IMETHODIMP
+nsTerminator::Observe(nsISupports*, const char* aTopic, const char16_t*) {
+  // This Observe is now only used for testing purposes.
+  // XXX: Check if we should change our testing strategy.
+  if (strcmp(aTopic, "terminator-test-quit-application") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::AppShutdownConfirmed);
+  } else if (strcmp(aTopic, "terminator-test-profile-change-net-teardown") ==
+             0) {
+    AdvancePhase(mozilla::ShutdownPhase::AppShutdownNetTeardown);
+  } else if (strcmp(aTopic, "terminator-test-profile-change-teardown") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::AppShutdownTeardown);
+  } else if (strcmp(aTopic, "terminator-test-profile-before-change") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::AppShutdown);
+  } else if (strcmp(aTopic, "terminator-test-profile-before-change-qm") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::AppShutdownQM);
+  } else if (strcmp(aTopic,
+                    "terminator-test-profile-before-change-telemetry") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::AppShutdownTelemetry);
+  } else if (strcmp(aTopic, "terminator-test-xpcom-will-shutdown") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::XPCOMWillShutdown);
+  } else if (strcmp(aTopic, "terminator-test-xpcom-shutdown") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::XPCOMShutdown);
+  } else if (strcmp(aTopic, "terminator-test-xpcom-shutdown-threads") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::XPCOMShutdownThreads);
+  } else if (strcmp(aTopic, "terminator-test-XPCOMShutdownFinal") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::XPCOMShutdownFinal);
+  } else if (strcmp(aTopic, "terminator-test-CCPostLastCycleCollection") == 0) {
+    AdvancePhase(mozilla::ShutdownPhase::CCPostLastCycleCollection);
+  }
+
+  return NS_OK;
 }
 
 // Prepare, allocate and start the watchdog thread.
@@ -421,12 +461,9 @@ void nsTerminator::StartWatchdog() {
 #endif
 
   UniquePtr<Options> options(new Options());
-  const PRIntervalTime ticksDuration = PR_MillisecondsToInterval(1000);
-  options->crashAfterTicks = crashAfterMS / ticksDuration;
-  // Handle systems where ticksDuration is greater than crashAfterMS.
-  if (options->crashAfterTicks == 0) {
-    options->crashAfterTicks = crashAfterMS / 1000;
-  }
+  // crashAfterTicks is guaranteed to be > 0 as
+  // crashAfterMS >= ADDITIONAL_WAIT_BEFORE_CRASH_MS >> HEARTBEAT_INTERVAL_MS
+  options->crashAfterTicks = crashAfterMS / HEARTBEAT_INTERVAL_MS;
 
   DebugOnly<PRThread*> watchdogThread =
       CreateSystemThread(RunWatchdog, options.release());
@@ -447,7 +484,7 @@ void nsTerminator::StartWriter() {
     return;
   }
 
-  rv = profLD->Append(NS_LITERAL_STRING("ShutdownDuration.json"));
+  rv = profLD->Append(u"ShutdownDuration.json"_ns);
   if (NS_FAILED(rv)) {
     return;
   }
@@ -468,13 +505,23 @@ void nsTerminator::StartWriter() {
   }
 }
 
-NS_IMETHODIMP
-nsTerminator::Observe(nsISupports*, const char* aTopic, const char16_t*) {
-  if (strcmp(aTopic, "profile-after-change") == 0) {
-    return SelfInit();
+// This helper is here to preserve the existing crash reporting behavior
+// based on observer topic names, using the shutdown phase name only for
+// phases without associated topic.
+const char* GetReadableNameForPhase(mozilla::ShutdownPhase aPhase) {
+  const char* readableName = mozilla::AppShutdown::GetObserverKey(aPhase);
+  if (!readableName) {
+    readableName = mozilla::AppShutdown::GetShutdownPhaseName(aPhase);
   }
+  return readableName;
+}
 
-  // Other notifications are shutdown-related.
+void nsTerminator::AdvancePhase(mozilla::ShutdownPhase aPhase) {
+  // If the phase is unknown, just ignore it.
+  auto step = GetStepForPhase(aPhase);
+  if (step < 0) {
+    return;
+  }
 
   // As we have seen examples in the wild of shutdown notifications
   // not being sent (or not being sent in the expected order), we do
@@ -483,41 +530,29 @@ nsTerminator::Observe(nsISupports*, const char* aTopic, const char16_t*) {
     Start();
   }
 
-  UpdateHeartbeat(aTopic);
+  UpdateHeartbeat(step);
 #if !defined(NS_FREE_PERMANENT_DATA)
   // Only allow nsTerminator to write on non-leak checked builds so we don't get
   // leak warnings on shutdown for intentional leaks (see bug 1242084). This
   // will be enabled again by bug 1255484 when 1255478 lands.
   UpdateTelemetry();
 #endif  // !defined(NS_FREE_PERMANENT_DATA)
-  UpdateCrashReport(aTopic);
-
-  // Perform a little cleanup
-  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
-  MOZ_RELEASE_ASSERT(os);
-  (void)os->RemoveObserver(this, aTopic);
-
-  return NS_OK;
+  UpdateCrashReport(GetReadableNameForPhase(aPhase));
 }
 
-void nsTerminator::UpdateHeartbeat(const char* aTopic) {
-  // Reset the clock, find out how long the current phase has lasted.
-  uint32_t ticks = gHeartbeat.exchange(0);
-  if (mCurrentStep > 0) {
-    sShutdownSteps[mCurrentStep].mTicks = ticks;
-  }
+void nsTerminator::UpdateHeartbeat(int32_t aStep) {
+  MOZ_ASSERT(aStep >= mCurrentStep);
 
-  // Find out where we now are in the current shutdown.
-  // Don't assume that shutdown takes place in the expected order.
-  int nextStep = -1;
-  for (size_t i = 0; i < ArrayLength(sShutdownSteps); ++i) {
-    if (strcmp(sShutdownSteps[i].mTopic, aTopic) == 0) {
-      nextStep = i;
-      break;
+  if (aStep > mCurrentStep) {
+    // Reset the clock, find out how long the current phase has lasted.
+    uint32_t ticks = gHeartbeat.exchange(0);
+    if (mCurrentStep >= 0) {
+      sShutdownSteps[mCurrentStep].mTicks = ticks;
     }
+    sShutdownSteps[aStep].mTicks = 0;
+
+    mCurrentStep = aStep;
   }
-  MOZ_ASSERT(nextStep != -1);
-  mCurrentStep = nextStep;
 }
 
 void nsTerminator::UpdateTelemetry() {
@@ -546,7 +581,7 @@ void nsTerminator::UpdateTelemetry() {
       telemetryData->AppendLiteral(", ");
     }
     telemetryData->AppendLiteral(R"(")");
-    telemetryData->Append(shutdownStep.mTopic);
+    telemetryData->Append(GetReadableNameForPhase(shutdownStep.mPhase));
     telemetryData->AppendLiteral(R"(": )");
     telemetryData->AppendInt(shutdownStep.mTicks);
   }
@@ -574,12 +609,6 @@ void nsTerminator::UpdateCrashReport(const char* aTopic) {
   nsAutoCString report(aTopic);
 
   Unused << CrashReporter::AnnotateCrashReport(
-      NS_LITERAL_CSTRING("ShutdownProgress"), report);
+      CrashReporter::Annotation::ShutdownProgress, report);
 }
-
-void XPCOMShutdownNotified() {
-  MOZ_DIAGNOSTIC_ASSERT(sShutdownNotified == false);
-  sShutdownNotified = true;
-}
-
 }  // namespace mozilla
