@@ -17,7 +17,7 @@ extern mozilla::LazyLogModule gUrlClassifierDbServiceLog;
 #define LOG_ENABLED() \
   MOZ_LOG_TEST(gUrlClassifierDbServiceLog, mozilla::LogLevel::Debug)
 
-#define METADATA_SUFFIX NS_LITERAL_CSTRING(".metadata")
+#define METADATA_SUFFIX ".metadata"_ns
 
 namespace mozilla {
 namespace safebrowsing {
@@ -162,7 +162,7 @@ nsresult LookupCacheV4::ClearLegacyFile() {
     return rv;
   }
 
-  rv = file->AppendNative(mTableName + NS_LITERAL_CSTRING(".pset"));
+  rv = file->AppendNative(mTableName + ".pset"_ns);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -192,7 +192,7 @@ nsresult LookupCacheV4::LoadLegacyFile() {
     return rv;
   }
 
-  rv = file->AppendNative(mTableName + NS_LITERAL_CSTRING(".pset"));
+  rv = file->AppendNative(mTableName + ".pset"_ns);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -264,9 +264,7 @@ nsresult LookupCacheV4::SanityCheck(const Header& aHeader) {
   return NS_OK;
 }
 
-nsCString LookupCacheV4::GetPrefixSetSuffix() const {
-  return NS_LITERAL_CSTRING(".vlpset");
-}
+nsCString LookupCacheV4::GetPrefixSetSuffix() const { return ".vlpset"_ns; }
 
 static nsresult AppendPrefixToMap(PrefixStringMap& prefixes,
                                   const nsACString& prefix) {
@@ -276,7 +274,7 @@ static nsresult AppendPrefixToMap(PrefixStringMap& prefixes,
     return NS_OK;
   }
 
-  nsCString* prefixString = prefixes.LookupOrAdd(len);
+  nsCString* prefixString = prefixes.GetOrInsertNew(len);
   if (!prefixString->Append(prefix, fallible)) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -514,19 +512,19 @@ nsresult LookupCacheV4::LoadMetadata(nsACString& aState, nsACString& aSHA256) {
 }
 
 VLPrefixSet::VLPrefixSet(const PrefixStringMap& aMap) : mCount(0) {
-  for (auto iter = aMap.ConstIter(); !iter.Done(); iter.Next()) {
-    uint32_t size = iter.Key();
-    MOZ_ASSERT(iter.Data()->Length() % size == 0,
+  for (const auto& entry : aMap) {
+    uint32_t size = entry.GetKey();
+    MOZ_ASSERT(entry.GetData()->Length() % size == 0,
                "PrefixString must be a multiple of the prefix size.");
-    mMap.Put(size, new PrefixString(*iter.Data(), size));
-    mCount += iter.Data()->Length() / size;
+    mMap.InsertOrUpdate(size, MakeUnique<PrefixString>(*entry.GetData(), size));
+    mCount += entry.GetData()->Length() / size;
   }
 }
 
 void VLPrefixSet::Merge(PrefixStringMap& aPrefixMap) {
-  for (auto iter = mMap.ConstIter(); !iter.Done(); iter.Next()) {
-    nsCString* prefixString = aPrefixMap.LookupOrAdd(iter.Key());
-    PrefixString* str = iter.UserData();
+  for (const auto& entry : mMap) {
+    nsCString* prefixString = aPrefixMap.GetOrInsertNew(entry.GetKey());
+    PrefixString* str = entry.GetWeak();
 
     nsAutoCString remainingString;
     str->getRemainingString(remainingString);
@@ -539,8 +537,8 @@ void VLPrefixSet::Merge(PrefixStringMap& aPrefixMap) {
 
 bool VLPrefixSet::GetSmallestPrefix(nsACString& aOutString) const {
   PrefixString* pick = nullptr;
-  for (auto iter = mMap.ConstIter(); !iter.Done(); iter.Next()) {
-    PrefixString* str = iter.UserData();
+  for (const auto& entry : mMap) {
+    PrefixString* str = entry.GetWeak();
 
     if (str->remaining() <= 0) {
       continue;
@@ -548,7 +546,7 @@ bool VLPrefixSet::GetSmallestPrefix(nsACString& aOutString) const {
 
     if (aOutString.IsEmpty()) {
       str->getPrefix(aOutString);
-      MOZ_ASSERT(aOutString.Length() == iter.Key());
+      MOZ_ASSERT(aOutString.Length() == entry.GetKey());
       pick = str;
       continue;
     }
@@ -557,7 +555,7 @@ bool VLPrefixSet::GetSmallestPrefix(nsACString& aOutString) const {
     str->getPrefix(cur);
     if (!cur.IsEmpty() && cur < aOutString) {
       aOutString.Assign(cur);
-      MOZ_ASSERT(aOutString.Length() == iter.Key());
+      MOZ_ASSERT(aOutString.Length() == entry.GetKey());
       pick = str;
     }
   }
