@@ -8,13 +8,10 @@
 #include "nsIInputStream.h"
 #include "nsUrlClassifierPrefixSet.h"
 #include "nsPrintfCString.h"
-#include "nsThreadUtils.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/EndianUtils.h"
 #include "mozilla/Logging.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/Unused.h"
 #include <algorithm>
 
 // MOZ_LOG=UrlClassifierPrefixSet:5
