@@ -30,6 +30,11 @@ function closeWindow(aClose, aPromptFunction, aSource) {
     ) {
       return false;
     }
+
+    // If the user explicitly closes the last tabs in the window close remaining tabs. Bug 490136
+    if (aClose) {
+      window.SessionStore?.maybeDontSaveTabs(window);
+    }
   } else if (
     typeof aPromptFunction == "function" &&
     !aPromptFunction(aSource)
