@@ -15,6 +15,7 @@
     static get inheritedAttributes() {
       return {
         "#scrollbutton-up": "disabled=scrolledtostart",
+        ".scrollbox-clip": "orient",
         scrollbox: "orient,align,pack,dir,smoothscroll",
         "#scrollbutton-down": "disabled=scrolledtoend",
       };
@@ -26,9 +27,11 @@
       <html:link rel="stylesheet" href="chrome://global/skin/arrowscrollbox.css"/>
       <toolbarbutton id="scrollbutton-up" part="scrollbutton-up"/>
       <spacer part="overflow-start-indicator"/>
-      <scrollbox part="scrollbox" flex="1">
-        <html:slot/>
-      </scrollbox>
+      <box class="scrollbox-clip" part="scrollbox-clip" flex="1">
+        <scrollbox part="scrollbox" flex="1">
+          <html:slot/>
+        </scrollbox>
+      </box>
       <spacer part="overflow-end-indicator"/>
       <toolbarbutton id="scrollbutton-down" part="scrollbutton-down"/>
     `;
@@ -185,7 +188,6 @@
 
     set smoothScroll(val) {
       this.setAttribute("smoothscroll", !!val);
-      return val;
     }
 
     get smoothScroll() {
@@ -212,7 +214,7 @@
       // line scroll amout should be the width (at horizontal scrollbox) or
       // the height (at vertical scrollbox) of the scrolled elements.
       // However, the elements may have different width or height.  So,
-      // for consistent speed, let's use avalage with of the elements.
+      // for consistent speed, let's use average width of the elements.
       var elements = this._getScrollableElements();
       return elements.length && this.scrollSize / elements.length;
     }
@@ -586,34 +588,35 @@
             scrolledToStart = true;
             scrolledToEnd = true;
           } else {
-            let [leftOrTop, rightOrBottom] = this.startEndProps;
-            let leftOrTopEdge = ele =>
-              Math.round(this._boundsWithoutFlushing(ele)[leftOrTop]);
-            let rightOrBottomEdge = ele =>
-              Math.round(this._boundsWithoutFlushing(ele)[rightOrBottom]);
+            let isAtEdge = (element, start) => {
+              let edge = start ? this.startEndProps[0] : this.startEndProps[1];
+              let scrollEdge = this._boundsWithoutFlushing(this.scrollbox)[
+                edge
+              ];
+              let elementEdge = this._boundsWithoutFlushing(element)[edge];
+              // This is enough slop (>2/3) so that no subpixel value should
+              // get us confused about whether we reached the end.
+              const EPSILON = 0.7;
+              if (start) {
+                return scrollEdge <= elementEdge + EPSILON;
+              }
+              return elementEdge <= scrollEdge + EPSILON;
+            };
 
             let elements = this._getScrollableElements();
-            let [leftOrTopElement, rightOrBottomElement] = [
+            let [startElement, endElement] = [
               elements[0],
               elements[elements.length - 1],
             ];
             if (this.isRTLScrollbox) {
-              [leftOrTopElement, rightOrBottomElement] = [
-                rightOrBottomElement,
-                leftOrTopElement,
-              ];
+              [startElement, endElement] = [endElement, startElement];
             }
-
-            if (
-              leftOrTopElement &&
-              leftOrTopEdge(leftOrTopElement) >= leftOrTopEdge(this.scrollbox)
-            ) {
+            if (startElement && isAtEdge(startElement, /* start = */ true)) {
               scrolledToStart = !this.isRTLScrollbox;
               scrolledToEnd = this.isRTLScrollbox;
             } else if (
-              rightOrBottomElement &&
-              rightOrBottomEdge(rightOrBottomElement) <=
-                rightOrBottomEdge(this.scrollbox)
+              endElement &&
+              isAtEdge(endElement, /* start = */ false)
             ) {
               scrolledToStart = this.isRTLScrollbox;
               scrolledToEnd = !this.isRTLScrollbox;
@@ -649,18 +652,13 @@
         return;
       }
 
+      const { deltaMode } = event;
       let doScroll = false;
       let instant;
       let scrollAmount = 0;
       if (this.getAttribute("orient") == "vertical") {
         doScroll = true;
-        if (event.deltaMode == event.DOM_DELTA_PIXEL) {
-          scrollAmount = event.deltaY;
-        } else if (event.deltaMode == event.DOM_DELTA_PAGE) {
-          scrollAmount = event.deltaY * this.scrollClientSize;
-        } else {
-          scrollAmount = event.deltaY * this.lineScrollAmount;
-        }
+        scrollAmount = event.deltaY;
       } else {
         // We allow vertical scrolling to scroll a horizontal scrollbox
         // because many users have a vertical scroll wheel but no
@@ -676,13 +674,9 @@
 
         if (this._prevMouseScrolls.every(prev => prev == isVertical)) {
           doScroll = true;
-          if (event.deltaMode == event.DOM_DELTA_PIXEL) {
-            scrollAmount = scrollByDelta;
+          scrollAmount = scrollByDelta;
+          if (deltaMode == event.DOM_DELTA_PIXEL) {
             instant = true;
-          } else if (event.deltaMode == event.DOM_DELTA_PAGE) {
-            scrollAmount = scrollByDelta * this.scrollClientSize;
-          } else {
-            scrollAmount = scrollByDelta * this.lineScrollAmount;
           }
         }
 
@@ -694,6 +688,24 @@
 
       if (doScroll) {
         let direction = scrollAmount < 0 ? -1 : 1;
+
+        if (deltaMode == event.DOM_DELTA_PAGE) {
+          scrollAmount *= this.scrollClientSize;
+        } else if (deltaMode == event.DOM_DELTA_LINE) {
+          // Try to not scroll by more than one page when using line scrolling,
+          // so that all elements are scrollable.
+          let lineAmount = this.lineScrollAmount;
+          let clientSize = this.scrollClientSize;
+          if (Math.abs(scrollAmount * lineAmount) > clientSize) {
+            // NOTE: This still tries to scroll a non-fractional amount of
+            // items per line scrolled.
+            scrollAmount =
+              Math.max(1, Math.floor(clientSize / lineAmount)) * direction;
+          }
+          scrollAmount *= lineAmount;
+        } else {
+          // DOM_DELTA_PIXEL, leave scrollAmount untouched.
+        }
         let startPos = this.scrollPosition;
 
         if (!this._isScrolling || this._direction != direction) {
