@@ -53,7 +53,7 @@ static string GenerateUUID() {
 
   CFRelease(uuid);
 #elif defined(HAVE_ARC4RANDOM_BUF)  // Android, BSD, ...
-  arc4random_buf(id, sizeof(UUID));
+  arc4random_buf(&id, sizeof(UUID));
 #else                               // Linux
   int fd = open("/dev/urandom", O_RDONLY);
 
@@ -123,7 +123,7 @@ static Json::Value CreateMetadataNode(const Json::Value& aExtra) {
     Annotation annotation;
 
     if (AnnotationFromString(annotation, iter.memberName())) {
-      if (IsAnnotationWhitelistedForPing(annotation)) {
+      if (IsAnnotationAllowlistedForPing(annotation)) {
         node[iter.memberName()] = *iter;
       }
     }
