@@ -2,7 +2,13 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+extern crate bench_collections_gtest;
+extern crate fog_gtest;
+#[cfg(feature = "libfuzzer")]
+extern crate gecko_fuzz_targets;
 extern crate gkrust_shared;
+extern crate l10nregistry_ffi_gtest;
+extern crate moz_task_gtest;
 extern crate mp4parse_gtest;
 extern crate nsstring_gtest;
 extern crate xpcom_gtest;
