@@ -1,16 +1,24 @@
 "use strict";
 
-do_get_profile();
+const PROFILE_DIR = do_get_profile().path;
 
-const { Promise } = ChromeUtils.import("resource://gre/modules/Promise.jsm");
 const { PromiseUtils } = ChromeUtils.import(
   "resource://gre/modules/PromiseUtils.jsm"
 );
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
 const { FileUtils } = ChromeUtils.import(
   "resource://gre/modules/FileUtils.jsm"
 );
 const { Sqlite } = ChromeUtils.import("resource://gre/modules/Sqlite.jsm");
+const { TelemetryTestUtils } = ChromeUtils.import(
+  "resource://testing-common/TelemetryTestUtils.jsm"
+);
+
+// Enable the collection (during test) for all products so even products
+// that don't collect the data will be able to run the test without failure.
+Services.prefs.setBoolPref(
+  "toolkit.telemetry.testing.overrideProductsCheck",
+  true
+);
 
 function sleep(ms) {
   return new Promise(resolve => {
@@ -104,19 +112,20 @@ add_task(async function test_open_with_defaultTransactionType() {
 });
 
 add_task(async function test_open_normal_error() {
-  let currentDir = await OS.File.getCurrentDirectory();
+  let currentDir = do_get_cwd().path;
 
-  let src = OS.Path.join(currentDir, "corrupt.sqlite");
-  Assert.ok(await OS.File.exists(src), "Database file found");
+  let src = PathUtils.join(currentDir, "corrupt.sqlite");
+  Assert.ok(await IOUtils.exists(src), "Database file found");
 
   // Ensure that our database doesn't already exist.
-  let path = OS.Path.join(OS.Constants.Path.profileDir, "corrupt.sqlite");
-  Assert.ok(
-    !(await OS.File.exists(path)),
+  let path = PathUtils.join(PROFILE_DIR, "corrupt.sqlite");
+  await Assert.rejects(
+    IOUtils.stat(path),
+    /Could not stat file\(.*\) because it does not exist/,
     "Database file should not exist yet"
   );
 
-  await OS.File.copy(src, path);
+  await IOUtils.copy(src, path);
 
   let openPromise = Sqlite.openConnection({ path });
   await Assert.rejects(
@@ -129,10 +138,7 @@ add_task(async function test_open_normal_error() {
 });
 
 add_task(async function test_open_unshared() {
-  let path = OS.Path.join(
-    OS.Constants.Path.profileDir,
-    "test_open_unshared.sqlite"
-  );
+  let path = PathUtils.join(PROFILE_DIR, "test_open_unshared.sqlite");
 
   let c = await Sqlite.openConnection({ path, sharedMemoryCache: false });
   await c.close();
@@ -456,7 +462,7 @@ add_task(async function test_execute_transaction_success() {
 add_task(async function test_execute_transaction_rollback() {
   let c = await getDummyDatabase("execute_transaction_rollback");
 
-  let deferred = Promise.defer();
+  let deferred = PromiseUtils.defer();
 
   c.executeTransaction(async function transaction(conn) {
     await conn.execute("INSERT INTO dirs (path) VALUES ('foo')");
@@ -532,19 +538,21 @@ add_task(async function test_multiple_transactions() {
 // an externally opened transaction exists.
 add_task(async function test_wrapped_connection_transaction() {
   let file = new FileUtils.File(
-    OS.Path.join(
-      OS.Constants.Path.profileDir,
-      "test_wrapStorageConnection.sqlite"
-    )
+    PathUtils.join(PROFILE_DIR, "test_wrapStorageConnection.sqlite")
   );
   let c = await new Promise((resolve, reject) => {
-    Services.storage.openAsyncDatabase(file, null, (status, db) => {
-      if (Components.isSuccessCode(status)) {
-        resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
-      } else {
-        reject(new Error(status));
+    Services.storage.openAsyncDatabase(
+      file,
+      /* openFlags */ Ci.mozIStorageService.OPEN_DEFAULT,
+      /* connectionFlags */ Ci.mozIStorageService.CONNECTION_DEFAULT,
+      (status, db) => {
+        if (Components.isSuccessCode(status)) {
+          resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
+        } else {
+          reject(new Error(status));
+        }
       }
-    });
+    );
   });
 
   let wrapper = await Sqlite.wrapStorageConnection({ connection: c });
@@ -567,6 +575,51 @@ add_task(async function test_wrapped_connection_transaction() {
   await c.asyncClose();
 });
 
+add_task(async function test_transaction_timeout() {
+  // Lower the transactions timeout for the test.
+  let defaultTimeout = Sqlite.TRANSACTIONS_TIMEOUT_MS;
+  Sqlite.TRANSACTIONS_TIMEOUT_MS = 500;
+  Services.telemetry.clearScalars();
+  let myResolve = () => {};
+  try {
+    let c = await getDummyDatabase("transaction_timeout");
+    Assert.ok(!c.transactionInProgress, "Should not be in a transaction");
+    let promise = c.executeTransaction(async function transaction(conn) {
+      // Make a change, we'll later check it is undone by ROLLBACK.
+      await conn.execute(
+        "CREATE TABLE test (id INTEGER PRIMARY KEY AUTOINCREMENT)"
+      );
+      Assert.ok(c.transactionInProgress, "Should be in a transaction");
+      // Return a never fulfilled promise.
+      await new Promise(resolve => {
+        // Keep this alive regardless GC, and clean it up in finally.
+        myResolve = resolve;
+      });
+    });
+
+    await Assert.rejects(
+      promise,
+      /Transaction timeout, most likely caused by unresolved pending work./,
+      "A transaction timeout should reject it"
+    );
+
+    let rows = await c.execute("SELECT * FROM dirs");
+    Assert.equal(rows.length, 0, "Changes should have been rolled back");
+    await c.close();
+
+    let scalars = TelemetryTestUtils.getProcessScalars("parent", true, true);
+    TelemetryTestUtils.assertKeyedScalar(
+      scalars,
+      "mozstorage.sqlitejsm_transaction_timeout",
+      "test_transaction_timeout@test_sqlite.js",
+      1
+    );
+  } finally {
+    Sqlite.TRANSACTIONS_TIMEOUT_MS = defaultTimeout;
+    myResolve();
+  }
+});
+
 add_task(async function test_shrink_memory() {
   let c = await getDummyDatabase("shrink_memory");
 
@@ -868,7 +921,11 @@ add_task(
       });
     } catch (ex) {
       print("Caught expected exception: " + ex);
-      Assert.ok(ex.result, "The ex.result value should be forwarded.");
+      Assert.greater(
+        ex.result,
+        0x80000000,
+        "The ex.result value should be forwarded."
+      );
     }
 
     // We did not get to the end of our in-transaction block.
@@ -901,7 +958,11 @@ add_task(async function test_programmatic_binding_implicit_transaction() {
     secondSucceeded = true;
   } catch (ex) {
     print("Caught expected exception: " + ex);
-    Assert.ok(ex.result, "The ex.result value should be forwarded.");
+    Assert.greater(
+      ex.result,
+      0x80000000,
+      "The ex.result value should be forwarded."
+    );
   }
 
   Assert.ok(!secondSucceeded);
@@ -943,7 +1004,7 @@ add_task(async function test_direct() {
   let begin = db.createAsyncStatement("BEGIN DEFERRED TRANSACTION");
   let end = db.createAsyncStatement("COMMIT TRANSACTION");
 
-  let deferred = Promise.defer();
+  let deferred = PromiseUtils.defer();
   begin.executeAsync({
     handleCompletion(reason) {
       deferred.resolve();
@@ -953,7 +1014,7 @@ add_task(async function test_direct() {
 
   statement.bindParameters(params);
 
-  deferred = Promise.defer();
+  deferred = PromiseUtils.defer();
   print("Executing async.");
   statement.executeAsync({
     handleResult(resultSet) {},
@@ -974,7 +1035,7 @@ add_task(async function test_direct() {
 
   await deferred.promise;
 
-  deferred = Promise.defer();
+  deferred = PromiseUtils.defer();
   end.executeAsync({
     handleCompletion(reason) {
       deferred.resolve();
@@ -986,7 +1047,7 @@ add_task(async function test_direct() {
   begin.finalize();
   end.finalize();
 
-  deferred = Promise.defer();
+  deferred = PromiseUtils.defer();
   db.asyncClose(function() {
     deferred.resolve();
   });
@@ -996,19 +1057,21 @@ add_task(async function test_direct() {
 // Test Sqlite.cloneStorageConnection.
 add_task(async function test_cloneStorageConnection() {
   let file = new FileUtils.File(
-    OS.Path.join(
-      OS.Constants.Path.profileDir,
-      "test_cloneStorageConnection.sqlite"
-    )
+    PathUtils.join(PROFILE_DIR, "test_cloneStorageConnection.sqlite")
   );
   let c = await new Promise((resolve, reject) => {
-    Services.storage.openAsyncDatabase(file, null, (status, db) => {
-      if (Components.isSuccessCode(status)) {
-        resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
-      } else {
-        reject(new Error(status));
+    Services.storage.openAsyncDatabase(
+      file,
+      /* openFlags */ Ci.mozIStorageService.OPEN_DEFAULT,
+      /* connectionFlags */ Ci.mozIStorageService.CONNECTION_DEFAULT,
+      (status, db) => {
+        if (Components.isSuccessCode(status)) {
+          resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
+        } else {
+          reject(new Error(status));
+        }
       }
-    });
+    );
   });
 
   let clone = await Sqlite.cloneStorageConnection({
@@ -1062,10 +1125,7 @@ add_task(async function test_clone() {
 
 // Test clone(readOnly) method.
 add_task(async function test_readOnly_clone() {
-  let path = OS.Path.join(
-    OS.Constants.Path.profileDir,
-    "test_readOnly_clone.sqlite"
-  );
+  let path = PathUtils.join(PROFILE_DIR, "test_readOnly_clone.sqlite");
   let c = await Sqlite.openConnection({ path, sharedMemoryCache: false });
 
   let clone = await c.clone(true);
@@ -1085,19 +1145,21 @@ add_task(async function test_readOnly_clone() {
 // Test Sqlite.wrapStorageConnection.
 add_task(async function test_wrapStorageConnection() {
   let file = new FileUtils.File(
-    OS.Path.join(
-      OS.Constants.Path.profileDir,
-      "test_wrapStorageConnection.sqlite"
-    )
+    PathUtils.join(PROFILE_DIR, "test_wrapStorageConnection.sqlite")
   );
   let c = await new Promise((resolve, reject) => {
-    Services.storage.openAsyncDatabase(file, null, (status, db) => {
-      if (Components.isSuccessCode(status)) {
-        resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
-      } else {
-        reject(new Error(status));
+    Services.storage.openAsyncDatabase(
+      file,
+      /* openFlags */ Ci.mozIStorageService.OPEN_DEFAULT,
+      /* connectionFlags */ Ci.mozIStorageService.CONNECTION_DEFAULT,
+      (status, db) => {
+        if (Components.isSuccessCode(status)) {
+          resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
+        } else {
+          reject(new Error(status));
+        }
       }
-    });
+    );
   });
 
   let wrapper = await Sqlite.wrapStorageConnection({ connection: c });
@@ -1145,7 +1207,7 @@ add_task(async function test_warning_message_on_finalization() {
   failTestsOnAutoClose(false);
   let c = await getDummyDatabase("warning_message_on_finalization");
   let identifier = c._connectionData._identifier;
-  let deferred = Promise.defer();
+  let deferred = PromiseUtils.defer();
 
   let listener = {
     observe(msg) {
@@ -1173,7 +1235,7 @@ add_task(async function test_warning_message_on_finalization() {
 
 add_task(async function test_error_message_on_unknown_finalization() {
   failTestsOnAutoClose(false);
-  let deferred = Promise.defer();
+  let deferred = PromiseUtils.defer();
 
   let listener = {
     observe(msg) {
@@ -1308,11 +1370,8 @@ add_task(async function test_interrupt() {
   // Testing the interrupt functionality is left to mozStorage unit tests, here
   // we'll just test error conditions.
   let c = await getDummyDatabase("interrupt");
-  Assert.throws(
-    () => c.interrupt(),
-    /NS_ERROR_ILLEGAL_VALUE/,
-    "Sqlite.interrupt() should throw on a writable connection"
-  );
+  await c.interrupt();
+  ok(true, "Sqlite.interrupt() should not throw on a writable connection");
   await c.close();
   Assert.throws(
     () => c.interrupt(),
