@@ -1,13 +1,11 @@
 /* eslint max-len: ["error", 80] */
 
 const { AddonTestUtils } = ChromeUtils.import(
-  "resource://testing-common/AddonTestUtils.jsm",
-  {}
+  "resource://testing-common/AddonTestUtils.jsm"
 );
 
 const { ExtensionPermissions } = ChromeUtils.import(
-  "resource://gre/modules/ExtensionPermissions.jsm",
-  {}
+  "resource://gre/modules/ExtensionPermissions.jsm"
 );
 
 const SUPPORT_URL = Services.urlFormatter.formatURL(
@@ -164,6 +162,16 @@ add_task(async function enableHtmlViews() {
       },
       type: "extension",
     },
+    {
+      id: "sitepermission@mochi.test",
+      name: "Test site permission add-on",
+      creator: { name: "you got it" },
+      description: "permission description",
+      fullDescription: "detailed description",
+      siteOrigin: "http://mochi.test",
+      sitePermissions: ["midi"],
+      type: "sitepermission",
+    },
     {
       id: "theme1@mochi.test",
       name: "Test theme",
@@ -455,6 +463,7 @@ add_task(async function testDetailOperations() {
 add_task(async function testFullDetails() {
   Services.telemetry.clearEvents();
   let id = "addon1@mochi.test";
+  let headingId = "addon1_mochi_test-heading";
   let win = await loadInitialView("extension");
   let doc = win.document;
 
@@ -475,6 +484,15 @@ add_task(async function testFullDetails() {
   card = getAddonCard(win, id);
   ok(card.hasAttribute("expanded"), "The detail card is expanded");
 
+  let cardHeading = card.querySelector("h1");
+  is(cardHeading.textContent, "Test add-on 1", "Card heading is set");
+  is(cardHeading.id, headingId, "Heading has correct id");
+  is(
+    card.querySelector(".card").getAttribute("aria-labelledby"),
+    headingId,
+    "Card is labelled by the heading"
+  );
+
   // Make sure the preview is hidden.
   preview = card.querySelector(".card-heading-image");
   ok(preview, "There is a preview");
@@ -492,6 +510,15 @@ add_task(async function testFullDetails() {
     "The full description replaces newlines with <br>"
   );
 
+  let sitepermissionsRow = details.querySelector(
+    ".addon-detail-sitepermissions"
+  );
+  is(
+    sitepermissionsRow.hidden,
+    true,
+    "AddonSitePermissionsList should be hidden for this addon type"
+  );
+
   let contrib = details.querySelector(".addon-detail-contribute");
   ok(contrib, "The contribution section is visible");
 
@@ -718,7 +745,7 @@ add_task(async function testDefaultTheme() {
   // Version.
   let version = rows.shift();
   checkLabel(version, "version");
-  is(version.lastChild.textContent, "1.2", "It's always version 1.2");
+  is(version.lastChild.textContent, "1.3", "It's always version 1.3");
 
   // Last updated.
   let lastUpdated = rows.shift();
@@ -782,6 +809,66 @@ add_task(async function testStaticTheme() {
   await closeView(win);
 });
 
+add_task(async function testSitePermission() {
+  let win = await loadInitialView("sitepermission");
+  let doc = win.document;
+
+  // The list card.
+  let card = getAddonCard(win, "sitepermission@mochi.test");
+  ok(!card.hasAttribute("expanded"), "The list card is not expanded");
+
+  // Load the detail view.
+  let loaded = waitForViewLoad(win);
+  card.querySelector('[action="expand"]').click();
+  await loaded;
+
+  card = getAddonCard(win, "sitepermission@mochi.test");
+
+  // Check all the deck buttons are hidden.
+  assertDeckHeadingHidden(card.details.tabGroup);
+
+  let rows = getDetailRows(card);
+  is(rows.length, 4, "There are 4 rows");
+
+  let sitepermissionsRow = card.querySelector(".addon-detail-sitepermissions");
+  is(
+    sitepermissionsRow.hidden,
+    false,
+    "AddonSitePermissionsList should be visible for this addon type"
+  );
+
+  // Automatic updates.
+  let row = rows.shift();
+  checkLabel(row, "updates");
+
+  // Private browsing.
+  let private = rows.shift();
+  checkLabel(private, "private-browsing");
+
+  let help = rows.shift();
+  ok(help.classList.contains("addon-detail-help-row"), "There's a help row");
+  ok(!row.hidden, "The help row is shown");
+  is(
+    doc.l10n.getAttributes(help).id,
+    "addon-detail-private-browsing-help",
+    "The help row is for private browsing"
+  );
+
+  // Author.
+  let author = rows.shift();
+  checkLabel(author, "author");
+
+  is(rows.length, 0, "There was only 1 row");
+
+  let permissions = Array.from(
+    card.querySelectorAll(".addon-permissions-list .permission-info")
+  );
+  is(permissions.length, 1, "a permission is listed");
+  is(permissions[0].textContent, "Access MIDI devices", "got midi permission");
+
+  await closeView(win);
+});
+
 add_task(async function testPrivateBrowsingExtension() {
   Services.telemetry.clearEvents();
   let id = "pb@mochi.test";
