@@ -17,20 +17,28 @@ module.exports = {
     chromeOuterWindowID: false,
     content: false,
     docShell: false,
-    privateNoteIntentionalCrash: false,
     processMessageManager: false,
     removeMessageListener: false,
     removeWeakMessageListener: false,
     sendAsyncMessage: false,
     sendSyncMessage: false,
-    sendRpcMessage: false,
     tabEventTarget: false,
+    RPMGetAppBuildID: false,
+    RPMGetInnerMostURI: false,
+    RPMGetIntPref: false,
+    RPMGetStringPref: false,
     RPMGetBoolPref: false,
     RPMSetBoolPref: false,
     RPMGetFormatURLPref: false,
     RPMIsWindowPrivate: false,
     RPMSendAsyncMessage: false,
+    RPMSendQuery: false,
     RPMAddMessageListener: false,
+    RPMRecordTelemetryEvent: false,
+    RPMAddToHistogram: false,
     RPMRemoveMessageListener: false,
+    RPMGetHttpResponseHeader: false,
+    RPMTryPingSecureWWWLink: false,
+    RPMOpenSecureWWWLink: false,
   },
 };
