@@ -6,65 +6,67 @@
  */
 "use strict";
 
-const espree = require("espree");
+const parser = require("@babel/eslint-parser");
+const { analyze } = require("eslint-scope");
+const { KEYS: defaultVisitorKeys } = require("eslint-visitor-keys");
 const estraverse = require("estraverse");
 const path = require("path");
 const fs = require("fs");
-const ini = require("ini-parser");
+const ini = require("multi-ini");
 const recommendedConfig = require("./configs/recommended");
 
-var gModules = null;
 var gRootDir = null;
 var directoryManifests = new Map();
 
 const callExpressionDefinitions = [
-  /^loader\.lazyGetter\(this, "(\w+)"/,
-  /^loader\.lazyImporter\(this, "(\w+)"/,
-  /^loader\.lazyServiceGetter\(this, "(\w+)"/,
-  /^loader\.lazyRequireGetter\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineLazyGetter\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineLazyModuleGetter\(this, "(\w+)"/,
-  /^ChromeUtils\.defineModuleGetter\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineLazyPreferenceGetter\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineLazyProxy\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineLazyScriptGetter\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineLazyServiceGetter\(this, "(\w+)"/,
-  /^XPCOMUtils\.defineConstant\(this, "(\w+)"/,
-  /^DevToolsUtils\.defineLazyModuleGetter\(this, "(\w+)"/,
-  /^DevToolsUtils\.defineLazyGetter\(this, "(\w+)"/,
-  /^Object\.defineProperty\(this, "(\w+)"/,
-  /^Reflect\.defineProperty\(this, "(\w+)"/,
+  /^loader\.lazyGetter\((?:globalThis|this), "(\w+)"/,
+  /^loader\.lazyImporter\((?:globalThis|this), "(\w+)"/,
+  /^loader\.lazyServiceGetter\((?:globalThis|this), "(\w+)"/,
+  /^loader\.lazyRequireGetter\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineLazyGetter\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineLazyModuleGetter\((?:globalThis|this), "(\w+)"/,
+  /^ChromeUtils\.defineModuleGetter\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineLazyPreferenceGetter\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineLazyProxy\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineLazyScriptGetter\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineLazyServiceGetter\((?:globalThis|this), "(\w+)"/,
+  /^XPCOMUtils\.defineConstant\((?:globalThis|this), "(\w+)"/,
+  /^DevToolsUtils\.defineLazyModuleGetter\((?:globalThis|this), "(\w+)"/,
+  /^DevToolsUtils\.defineLazyGetter\((?:globalThis|this), "(\w+)"/,
+  /^Object\.defineProperty\((?:globalThis|this), "(\w+)"/,
+  /^Reflect\.defineProperty\((?:globalThis|this), "(\w+)"/,
   /^this\.__defineGetter__\("(\w+)"/,
 ];
 
 const callExpressionMultiDefinitions = [
   "XPCOMUtils.defineLazyGlobalGetters(this,",
+  "XPCOMUtils.defineLazyGlobalGetters(globalThis,",
   "XPCOMUtils.defineLazyModuleGetters(this,",
+  "XPCOMUtils.defineLazyModuleGetters(globalThis,",
   "XPCOMUtils.defineLazyServiceGetters(this,",
-];
-
-const imports = [
-  /^(?:Cu|Components\.utils|ChromeUtils)\.import\(".*\/((.*?)\.jsm?)", this\)/,
+  "XPCOMUtils.defineLazyServiceGetters(globalThis,",
+  "loader.lazyRequireGetter(this,",
+  "loader.lazyRequireGetter(globalThis,",
 ];
 
 const workerImportFilenameMatch = /(.*\/)*((.*?)\.jsm?)/;
 
 module.exports = {
-  get modulesGlobalData() {
-    if (!gModules) {
-      if (this.isMozillaCentralBased()) {
-        gModules = require(path.join(this.rootDir, "tools", "lint", "eslint", "modules.json"));
-      } else {
-        gModules = require("./modules.json");
-      }
+  get iniParser() {
+    if (!this._iniParser) {
+      this._iniParser = new ini.Parser();
     }
+    return this._iniParser;
+  },
 
-    return gModules;
+  get servicesData() {
+    return require("./services.json");
   },
 
   /**
    * Gets the abstract syntax tree (AST) of the JavaScript source code contained
-   * in sourceText.
+   * in sourceText. This matches the results for an eslint parser, see
+   * https://eslint.org/docs/developer-guide/working-with-custom-parsers.
    *
    * @param  {String} sourceText
    *         Text containing valid JavaScript.
@@ -73,14 +75,28 @@ module.exports = {
    *         the configuration from getPermissiveConfig().
    *
    * @return {Object}
-   *         The resulting AST.
+   *         Returns an object containing `ast`, `scopeManager` and
+   *         `visitorKeys`
    */
-  getAST(sourceText, astOptions = {}) {
+  parseCode(sourceText, astOptions = {}) {
     // Use a permissive config file to allow parsing of anything that Espree
     // can parse.
-    let config = {...this.getPermissiveConfig(), ...astOptions};
+    let config = { ...this.getPermissiveConfig(), ...astOptions };
 
-    return espree.parse(sourceText, config);
+    let parseResult =
+      "parseForESLint" in parser
+        ? parser.parseForESLint(sourceText, config)
+        : { ast: parser.parse(sourceText, config) };
+
+    let visitorKeys = parseResult.visitorKeys || defaultVisitorKeys;
+    visitorKeys.ExperimentalRestProperty = visitorKeys.RestElement;
+    visitorKeys.ExperimentalSpreadProperty = visitorKeys.SpreadElement;
+
+    return {
+      ast: parseResult.ast,
+      scopeManager: parseResult.scopeManager || analyze(parseResult.ast),
+      visitorKeys,
+    };
   },
 
   /**
@@ -97,10 +113,15 @@ module.exports = {
       case "MemberExpression":
         if (node.computed) {
           let filename = context && context.getFilename();
-          throw new Error(`getASTSource unsupported computed MemberExpression in ${filename}`);
+          throw new Error(
+            `getASTSource unsupported computed MemberExpression in ${filename}`
+          );
         }
-        return this.getASTSource(node.object) + "." +
-          this.getASTSource(node.property);
+        return (
+          this.getASTSource(node.object) +
+          "." +
+          this.getASTSource(node.property)
+        );
       case "ThisExpression":
         return "this";
       case "Identifier":
@@ -121,10 +142,19 @@ module.exports = {
       case "ArrowFunctionExpression":
         return "() => {}";
       case "AssignmentExpression":
-        return this.getASTSource(node.left) + " = " +
-          this.getASTSource(node.right);
+        return (
+          this.getASTSource(node.left) + " = " + this.getASTSource(node.right)
+        );
       case "BinaryExpression":
-        return this.getASTSource(node.left) + " " + node.operator + " " + this.getASTSource(node.right);
+        return (
+          this.getASTSource(node.left) +
+          " " +
+          node.operator +
+          " " +
+          this.getASTSource(node.right)
+        );
+      case "UnaryExpression":
+        return node.operator + " " + this.getASTSource(node.argument);
       default:
         throw new Error("getASTSource unsupported node type: " + node.type);
     }
@@ -137,11 +167,13 @@ module.exports = {
    *
    * @param  {Object} ast
    *         The AST to walk.
+   * @param  {Array} visitorKeys
+   *         The visitor keys to use for the AST.
    * @param  {Function} listener
    *         A callback function to call for the nodes. Passed three arguments,
    *         event type, node and an array of parent nodes for the current node.
    */
-  walkAST(ast, listener) {
+  walkAST(ast, visitorKeys, listener) {
     let parents = [];
 
     estraverse.traverse(ast, {
@@ -157,6 +189,8 @@ module.exports = {
         }
         parents.pop();
       },
+
+      keys: visitorKeys,
     });
     if (parents.length) {
       throw new Error("Entered more nodes than left.");
@@ -182,15 +216,15 @@ module.exports = {
   convertWorkerExpressionToGlobals(node, isGlobal, dirname) {
     var getGlobalsForFile = require("./globals").getGlobalsForFile;
 
-    let globalModules = this.modulesGlobalData;
-
     let results = [];
     let expr = node.expression;
 
-    if (node.expression.type === "CallExpression" &&
-        expr.callee &&
-        expr.callee.type === "Identifier" &&
-        expr.callee.name === "importScripts") {
+    if (
+      node.expression.type === "CallExpression" &&
+      expr.callee &&
+      expr.callee.type === "Identifier" &&
+      expr.callee.name === "importScripts"
+    ) {
       for (var arg of expr.arguments) {
         var match = arg.value && arg.value.match(workerImportFilenameMatch);
         if (match) {
@@ -200,13 +234,9 @@ module.exports = {
               let additionalGlobals = getGlobalsForFile(filePath);
               results = results.concat(additionalGlobals);
             }
-          } else if (match[2] in globalModules) {
-            results = results.concat(globalModules[match[2]].map(name => {
-              return { name, writable: true };
-            }));
-          } else {
-            results.push({ name: match[3], writable: true, explicit: true });
           }
+          // Import with relative/absolute path should explicitly use
+          // `import-globals-from` comment.
         }
       }
     }
@@ -231,12 +261,14 @@ module.exports = {
    *                     If the global is writeable or not.
    */
   convertThisAssignmentExpressionToGlobals(node, isGlobal) {
-    if (isGlobal &&
-        node.expression.left &&
-        node.expression.left.object &&
-        node.expression.left.object.type === "ThisExpression" &&
-        node.expression.left.property &&
-        node.expression.left.property.type === "Identifier") {
+    if (
+      isGlobal &&
+      node.expression.left &&
+      node.expression.left.object &&
+      node.expression.left.object.type === "ThisExpression" &&
+      node.expression.left.property &&
+      node.expression.left.property.type === "Identifier"
+    ) {
       return [{ name: node.expression.left.property.name, writable: true }];
     }
     return [];
@@ -244,7 +276,7 @@ module.exports = {
 
   /**
    * Attempts to convert an CallExpressions that look like module imports
-   * into global variable definitions, using modules.json data if appropriate.
+   * into global variable definitions.
    *
    * @param  {Object} node
    *         The AST node to convert.
@@ -260,14 +292,16 @@ module.exports = {
    */
   convertCallExpressionToGlobals(node, isGlobal) {
     let express = node.expression;
-    if (express.type === "CallExpression" &&
-        express.callee.type === "MemberExpression" &&
-        express.callee.object &&
-        express.callee.object.type === "Identifier" &&
-        express.arguments.length === 1 &&
-        express.arguments[0].type === "ArrayExpression" &&
-        express.callee.property.type === "Identifier" &&
-        express.callee.property.name === "importGlobalProperties") {
+    if (
+      express.type === "CallExpression" &&
+      express.callee.type === "MemberExpression" &&
+      express.callee.object &&
+      express.callee.object.type === "Identifier" &&
+      express.arguments.length === 1 &&
+      express.arguments[0].type === "ArrayExpression" &&
+      express.callee.property.type === "Identifier" &&
+      express.callee.property.name === "importGlobalProperties"
+    ) {
       return express.arguments[0].elements.map(literal => {
         return {
           explicit: true,
@@ -284,31 +318,6 @@ module.exports = {
       return [];
     }
 
-    for (let reg of imports) {
-      let match = source.match(reg);
-      if (match) {
-        // The two argument form is only acceptable in the global scope
-        if (node.expression.arguments.length > 1 && !isGlobal) {
-          return [];
-        }
-
-        let globalModules = this.modulesGlobalData;
-
-        if (match[1] in globalModules) {
-          // XXX We mark as explicit when there is only one exported symbol from
-          // the module. For now this avoids no-unused-vars complaining in the
-          // cases where we import everything from a module but only use one
-          // of them.
-          let explicit = globalModules[match[1]].length == 1;
-          return globalModules[match[1]].map(name => ({
-            name, writable: true, explicit,
-          }));
-        }
-
-        return [{ name: match[2], writable: true, explicit: true }];
-      }
-    }
-
     // The definition matches below must be in the global scope for us to define
     // a global, so bail out early if we're not a global.
     if (!isGlobal) {
@@ -322,27 +331,43 @@ module.exports = {
       }
     }
 
-    if (callExpressionMultiDefinitions.some(expr => source.startsWith(expr)) &&
-        node.expression.arguments[1]) {
+    if (
+      callExpressionMultiDefinitions.some(expr => source.startsWith(expr)) &&
+      node.expression.arguments[1]
+    ) {
       let arg = node.expression.arguments[1];
       if (arg.type === "ObjectExpression") {
         return arg.properties
-                  .map(p => ({ name: p.type === "Property" && p.key.name, writable: true, explicit: true }))
-                  .filter(g => g.name);
+          .map(p => ({
+            name: p.type === "Property" && p.key.name,
+            writable: true,
+            explicit: true,
+          }))
+          .filter(g => g.name);
       }
       if (arg.type === "ArrayExpression") {
         return arg.elements
-                  .map(p => ({ name: p.type === "Literal" && p.value, writable: true, explicit: true }))
-                  .filter(g => typeof g.name == "string");
+          .map(p => ({
+            name: p.type === "Literal" && p.value,
+            writable: true,
+            explicit: true,
+          }))
+          .filter(g => typeof g.name == "string");
       }
     }
 
-    if (node.expression.callee.type == "MemberExpression" &&
-        node.expression.callee.property.type == "Identifier" &&
-        node.expression.callee.property.name == "defineLazyScriptGetter") {
+    if (
+      node.expression.callee.type == "MemberExpression" &&
+      node.expression.callee.property.type == "Identifier" &&
+      node.expression.callee.property.name == "defineLazyScriptGetter"
+    ) {
       // The case where we have a single symbol as a string has already been
       // handled by the regexp, so we have an array of symbols here.
-      return node.expression.arguments[1].elements.map(n => ({ name: n.value, writable: true, explicit: true }));
+      return node.expression.arguments[1].elements.map(n => ({
+        name: n.value,
+        writable: true,
+        explicit: true,
+      }));
     }
 
     return [];
@@ -368,7 +393,11 @@ module.exports = {
     variable.eslintExplicitGlobal = false;
     variable.writeable = writable;
     if (node) {
-      variable.defs.push({node, name: {name}});
+      variable.defs.push({
+        type: "Variable",
+        node,
+        name: { name, parent: node.parent },
+      });
       variable.identifiers.push(node);
     }
 
@@ -402,7 +431,9 @@ module.exports = {
    *        The AST node that defined the globals.
    */
   addGlobals(globalVars, scope, node) {
-    globalVars.forEach(v => this.addVarToScope(v.name, scope, v.writable, v.explicit && node));
+    globalVars.forEach(v =>
+      this.addVarToScope(v.name, scope, v.writable, v.explicit && node)
+    );
   },
 
   /**
@@ -413,14 +444,33 @@ module.exports = {
    *         Espree compatible permissive config.
    */
   getPermissiveConfig() {
-    return {
+    const config = {
       range: true,
+      requireConfigFile: false,
+      babelOptions: {
+        // configFile: path.join(gRootDir, ".babel-eslint.rc.js"),
+        // parserOpts: {
+        //   plugins: [
+        //     "@babel/plugin-proposal-class-static-block",
+        //     "@babel/plugin-syntax-class-properties",
+        //     "@babel/plugin-syntax-jsx",
+        //   ],
+        // },
+      },
       loc: true,
       comment: true,
       attachComment: true,
       ecmaVersion: this.getECMAVersion(),
       sourceType: "script",
     };
+
+    if (this.isMozillaCentralBased()) {
+      config.babelOptions.configFile = path.join(
+        gRootDir,
+        ".babel-eslint.rc.js"
+      );
+    }
+    return config;
   },
 
   /**
@@ -503,11 +553,14 @@ module.exports = {
     let filepath = this.cleanUpPath(scope.getFilename());
     let dir = path.dirname(filepath);
 
-    let names =
-      fs.readdirSync(dir)
-        .filter(name => (name.startsWith("head") ||
-                         name.startsWith("xpcshell-head")) && name.endsWith(".js"))
-        .map(name => path.join(dir, name));
+    let names = fs
+      .readdirSync(dir)
+      .filter(
+        name =>
+          (name.startsWith("head") || name.startsWith("xpcshell-head")) &&
+          name.endsWith(".js")
+      )
+      .map(name => path.join(dir, name));
     return names;
   },
 
@@ -542,14 +595,14 @@ module.exports = {
       }
 
       try {
-        let manifest = ini.parse(fs.readFileSync(path.join(dir, name), "utf8"));
-
+        let manifest = this.iniParser.parse(
+          fs.readFileSync(path.join(dir, name), "utf8").split("\n")
+        );
         manifests.push({
           file: path.join(dir, name),
           manifest,
         });
-      } catch (e) {
-      }
+      } catch (e) {}
     }
 
     directoryManifests.set(dir, manifests);
@@ -601,6 +654,15 @@ module.exports = {
     return !!this.getTestType(scope);
   },
 
+  /*
+   * Check if this is an .sjs file.
+   */
+  getIsSjs(scope) {
+    let filepath = this.cleanUpPath(scope.getFilename());
+
+    return path.extname(filepath) == ".sjs";
+  },
+
   /**
    * Gets the type of test or null if this isn't a test.
    *
@@ -673,7 +735,10 @@ module.exports = {
         return null;
       }
 
-      let possibleRoot = searchUpForIgnore(path.dirname(module.filename), ".eslintignore");
+      let possibleRoot = searchUpForIgnore(
+        path.dirname(module.filename),
+        ".eslintignore"
+      );
       if (!possibleRoot) {
         possibleRoot = searchUpForIgnore(path.resolve(), ".eslintignore");
       }
@@ -717,9 +782,9 @@ module.exports = {
       // without any path info (happens in Atom with linter-eslint)
       return path.join(cwd, fileName);
     }
-      // Case 1: executed form in a nested directory, e.g. from a text editor:
-      //   fileName: a/b/c/d.js
-      //   cwd: /path/to/mozilla/repo/a/b/c
+    // Case 1: executed form in a nested directory, e.g. from a text editor:
+    //   fileName: a/b/c/d.js
+    //   cwd: /path/to/mozilla/repo/a/b/c
     var dirName = path.dirname(fileName);
     return cwd.slice(0, cwd.length - dirName.length) + fileName;
   },
@@ -735,8 +800,14 @@ module.exports = {
 
   get globalScriptPaths() {
     return [
-      path.join(this.rootDir, "browser", "base", "content", "browser.xul"),
-      path.join(this.rootDir, "browser", "base", "content", "global-scripts.inc"),
+      path.join(this.rootDir, "browser", "base", "content", "browser.xhtml"),
+      path.join(
+        this.rootDir,
+        "browser",
+        "base",
+        "content",
+        "global-scripts.inc"
+      ),
     ];
   },
 
@@ -745,10 +816,46 @@ module.exports = {
   },
 
   getSavedEnvironmentItems(environment) {
-    return require("./environments/saved-globals.json").environments[environment];
+    return require("./environments/saved-globals.json").environments[
+      environment
+    ];
   },
 
   getSavedRuleData(rule) {
     return require("./rules/saved-rules-data.json").rulesData[rule];
   },
+
+  getBuildEnvironment() {
+    var { execFileSync } = require("child_process");
+    var output = execFileSync(
+      path.join(this.rootDir, "mach"),
+      ["environment", "--format=json"],
+      { silent: true }
+    );
+    return JSON.parse(output);
+  },
+
+  /**
+   * Extract the path of require (and require-like) helpers used in DevTools.
+   */
+  getDevToolsRequirePath(node) {
+    if (
+      node.callee.type == "Identifier" &&
+      node.callee.name == "require" &&
+      node.arguments.length == 1 &&
+      node.arguments[0].type == "Literal"
+    ) {
+      return node.arguments[0].value;
+    } else if (
+      node.callee.type == "MemberExpression" &&
+      node.callee.property.type == "Identifier" &&
+      (node.callee.property.name == "lazyRequireGetter" ||
+        node.callee.property.name == "lazyImporter") &&
+      node.arguments.length >= 3 &&
+      node.arguments[2].type == "Literal"
+    ) {
+      return node.arguments[2].value;
+    }
+    return null;
+  },
 };
