@@ -13,6 +13,7 @@
 #include "nsIObserverService.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Unused.h"
+#include "SpecialSystemDirectory.h"
 
 #ifdef XP_UNIX  // {
 #  include "mozilla/Preferences.h"
@@ -269,7 +270,7 @@ void FifoWatcher::RegisterCallback(const nsCString& aCommand,
   mFifoInfo.AppendElement(aFifoInfo);
 }
 
-FifoWatcher::~FifoWatcher() {}
+FifoWatcher::~FifoWatcher() = default;
 
 int FifoWatcher::OpenFd() {
   // If the memory_info_dumper.directory pref is specified, put the fifo
@@ -291,7 +292,7 @@ int FifoWatcher::OpenFd() {
     }
   }
 
-  rv = file->AppendNative(NS_LITERAL_CSTRING("debug_info_trigger"));
+  rv = file->AppendNative("debug_info_trigger"_ns);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return -1;
   }
@@ -417,7 +418,12 @@ nsresult nsDumpUtils::OpenTempFile(const nsACString& aFilename, nsIFile** aFile,
 #endif
   nsresult rv;
   if (!*aFile) {
-    rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, aFile);
+    if (NS_IsMainThread()) {
+      // This allows tests to override, but isn't safe off-mainthread.
+      rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, aFile);
+    } else {
+      rv = GetSpecialSystemDirectory(OS_TemporaryDirectory, aFile);
+    }
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -428,7 +434,7 @@ nsresult nsDumpUtils::OpenTempFile(const nsACString& aFilename, nsIFile** aFile,
   // but only the user which created the file can remove it.  We want non-root
   // users to be able to remove these files, so we write them into a
   // subdirectory of the temp directory and chmod 777 that directory.
-  if (aFoldername != EmptyCString()) {
+  if (!aFoldername.IsEmpty()) {
     rv = (*aFile)->AppendNative(aFoldername);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
