@@ -9,7 +9,6 @@
 
 #include "nsXPCOM.h"
 
-#include "xpcom-private.h"
 #include "nsIComponentManager.h"
 #include "nsIComponentRegistrar.h"
 #include "nsIMemoryReporter.h"
@@ -19,7 +18,8 @@
 #include "mozilla/Atomics.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/Module.h"
-#include "mozilla/Mutex.h"
+#include "mozilla/Monitor.h"
+#include "mozilla/UniquePtr.h"
 #include "nsXULAppAPI.h"
 #include "nsIFactory.h"
 #include "nsIInterfaceRequestor.h"
@@ -27,10 +27,9 @@
 #include "PLDHashTable.h"
 #include "prtime.h"
 #include "nsCOMPtr.h"
-#include "nsAutoPtr.h"
 #include "nsWeakReference.h"
 #include "nsCOMArray.h"
-#include "nsDataHashtable.h"
+#include "nsTHashMap.h"
 #include "nsInterfaceHashtable.h"
 #include "nsClassHashtable.h"
 #include "nsTArray.h"
@@ -56,7 +55,6 @@ extern const mozilla::Module kXPCOMModule;
 
 namespace {
 class EntryWrapper;
-class MutexLock;
 }  // namespace
 
 namespace mozilla {
@@ -68,52 +66,6 @@ bool FastProcessSelectorMatches(Module::ProcessSelector aSelector);
 }  // namespace xpcom
 }  // namespace mozilla
 
-/**
- * This is a wrapper around mozilla::Mutex which provides runtime
- * checking for a deadlock where the same thread tries to lock a mutex while
- * it is already locked. This checking is present in both debug and release
- * builds.
- */
-class SafeMutex {
- public:
-  explicit SafeMutex(const char* aName)
-      : mMutex(aName), mOwnerThread(nullptr) {}
-
-  ~SafeMutex() {}
-
-  void Lock() {
-    AssertNotCurrentThreadOwns();
-    mMutex.Lock();
-    MOZ_ASSERT(mOwnerThread == nullptr);
-    mOwnerThread = PR_GetCurrentThread();
-  }
-
-  void Unlock() {
-    MOZ_ASSERT(mOwnerThread == PR_GetCurrentThread());
-    mOwnerThread = nullptr;
-    mMutex.Unlock();
-  }
-
-  void AssertCurrentThreadOwns() const {
-    // This method is a debug-only check
-    MOZ_ASSERT(mOwnerThread == PR_GetCurrentThread());
-  }
-
-  MOZ_NEVER_INLINE void AssertNotCurrentThreadOwns() const {
-    // This method is a release-mode check
-    if (PR_GetCurrentThread() == mOwnerThread) {
-      MOZ_CRASH();
-    }
-  }
-
- private:
-  mozilla::Mutex mMutex;
-  mozilla::Atomic<PRThread*, mozilla::Relaxed> mOwnerThread;
-};
-
-typedef mozilla::BaseAutoLock<SafeMutex&> SafeMutexAutoLock;
-typedef mozilla::BaseAutoUnlock<SafeMutex&> SafeMutexAutoUnlock;
-
 class nsComponentManagerImpl final : public nsIComponentManager,
                                      public nsIServiceManager,
                                      public nsSupportsWeakReference,
@@ -127,7 +79,7 @@ class nsComponentManagerImpl final : public nsIComponentManager,
   NS_DECL_NSICOMPONENTREGISTRAR
   NS_DECL_NSIMEMORYREPORTER
 
-  static nsresult Create(nsISupports* aOuter, REFNSIID aIID, void** aResult);
+  static nsresult Create(REFNSIID aIID, void** aResult);
 
   nsresult RegistryLocationForFile(nsIFile* aFile, nsCString& aResult);
   nsresult FileForRegistryLocation(const nsCString& aLocation, nsIFile** aSpec);
@@ -150,18 +102,19 @@ class nsComponentManagerImpl final : public nsIComponentManager,
 
   already_AddRefed<nsIFactory> LoadFactory(nsFactoryEntry* aEntry);
 
-  nsDataHashtable<nsIDPointerHashKey, nsFactoryEntry*> mFactories;
-  nsDataHashtable<nsCStringHashKey, nsFactoryEntry*> mContractIDs;
+  nsTHashMap<nsIDPointerHashKey, nsFactoryEntry*> mFactories;
+  nsTHashMap<nsCStringHashKey, nsFactoryEntry*> mContractIDs;
 
-  SafeMutex mLock;
+  mozilla::Monitor mLock MOZ_UNANNOTATED;
 
   mozilla::Maybe<EntryWrapper> LookupByCID(const nsID& aCID);
-  mozilla::Maybe<EntryWrapper> LookupByCID(const MutexLock&, const nsID& aCID);
+  mozilla::Maybe<EntryWrapper> LookupByCID(const mozilla::MonitorAutoLock&,
+                                           const nsID& aCID);
 
   mozilla::Maybe<EntryWrapper> LookupByContractID(
       const nsACString& aContractID);
   mozilla::Maybe<EntryWrapper> LookupByContractID(
-      const MutexLock&, const nsACString& aContractID);
+      const mozilla::MonitorAutoLock&, const nsACString& aContractID);
 
   nsresult GetService(mozilla::xpcom::ModuleID, const nsIID& aIID,
                       void** aResult);
@@ -222,7 +175,7 @@ class nsComponentManagerImpl final : public nsIComponentManager,
 
   // The KnownModule is kept alive by these members, it is
   // referenced by pointer from the factory entries.
-  nsTArray<nsAutoPtr<KnownModule>> mKnownStaticModules;
+  nsTArray<mozilla::UniquePtr<KnownModule>> mKnownStaticModules;
   // The key is the URI string of the module
   nsClassHashtable<nsCStringHashKey, KnownModule> mKnownModules;
 
@@ -243,7 +196,7 @@ class nsComponentManagerImpl final : public nsIComponentManager,
                               mozilla::FileLocation& aFile, bool aChromeOnly)
         : mType(aType), mFile(aFile), mChromeOnly(aChromeOnly) {}
 
-    ~ManifestProcessingContext() {}
+    ~ManifestProcessingContext() = default;
 
     NSLocationType mType;
     mozilla::FileLocation mFile;
@@ -278,7 +231,8 @@ class nsComponentManagerImpl final : public nsIComponentManager,
 
   inline PendingServiceInfo* AddPendingService(const nsCID& aServiceCID,
                                                PRThread* aThread);
-  inline void RemovePendingService(const nsCID& aServiceCID);
+  inline void RemovePendingService(mozilla::MonitorAutoLock& aLock,
+                                   const nsCID& aServiceCID);
   inline PRThread* GetPendingServiceThread(const nsCID& aServiceCID) const;
 
   nsTArray<PendingServiceInfo> mPendingServices;
@@ -290,8 +244,9 @@ class nsComponentManagerImpl final : public nsIComponentManager,
  private:
   ~nsComponentManagerImpl();
 
-  nsresult GetServiceLocked(MutexLock& aLock, EntryWrapper& aEntry,
-                            const nsIID& aIID, void** aResult);
+  nsresult GetServiceLocked(mozilla::Maybe<mozilla::MonitorAutoLock>& aLock,
+                            EntryWrapper& aEntry, const nsIID& aIID,
+                            void** aResult);
 };
 
 #define NS_MAX_FILENAME_LEN 1024
@@ -309,8 +264,7 @@ struct nsFactoryEntry {
 
   already_AddRefed<nsIFactory> GetFactory();
 
-  nsresult CreateInstance(nsISupports* aOuter, const nsIID& aIID,
-                          void** aResult);
+  nsresult CreateInstance(const nsIID& aIID, void** aResult);
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf);
 
