@@ -124,7 +124,7 @@
 //!
 //! ```ignore
 //! // nsISupports foo(in long long bar, in AString baz);
-//! unsafe fn Foo(&self, bar: libc::int64_t, baz: *const nsAString,
+//! unsafe fn Foo(&self, bar: i64, baz: *const nsAString,
 //!               _retval: *mut *const nsISupports) -> nsresult;
 //!
 //! // AString qux(in nsISupports ham);
@@ -140,30 +140,30 @@
 //! [`RefCounted`]: ../xpcom/struct.RefCounted.html
 //! [`RefPtr`]: ../xpcom/struct.RefPtr.html
 
-// NOTE: We use some really big quote! invocations, so we need a high recursion
-// limit.
-#![recursion_limit="256"]
-
-#[macro_use]
-extern crate quote;
-
-extern crate syn;
-
-extern crate proc_macro;
-
-#[macro_use]
-extern crate lazy_static;
-
-use proc_macro::TokenStream;
-
-use quote::{ToTokens, Tokens};
-
-use syn::*;
-
-use syn::punctuated::Punctuated;
-
+use lazy_static::lazy_static;
+use proc_macro2::{Span, TokenStream};
+use quote::{format_ident, quote, ToTokens};
 use std::collections::{HashMap, HashSet};
-use std::error::Error;
+use syn::punctuated::Punctuated;
+use syn::{
+    parse_macro_input, parse_quote, Attribute, Data, DataStruct, DeriveInput, Field, Fields, Ident,
+    Lit, Meta, NestedMeta, Token, Type,
+};
+
+macro_rules! bail {
+    (@($t:expr), $s:expr) => {
+        return Err(syn::Error::new_spanned(&$t, &$s[..]))
+    };
+    (@($t:expr), $f:expr, $($e:expr),*) => {
+        return Err(syn::Error::new_spanned(&$t, &format!($f, $($e),*)[..]))
+    };
+    ($s:expr) => {
+        return Err(syn::Error::new(Span::call_site(), &$s[..]))
+    };
+    ($f:expr, $($e:expr),*) => {
+        return Err(syn::Error::new(Span::call_site(), &format!($f, $($e),*)[..]))
+    };
+}
 
 /* These are the structs generated by the rust_macros.py script */
 
@@ -192,14 +192,22 @@ struct Interface {
 }
 
 impl Interface {
-    fn base(&self) -> Result<Option<&'static Interface>, Box<Error>> {
-        Ok(if let Some(base) = self.base {
-            Some(*IFACES.get(base).ok_or_else(
-                || format!("Base interface {} does not exist", base)
-            )?)
-        } else {
-            None
-        })
+    fn base(&self) -> Option<&'static Interface> {
+        Some(IFACES[self.base?])
+    }
+
+    fn methods(&self) -> Result<&'static [Method], syn::Error> {
+        match self.methods {
+            Ok(methods) => Ok(methods),
+            Err(reason) => Err(syn::Error::new(
+                Span::call_site(),
+                &format!(
+                    "Interface {} cannot be implemented in rust \
+                     because {} is not supported yet",
+                    self.name, reason
+                ),
+            )),
+        }
     }
 }
 
@@ -208,7 +216,7 @@ lazy_static! {
     /// the form of a `HashMap` from interface names to their descriptions.
     static ref IFACES: HashMap<&'static str, &'static Interface> = {
         let lists: &[&[Interface]] =
-            include!(concat!(env!("MOZ_TOPOBJDIR"), "/dist/xpcrs/bt/all.rs"));
+            include!(mozbuild::objdir_path!("dist/xpcrs/bt/all.rs"));
 
         let mut hm = HashMap::new();
         for &list in lists {
@@ -229,7 +237,7 @@ enum RefcntKind {
 
 /// Produces the tokens for the type representation.
 impl ToTokens for RefcntKind {
-    fn to_tokens(&self, tokens: &mut Tokens) {
+    fn to_tokens(&self, tokens: &mut TokenStream) {
         match *self {
             RefcntKind::NonAtomic => quote!(xpcom::Refcnt).to_tokens(tokens),
             RefcntKind::Atomic => quote!(xpcom::AtomicRefcnt).to_tokens(tokens),
@@ -238,17 +246,20 @@ impl ToTokens for RefcntKind {
 }
 
 /// Scans through the attributes on a struct, and extracts the type of the refcount to use.
-fn get_refcnt_kind(attrs: &[Attribute]) -> Result<RefcntKind, Box<Error>> {
+fn get_refcnt_kind(attrs: &[Attribute]) -> Result<RefcntKind, syn::Error> {
     for attr in attrs {
-        if let Some(Meta::NameValue(ref attr)) = attr.interpret_meta() {
-            if attr.ident.as_ref() != "refcnt" {
+        if let Meta::NameValue(syn::MetaNameValue {
+            ref path, ref lit, ..
+        }) = attr.parse_meta()?
+        {
+            if !path.is_ident("refcnt") {
                 continue;
             }
 
-            let value = if let Lit::Str(ref s) = attr.lit {
+            let value = if let Lit::Str(ref s) = lit {
                 s.value()
             } else {
-                Err("Unexpected non-string value in #[refcnt]")?
+                bail!(@(attr), "Unexpected non-string value in #[refcnt]");
             };
 
             return if value == "nonatomic" {
@@ -256,36 +267,44 @@ fn get_refcnt_kind(attrs: &[Attribute]) -> Result<RefcntKind, Box<Error>> {
             } else if value == "atomic" {
                 Ok(RefcntKind::Atomic)
             } else {
-                Err("Unexpected value in #[refcnt]. \
-                     Expected `nonatomic`, or `atomic`")?
+                bail!(@(attr), "Unexpected value in #[refcnt]. \
+                                Expected `nonatomic`, or `atomic`");
             };
         }
     }
 
-    Err("Expected #[refcnt] attribute")?
+    bail!("Expected #[refcnt] attribute")
 }
 
 /// Scan the attributes looking for an #[xpimplements] attribute. The identifier
 /// arguments passed to this attribute are the interfaces which the type wants to
 /// directly implement.
-fn get_bases(attrs: &[Attribute]) -> Result<Vec<&'static Interface>, Box<Error>> {
+fn get_bases(attrs: &[Attribute]) -> Result<Vec<&'static Interface>, syn::Error> {
     let mut inherits = Vec::new();
     for attr in attrs {
-        if let Some(Meta::List(ref attr)) = attr.interpret_meta() {
-            if attr.ident.as_ref() != "xpimplements" {
+        if let Meta::List(syn::MetaList {
+            ref path,
+            ref nested,
+            ..
+        }) = attr.parse_meta()?
+        {
+            if !path.is_ident("xpimplements") {
                 continue;
             }
 
-            for item in &attr.nested {
-                if let NestedMeta::Meta(Meta::Word(ref iface)) = *item {
-                    if let Some(&iface) = IFACES.get(iface.as_ref()) {
-                        inherits.push(iface);
-                    } else {
-                        Err(format!("Unexpected invalid base interface `{}` in \
-                                     #[xpimplements(..)]", iface))?
-                    }
+            for item in nested.iter() {
+                let iface = match *item {
+                    NestedMeta::Meta(syn::Meta::Path(ref iface)) => iface,
+                    _ => bail!(@(attr), "Unexpected non-identifier in #[xpimplements(..)]"),
+                };
+                let ident = match iface.get_ident() {
+                    Some(ref iface) => iface.to_string(),
+                    _ => bail!(@(attr), "Too many components in xpimplements path"),
+                };
+                if let Some(&iface) = IFACES.get(ident.as_str()) {
+                    inherits.push(iface);
                 } else {
-                    Err("Unexpected non-identifier in #[xpimplements(..)]")?
+                    bail!(@(attr), "Unexpected invalid base interface `{}` in #[xpimplements(..)]", ident);
                 }
             }
         }
@@ -294,86 +313,98 @@ fn get_bases(attrs: &[Attribute]) -> Result<Vec<&'static Interface>, Box<Error>>
 }
 
 /// Extract the fields list from the input struct.
-fn get_fields(di: &DeriveInput) -> Result<&Punctuated<Field, Token![,]>, Box<Error>> {
+fn get_fields(di: &DeriveInput) -> Result<&Punctuated<Field, Token![,]>, syn::Error> {
     match di.data {
         Data::Struct(DataStruct {
-            fields: Fields::Named(ref named), ..
+            fields: Fields::Named(ref named),
+            ..
         }) => Ok(&named.named),
-        _ => Err("The initializer struct must be a standard \
-                  named value struct definition".into())
+        _ => bail!(@(di), "The initializer struct must be a standard named \
+                          value struct definition"),
     }
 }
 
 /// Takes the `Init*` struct in, and generates a `DeriveInput` for the "real" struct.
-fn gen_real_struct(init: &DeriveInput, bases: &[&Interface], refcnt_ty: RefcntKind) -> Result<DeriveInput, Box<Error>> {
+fn gen_real_struct(
+    init: &DeriveInput,
+    bases: &[&Interface],
+    refcnt_ty: RefcntKind,
+) -> Result<DeriveInput, syn::Error> {
     // Determine the name for the real struct based on the name of the
     // initializer struct's name.
-    if !init.ident.as_ref().starts_with("Init") {
-        Err("The target struct's name must begin with Init")?
+    if !init.ident.to_string().starts_with("Init") {
+        bail!(@(init.ident), "The target struct's name must begin with Init");
     }
-    let name: Ident = init.ident.as_ref()[4..].into();
+    let name = Ident::new(&init.ident.to_string()[4..], init.ident.span());
     let vis = &init.vis;
 
     let bases = bases.iter().map(|base| {
-        let ident = Ident::from(format!("__base_{}", base.name));
-        let vtable = Ident::from(format!("{}VTable", base.name));
+        let ident = format_ident!("__base_{}", base.name);
+        let vtable = format_ident!("{}VTable", base.name);
         quote!(#ident : *const xpcom::interfaces::#vtable)
-     });
+    });
 
     let fields = get_fields(init)?;
+    let (impl_generics, _, where_clause) = init.generics.split_for_impl();
     Ok(parse_quote! {
-        #[repr(C)]
-        #vis struct #name {
-            #(#bases,)*
-            __refcnt: #refcnt_ty,
-            #fields
-        }
-     })
+       #[repr(C)]
+       #vis struct #name #impl_generics #where_clause {
+           #(#bases,)*
+           __refcnt: #refcnt_ty,
+           #fields
+       }
+    })
 }
 
 /// Generates the `extern "system"` methods which are actually included in the
 /// VTable for the given interface.
 ///
-/// These methods attempt to invoke the `recover_self` method to translate from
-/// the passed-in raw pointer to the actual `&self` value, and it is expected to
-/// be in scope.
-fn gen_vtable_methods(iface: &Interface) -> Result<Tokens, Box<Error>> {
-    let base_ty = Ident::from(iface.name);
+/// `idx` must be the offset in pointers of the pointer to this vtable in the
+/// struct `real`. This is soundness-critical, as it will be used to offset
+/// pointers received from xpcom back to the concrete implementation.
+fn gen_vtable_methods(
+    real: &DeriveInput,
+    iface: &Interface,
+    vtable_index: usize,
+) -> Result<TokenStream, syn::Error> {
+    let base_ty = format_ident!("{}", iface.name);
 
-    let base_methods = if let Some(base) = iface.base()? {
-        gen_vtable_methods(base)?
+    let base_methods = if let Some(base) = iface.base() {
+        gen_vtable_methods(real, base, vtable_index)?
     } else {
-        quote!{}
+        quote! {}
     };
 
-    let methods = iface.methods
-        .map_err(|reason| format!("Interface {} cannot be implemented in rust \
-                                   because {} is not supported yet", iface.name, reason))?;
+    let ty_name = &real.ident;
+    let (impl_generics, ty_generics, where_clause) = real.generics.split_for_impl();
 
     let mut method_defs = Vec::new();
-    for method in methods {
-        let name = Ident::from(method.name);
+    for method in iface.methods()? {
         let ret = syn::parse_str::<Type>(method.ret)?;
 
         let mut params = Vec::new();
         let mut args = Vec::new();
         for param in method.params {
-            let name = Ident::from(param.name);
+            let name = format_ident!("{}", param.name);
             let ty = syn::parse_str::<Type>(param.ty)?;
 
-            params.push(quote!{#name : #ty,});
-            args.push(quote!{#name,});
+            params.push(quote! {#name : #ty,});
+            args.push(quote! {#name,});
         }
 
-        method_defs.push(quote!{
-            unsafe extern "system" fn #name (this: *const #base_ty, #(#params)*) -> #ret {
-                let lt = ();
-                recover_self(this, &lt).#name(#(#args)*)
+        let name = format_ident!("{}", method.name);
+        method_defs.push(quote! {
+            unsafe extern "system" fn #name #impl_generics (
+                this: *const #base_ty, #(#params)*
+            ) -> #ret #where_clause {
+                let this: &#ty_name #ty_generics =
+                    ::xpcom::reexports::transmute_from_vtable_ptr(&this, #vtable_index);
+                this.#name(#(#args)*)
             }
         });
     }
 
-    Ok(quote!{
+    Ok(quote! {
         #base_methods
         #(#method_defs)*
     })
@@ -381,26 +412,28 @@ fn gen_vtable_methods(iface: &Interface) -> Result<Tokens, Box<Error>> {
 
 /// Generates the VTable for a given base interface. This assumes that the
 /// implementations of each of the `extern "system"` methods are in scope.
-fn gen_inner_vtable(iface: &Interface) -> Result<Tokens, Box<Error>> {
-    let vtable_ty = Ident::from(format!("{}VTable", iface.name));
-
-    let methods = iface.methods
-        .map_err(|reason| format!("Interface {} cannot be implemented in rust \
-                                   because {} is not supported yet", iface.name, reason))?;
+fn gen_inner_vtable(real: &DeriveInput, iface: &Interface) -> Result<TokenStream, syn::Error> {
+    let vtable_ty = format_ident!("{}VTable", iface.name);
 
     // Generate the vtable for the base interface.
-    let base_vtable = if let Some(base) = iface.base()? {
-        let vt = gen_inner_vtable(base)?;
-        quote!{__base: #vt,}
+    let base_vtable = if let Some(base) = iface.base() {
+        let vt = gen_inner_vtable(real, base)?;
+        quote! {__base: #vt,}
     } else {
-        quote!{}
+        quote! {}
     };
 
     // Include each of the method definitions for this interface.
-    let vtable_init = methods.into_iter().map(|method| {
-        let name = Ident::from(method.name);
-        quote!{ #name : #name , }
-    }).collect::<Vec<_>>();
+    let (_, ty_generics, _) = real.generics.split_for_impl();
+    let turbofish = ty_generics.as_turbofish();
+    let vtable_init = iface
+        .methods()?
+        .into_iter()
+        .map(|method| {
+            let name = format_ident!("{}", method.name);
+            quote! { #name : #name #turbofish, }
+        })
+        .collect::<Vec<_>>();
 
     Ok(quote!(#vtable_ty {
         #base_vtable
@@ -408,40 +441,45 @@ fn gen_inner_vtable(iface: &Interface) -> Result<Tokens, Box<Error>> {
     }))
 }
 
-fn gen_root_vtable(name: &Ident, base: &Interface) -> Result<Tokens, Box<Error>> {
-    let field = Ident::from(format!("__base_{}", base.name));
-    let vtable_ty = Ident::from(format!("{}VTable", base.name));
-    let methods = gen_vtable_methods(base)?;
-    let value = gen_inner_vtable(base)?;
+fn gen_root_vtable(
+    real: &DeriveInput,
+    base: &Interface,
+    idx: usize,
+) -> Result<TokenStream, syn::Error> {
+    let field = format_ident!("__base_{}", base.name);
+    let vtable_ty = format_ident!("{}VTable", base.name);
+
+    let (impl_generics, ty_generics, where_clause) = real.generics.split_for_impl();
+    let turbofish = ty_generics.as_turbofish();
+
+    let methods = gen_vtable_methods(real, base, idx)?;
+    let vtable = gen_inner_vtable(real, base)?;
 
     // Define the `recover_self` method. This performs an offset calculation to
     // recover a pointer to the original struct from a pointer to the given
     // VTable field.
-    Ok(quote!{#field: {
-        // NOTE: The &'a () dummy lifetime parameter is useful as it easily
-        // allows the caller to limit the lifetime of the returned parameter
-        // to a local lifetime, preventing the calling of methods with
-        // receivers like `&'static self`.
-        #[inline]
-        unsafe fn recover_self<'a, T>(this: *const T, _: &'a ()) -> &'a #name {
-            // Calculate the offset of the field in our struct.
-            // XXX: Should we use the fact that our type is #[repr(C)] to avoid
-            // this?
-            let base = 0x1000;
-            let member = &(*(0x1000 as *const #name)).#field
-                as *const _ as usize;
-            let off = member - base;
-
-            // Offset the pointer by that offset.
-            &*((this as usize - off) as *const #name)
-        }
-
+    Ok(quote! {#field: {
         // The method implementations which will be used to build the vtable.
         #methods
 
-        // The actual VTable definition.
-        static VTABLE: #vtable_ty = #value;
-        &VTABLE
+        // The actual VTable definition. This is in a separate method in order
+        // to allow it to be generic.
+        #[inline]
+        fn get_vtable #impl_generics () -> &'static ::xpcom::reexports::VTableExtra<#vtable_ty> #where_clause {
+            &::xpcom::reexports::VTableExtra {
+                #[cfg(not(windows))]
+                offset: {
+                    // NOTE: workaround required to avoid depending on the
+                    // unstable const expression feature `const {}`.
+                    const OFFSET: isize = -((::std::mem::size_of::<usize>() * #idx) as isize);
+                    OFFSET
+                },
+                #[cfg(not(windows))]
+                typeinfo: 0 as *const _,
+                vtable: #vtable,
+            }
+        }
+        &get_vtable #turbofish ().vtable
     },})
 }
 
@@ -452,29 +490,23 @@ fn gen_root_vtable(name: &Ident, base: &Interface) -> Result<Tokens, Box<Error>>
 fn gen_casts(
     seen: &mut HashSet<&'static str>,
     iface: &Interface,
-    name: &Ident,
+    real: &DeriveInput,
     coerce_name: &Ident,
     vtable_field: &Ident,
-) -> Result<(Tokens, Tokens), Box<Error>> {
+) -> Result<(TokenStream, TokenStream), syn::Error> {
     if !seen.insert(iface.name) {
-        return Ok((quote!{}, quote!{}));
+        return Ok((quote! {}, quote! {}));
     }
 
     // Generate the cast implementations for the base interfaces.
-    let (base_qi, base_coerce) = if let Some(base) = iface.base()? {
-        gen_casts(
-            seen,
-            base,
-            name,
-            coerce_name,
-            vtable_field,
-        )?
+    let (base_qi, base_coerce) = if let Some(base) = iface.base() {
+        gen_casts(seen, base, real, coerce_name, vtable_field)?
     } else {
-        (quote!{}, quote!{})
+        (quote! {}, quote! {})
     };
 
     // Add the if statment to QueryInterface for the base class.
-    let base_name = Ident::from(iface.name);
+    let base_name = format_ident!("{}", iface.name);
 
     let qi = quote! {
         #base_qi
@@ -490,11 +522,13 @@ fn gen_casts(
     };
 
     // Add an implementation of the `*Coerce` trait for the base interface.
+    let name = &real.ident;
+    let (impl_generics, ty_generics, where_clause) = real.generics.split_for_impl();
     let coerce = quote! {
         #base_coerce
 
-        impl #coerce_name for ::xpcom::interfaces::#base_name {
-            fn coerce_from(v: &#name) -> &Self {
+        impl #impl_generics #coerce_name #ty_generics for ::xpcom::interfaces::#base_name #where_clause {
+            fn coerce_from(v: &#name #ty_generics) -> &Self {
                 unsafe {
                     // Get the address of the VTable field. This should be a
                     // pointer to a pointer to a vtable, which we can then cast
@@ -510,22 +544,79 @@ fn gen_casts(
     Ok((qi, coerce))
 }
 
-/// The root xpcom procedural macro definition.
-fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
-    if !init.generics.params.is_empty() || !init.generics.where_clause.is_none() {
-        return Err("Cannot #[derive(xpcom)] on a generic type, due to \
-                    rust limitations. It is not possible to instantiate \
-                    a static with a generic type parameter, meaning that \
-                    generic types cannot have their VTables instantiated \
-                    correctly.".into());
+fn check_generics(generics: &syn::Generics) -> Result<(), syn::Error> {
+    for param in &generics.params {
+        let tp = match param {
+            syn::GenericParam::Type(tp) => tp,
+            syn::GenericParam::Lifetime(lp) => bail!(
+                @(lp),
+                "Cannot #[derive(xpcom)] on types with lifetime parameters. \
+                Implementors of XPCOM interfaces must not contain non-'static \
+                lifetimes.",
+            ),
+            // XXX: Once const generics become stable, it may be as simple as
+            // removing this bail! to support them.
+            syn::GenericParam::Const(cp) => {
+                bail!(@(cp), "Cannot #[derive(xpcom)] on types with const generics.")
+            }
+        };
+
+        let mut static_lt = false;
+        for bound in &tp.bounds {
+            match bound {
+                syn::TypeParamBound::Lifetime(lt) if lt.ident == "static" => {
+                    static_lt = true;
+                    break;
+                }
+                _ => {}
+            }
+        }
+
+        if !static_lt {
+            bail!(
+                @(param),
+                "Every generic parameter for xpcom implementation must have a \
+                'static lifetime bound declared in the generics. Implicit \
+                lifetime bounds or lifetime bounds in where clauses are not \
+                detected by the macro and will be ignored. \
+                Implementors of XPCOM interfaces must not contain non-'static \
+                lifetimes.",
+            );
+        }
     }
+    Ok(())
+}
+
+/// The root xpcom procedural macro definition.
+fn xpcom(init: DeriveInput) -> Result<TokenStream, syn::Error> {
+    check_generics(&init.generics)?;
 
     let bases = get_bases(&init.attrs)?;
     if bases.is_empty() {
-        return Err("Types with #[derive(xpcom)] must implement at least one \
-                    interface. Interfaces can be implemented by adding the \
-                    #[xpimplements(nsIFoo, nsIBar)] attribute to the struct \
-                    declaration.".into());
+        bail!(
+            "Types with #[derive(xpcom)] must implement at least one \
+             interface. Interfaces can be implemented by adding the \
+             #[xpimplements(nsIFoo, nsIBar)] attribute to the struct \
+             declaration."
+        );
+    }
+
+    // Ensure that all our base interface methods have unique names.
+    let mut method_names = HashMap::new();
+    for base in &bases {
+        for method in base.methods()? {
+            if let Some(existing) = method_names.insert(method.name, base.name) {
+                bail!(
+                    "The method `{0}` is declared on both `{1}` and `{2}`,
+                     but a Rust type cannot implement two methods with the \
+                     same name. You can add the `[binaryname(Renamed{0})]` \
+                     XPIDL attribute to one of the declarations to rename it.",
+                    method.name,
+                    existing,
+                    base.name
+                );
+            }
+        }
     }
 
     // Determine what reference count type to use, and generate the real struct.
@@ -534,19 +625,19 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
 
     let name_init = &init.ident;
     let name = &real.ident;
-    let coerce_name = Ident::from(format!("{}Coerce", name.as_ref()));
+    let coerce_name = format_ident!("{}Coerce", name);
 
     // Generate a VTable for each of the base interfaces.
     let mut vtables = Vec::new();
-    for base in &bases {
-        vtables.push(gen_root_vtable(name, base)?);
+    for (idx, base) in bases.iter().enumerate() {
+        vtables.push(gen_root_vtable(&real, base, idx)?);
     }
 
     // Generate the field initializers for the final struct, moving each field
     // out of the original __init struct.
     let inits = get_fields(&init)?.iter().map(|field| {
         let id = &field.ident;
-        quote!{ #id : __init.#id, }
+        quote! { #id : __init.#id, }
     });
 
     let vis = &real.vis;
@@ -559,20 +650,37 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
         let (qi, coerce) = gen_casts(
             &mut seen,
             base,
-            name,
+            &real,
             &coerce_name,
-            &Ident::from(format!("__base_{}", base.name)),
+            &format_ident!("__base_{}", base.name),
         )?;
         qi_impl.push(qi);
         coerce_impl.push(coerce);
     }
 
+    let size_for_logs = if real.generics.params.is_empty() {
+        quote!(::std::mem::size_of::<Self>() as u32)
+    } else {
+        // Refcount logging requires all types with the same name to have the
+        // same size, and generics aren't taken into account when creating our
+        // name string, so we need to make sure that all possible instantiations
+        // report the same size. To do that, we fake a size based on the number
+        // of vtable pointers and the known refcount field.
+        let fake_size_npointers = bases.len() + 1;
+        quote!((::std::mem::size_of::<usize>() * #fake_size_npointers) as u32)
+    };
+
+    let (impl_generics, ty_generics, where_clause) = real.generics.split_for_impl();
+    let name_for_logs = quote!(
+        concat!(module_path!(), "::", stringify!(#name #ty_generics), "\0").as_ptr()
+            as *const ::xpcom::reexports::libc::c_char
+    );
     Ok(quote! {
         #real
 
-        impl #name {
+        impl #impl_generics #name #ty_generics #where_clause {
             /// This method is used for
-            fn allocate(__init: #name_init) -> ::xpcom::RefPtr<Self> {
+            fn allocate(__init: #name_init #ty_generics) -> ::xpcom::RefPtr<Self> {
                 #[allow(unused_imports)]
                 use ::xpcom::*;
                 #[allow(unused_imports)]
@@ -582,6 +690,10 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
                     libc, nsACString, nsAString, nsCString, nsString, nsresult
                 };
 
+                // Helper for asserting that for all instantiations, this
+                // object has the 'static lifetime.
+                fn xpcom_types_must_be_static<T: 'static>(t: &T) {}
+
                 unsafe {
                     // NOTE: This is split into multiple lines to make the
                     // output more readable.
@@ -591,6 +703,7 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
                         #(#inits)*
                     };
                     let boxed = ::std::boxed::Box::new(value);
+                    xpcom_types_must_be_static(&*boxed);
                     let raw = ::std::boxed::Box::into_raw(boxed);
                     ::xpcom::RefPtr::from_raw(raw).unwrap()
                 }
@@ -598,14 +711,27 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
 
             /// Automatically generated implementation of AddRef for nsISupports.
             #vis unsafe fn AddRef(&self) -> ::xpcom::interfaces::nsrefcnt {
-                self.__refcnt.inc()
+                let new = self.__refcnt.inc();
+                ::xpcom::trace_refcnt::NS_LogAddRef(
+                    self as *const _ as *mut ::xpcom::reexports::libc::c_void,
+                    new,
+                    #name_for_logs,
+                    #size_for_logs,
+                );
+                new
             }
 
             /// Automatically generated implementation of Release for nsISupports.
             #vis unsafe fn Release(&self) -> ::xpcom::interfaces::nsrefcnt {
                 let new = self.__refcnt.dec();
+                ::xpcom::trace_refcnt::NS_LogRelease(
+                    self as *const _ as *mut ::xpcom::reexports::libc::c_void,
+                    new,
+                    #name_for_logs,
+                    #size_for_logs,
+                );
                 if new == 0 {
-                    // XXX: dealloc
+                    // dealloc
                     ::std::boxed::Box::from_raw(self as *const Self as *mut Self);
                 }
                 new
@@ -630,12 +756,12 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
             /// Perform a QueryInterface call on this object, attempting to
             /// dynamically cast it to the requested interface type. Returns
             /// Some(RefPtr<T>) if the cast succeeded, and None otherwise.
-            #vis fn query_interface<T: ::xpcom::XpCom>(&self) ->
-                ::std::option::Option<::xpcom::RefPtr<T>>
+            #vis fn query_interface<XPCOM_InterfaceType: ::xpcom::XpCom>(&self)
+                -> ::std::option::Option<::xpcom::RefPtr<XPCOM_InterfaceType>>
             {
-                let mut ga = ::xpcom::GetterAddrefs::<T>::new();
+                let mut ga = ::xpcom::GetterAddrefs::<XPCOM_InterfaceType>::new();
                 unsafe {
-                    if self.QueryInterface(&T::IID, ga.void_ptr()).succeeded() {
+                    if self.QueryInterface(&XPCOM_InterfaceType::IID, ga.void_ptr()).succeeded() {
                         ga.refptr()
                     } else {
                         None
@@ -645,8 +771,8 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
 
             /// Coerce this type safely to any of the interfaces which it
             /// implements without `AddRef`ing it.
-            #vis fn coerce<T: #coerce_name>(&self) -> &T {
-                T::coerce_from(self)
+            #vis fn coerce<XPCOM_InterfaceType: #coerce_name #ty_generics>(&self) -> &XPCOM_InterfaceType {
+                XPCOM_InterfaceType::coerce_from(self)
             }
         }
 
@@ -658,15 +784,15 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
         /// rather acts as a trait bound and implementation for the `coerce`
         /// methods.
         #[doc(hidden)]
-        #vis trait #coerce_name {
+        #vis trait #coerce_name #impl_generics #where_clause {
             /// Convert a value of the `#[derive(xpcom)]` type into the
             /// implementing interface type.
-            fn coerce_from(v: &#name) -> &Self;
+            fn coerce_from(v: &#name #ty_generics) -> &Self;
         }
 
         #(#coerce_impl)*
 
-        unsafe impl ::xpcom::RefCounted for #name {
+        unsafe impl #impl_generics ::xpcom::RefCounted for #name #ty_generics #where_clause {
             unsafe fn addref(&self) {
                 self.AddRef();
             }
@@ -679,7 +805,10 @@ fn xpcom(init: DeriveInput) -> Result<Tokens, Box<Error>> {
 }
 
 #[proc_macro_derive(xpcom, attributes(xpimplements, refcnt))]
-pub fn xpcom_internal(input: TokenStream) -> TokenStream {
-    xpcom(parse(input).expect("Invalid derive input"))
-        .expect("#[derive(xpcom)] failed").into()
+pub fn xpcom_internal(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
+    let input = parse_macro_input!(input as DeriveInput);
+    match xpcom(input) {
+        Ok(ts) => ts.into(),
+        Err(err) => err.to_compile_error().into(),
+    }
 }
