@@ -19,16 +19,15 @@
 
 using namespace mozilla;
 
-class Task final : public nsIRunnable {
+class TestTask final : public Runnable {
  public:
-  NS_DECL_THREADSAFE_ISUPPORTS
-
-  Task(int i, Atomic<int>& aCounter) : mIndex(i), mCounter(aCounter) {}
+  TestTask(int i, Atomic<int>& aCounter)
+      : Runnable("TestThreadPool::Task"), mIndex(i), mCounter(aCounter) {}
 
   NS_IMETHOD Run() override {
     printf("###(%d) running from thread: %p\n", mIndex,
            (void*)PR_GetCurrentThread());
-    int r = (int)((float)rand() * 200 / RAND_MAX);
+    int r = (int)((float)rand() * 200 / float(RAND_MAX));
     PR_Sleep(PR_MillisecondsToInterval(r));
     printf("###(%d) exiting from thread: %p\n", mIndex,
            (void*)PR_GetCurrentThread());
@@ -37,12 +36,11 @@ class Task final : public nsIRunnable {
   }
 
  private:
-  ~Task() {}
+  ~TestTask() = default;
 
   int mIndex;
   Atomic<int>& mCounter;
 };
-NS_IMPL_ISUPPORTS(Task, nsIRunnable)
 
 TEST(ThreadPool, Main)
 {
@@ -51,7 +49,7 @@ TEST(ThreadPool, Main)
   Atomic<int> count(0);
 
   for (int i = 0; i < 100; ++i) {
-    nsCOMPtr<nsIRunnable> task = new Task(i, count);
+    nsCOMPtr<nsIRunnable> task = new TestTask(i, count);
     EXPECT_TRUE(task);
 
     pool->Dispatch(task, NS_DISPATCH_NORMAL);
@@ -112,7 +110,7 @@ TEST(ThreadPool, Parallelism)
   // We should wake up the idle thread and spawn a new thread so these 2 events
   // can run in parallel. We will time out if r1 and r2 run in sequence for r1
   // won't finish until r2 finishes.
-  Monitor mon("ThreadPool::Parallelism");
+  Monitor mon MOZ_UNANNOTATED("ThreadPool::Parallelism");
   bool done = false;
   nsCOMPtr<nsIRunnable> r1 = new Runnable1(mon, done);
   nsCOMPtr<nsIRunnable> r2 = new Runnable2(mon, done);
@@ -128,14 +126,14 @@ TEST(ThreadPool, ShutdownWithTimeout)
 
   Atomic<int> allThreadsCount(0);
   for (int i = 0; i < 4; ++i) {
-    nsCOMPtr<nsIRunnable> task = new Task(i, allThreadsCount);
+    nsCOMPtr<nsIRunnable> task = new TestTask(i, allThreadsCount);
     EXPECT_TRUE(task);
 
     pool->Dispatch(task, NS_DISPATCH_NORMAL);
   }
 
-  // Wait for a max of 300 ms. All threads should be done by then.
-  pool->ShutdownWithTimeout(300);
+  // Wait for a max of 350 ms. All threads should be done by then.
+  pool->ShutdownWithTimeout(350);
   EXPECT_EQ(allThreadsCount, 4);
 
   Atomic<int> infiniteLoopCount(0);
@@ -143,7 +141,7 @@ TEST(ThreadPool, ShutdownWithTimeout)
   Atomic<bool> shutdownAck(false);
   pool = new nsThreadPool();
   for (int i = 0; i < 3; ++i) {
-    nsCOMPtr<nsIRunnable> task = new Task(i, infiniteLoopCount);
+    nsCOMPtr<nsIRunnable> task = new TestTask(i, infiniteLoopCount);
     EXPECT_TRUE(task);
 
     pool->Dispatch(task, NS_DISPATCH_NORMAL);
@@ -176,7 +174,7 @@ TEST(ThreadPool, ShutdownWithTimeoutThenSleep)
   nsCOMPtr<nsIThreadPool> pool = new nsThreadPool();
 
   for (int i = 0; i < 3; ++i) {
-    nsCOMPtr<nsIRunnable> task = new Task(i, count);
+    nsCOMPtr<nsIRunnable> task = new TestTask(i, count);
     EXPECT_TRUE(task);
 
     pool->Dispatch(task, NS_DISPATCH_NORMAL);
@@ -195,9 +193,9 @@ TEST(ThreadPool, ShutdownWithTimeoutThenSleep)
           }),
       NS_DISPATCH_NORMAL);
 
-  // Wait for a max of 300 ms. The thread should still be sleeping, and will
+  // Wait for a max of 350 ms. The thread should still be sleeping, and will
   // be leaked.
-  pool->ShutdownWithTimeout(300);
+  pool->ShutdownWithTimeout(350);
   // We can't be exact here; the thread we're running on might have gotten
   // suspended and the sleeping thread, above, might have finished.
   EXPECT_GE(count, 3);
