@@ -10,10 +10,11 @@
 #include "mozilla/BlockingResourceBase.h"
 #include "mozilla/PlatformConditionVariable.h"
 #include "mozilla/Mutex.h"
+#include "mozilla/TimeStamp.h"
 
-#ifdef MOZILLA_INTERNAL_API
-#  include "GeckoProfiler.h"
-#endif  // MOZILLA_INTERNAL_API
+#if defined(MOZILLA_INTERNAL_API) && !defined(DEBUG)
+#  include "mozilla/ProfilerThreadSleep.h"
+#endif  // defined( MOZILLA_INTERNAL_API) && !defined(DEBUG)
 
 namespace mozilla {
 
@@ -43,7 +44,7 @@ class OffTheBooksCondVar : BlockingResourceBase {
    * ~OffTheBooksCondVar
    * Clean up after this OffTheBooksCondVar, but NOT its associated Mutex.
    **/
-  ~OffTheBooksCondVar() {}
+  ~OffTheBooksCondVar() = default;
 
   /**
    * Wait
@@ -73,38 +74,34 @@ class OffTheBooksCondVar : BlockingResourceBase {
    * Notify
    * @see prcvar.h
    **/
-  nsresult Notify() {
-    mImpl.notify_one();
-    return NS_OK;
-  }
+  void Notify() { mImpl.notify_one(); }
 
   /**
    * NotifyAll
    * @see prcvar.h
    **/
-  nsresult NotifyAll() {
-    mImpl.notify_all();
-    return NS_OK;
-  }
+  void NotifyAll() { mImpl.notify_all(); }
 
 #ifdef DEBUG
   /**
    * AssertCurrentThreadOwnsMutex
    * @see Mutex::AssertCurrentThreadOwns
    **/
-  void AssertCurrentThreadOwnsMutex() { mLock->AssertCurrentThreadOwns(); }
+  void AssertCurrentThreadOwnsMutex() const ASSERT_CAPABILITY(mLock) {
+    mLock->AssertCurrentThreadOwns();
+  }
 
   /**
    * AssertNotCurrentThreadOwnsMutex
    * @see Mutex::AssertNotCurrentThreadOwns
    **/
-  void AssertNotCurrentThreadOwnsMutex() {
+  void AssertNotCurrentThreadOwnsMutex() const ASSERT_CAPABILITY(!mLock) {
     mLock->AssertNotCurrentThreadOwns();
   }
 
 #else
-  void AssertCurrentThreadOwnsMutex() {}
-  void AssertNotCurrentThreadOwnsMutex() {}
+  void AssertCurrentThreadOwnsMutex() const ASSERT_CAPABILITY(mLock) {}
+  void AssertNotCurrentThreadOwnsMutex() const ASSERT_CAPABILITY(!mLock) {}
 
 #endif  // ifdef DEBUG
 
@@ -129,7 +126,7 @@ class CondVar : public OffTheBooksCondVar {
     MOZ_COUNT_CTOR(CondVar);
   }
 
-  ~CondVar() { MOZ_COUNT_DTOR(CondVar); }
+  MOZ_COUNTED_DTOR(CondVar)
 
  private:
   CondVar();
